@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: conf.c 10391 2010-03-17 19:23:03Z charles $
+ * $Id: conf.c 12223 2011-03-24 21:49:42Z jordan $
  *
- * Copyright (c) 2005-2008 Transmission authors and contributors
+ * Copyright (c) Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -24,7 +24,7 @@
 
 #include <errno.h>
 #include <stdio.h>
-#include <stdlib.h>
+#include <stdlib.h> /* strtol() */
 #include <string.h>
 
 #include <unistd.h>
@@ -40,15 +40,15 @@
 #include "tr-prefs.h"
 #include "util.h"
 
-#define MY_NAME "transmission"
+#define MY_CONFIG_NAME "transmission"
+#define MY_READABLE_NAME "transmission-gtk"
 
 static char * gl_confdir = NULL;
 static char * gl_lockpath = NULL;
 
 /* errstr may be NULL, this might be called before GTK is initialized */
 gboolean
-cf_init( const char   * configDir,
-         char        ** errstr )
+cf_init( const char * configDir, char ** errstr )
 {
     if( errstr != NULL )
         *errstr = NULL;
@@ -73,9 +73,7 @@ cf_init( const char   * configDir,
 
 /* errstr may be NULL, this might be called before GTK is initialized */
 static gboolean
-lockfile( const char             * filename,
-          gtr_lockfile_state_t   * tr_state,
-          char                  ** errstr )
+lockfile( const char * filename, gtr_lockfile_state_t * tr_state, char ** errstr )
 {
     const gtr_lockfile_state_t state = gtr_lockfile( filename );
     const gboolean success = state == GTR_LOCKFILE_SUCCESS;
@@ -187,7 +185,6 @@ tr_prefs_init_defaults( tr_benc * d )
     tr_bencDictAddInt( d, PREF_KEY_MAIN_WINDOW_WIDTH, 300 );
     tr_bencDictAddInt( d, PREF_KEY_MAIN_WINDOW_X, 50 );
     tr_bencDictAddInt( d, PREF_KEY_MAIN_WINDOW_Y, 50 );
-    tr_bencDictAddStr( d, PREF_KEY_MAIN_WINDOW_LAYOUT_ORDER, "menu,toolbar,filter,list,statusbar" );
 
     str = NULL;
 #if GLIB_CHECK_VERSION( 2, 14, 0 )
@@ -196,9 +193,6 @@ tr_prefs_init_defaults( tr_benc * d )
     if( !str ) str = tr_getDefaultDownloadDir( );
     tr_bencDictAddStr( d, TR_PREFS_KEY_DOWNLOAD_DIR, str );
 
-    tr_bencDictAddBool( d, PREF_KEY_ASKQUIT, TRUE );
-
-    tr_bencDictAddStr( d, PREF_KEY_FILTER_MODE, "show-all" );
     tr_bencDictAddStr( d, PREF_KEY_SORT_MODE, "sort-by-name" );
     tr_bencDictAddBool( d, PREF_KEY_SORT_REVERSED, FALSE );
     tr_bencDictAddBool( d, PREF_KEY_COMPACT_VIEW, FALSE );
@@ -221,7 +215,7 @@ getPrefs( void )
     {
         tr_bencInitDict( &settings, 0 );
         tr_prefs_init_defaults( &settings );
-        tr_sessionLoadSettings( &settings, gl_confdir, MY_NAME );
+        tr_sessionLoadSettings( &settings, gl_confdir, MY_CONFIG_NAME );
         loaded = TRUE;
     }
 
@@ -233,13 +227,13 @@ getPrefs( void )
 ***/
 
 tr_benc*
-pref_get_all( void )
+gtr_pref_get_all( void )
 {
     return getPrefs( );
 }
 
 int64_t
-pref_int_get( const char * key )
+gtr_pref_int_get( const char * key )
 {
     int64_t i = 0;
 
@@ -248,14 +242,13 @@ pref_int_get( const char * key )
 }
 
 void
-pref_int_set( const char * key,
-              int64_t      value )
+gtr_pref_int_set( const char * key, int64_t value )
 {
     tr_bencDictAddInt( getPrefs( ), key, value );
 }
 
 double
-pref_double_get( const char * key )
+gtr_pref_double_get( const char * key )
 {
     double d = 0.0;
     tr_bencDictFindReal( getPrefs( ), key, &d );
@@ -263,8 +256,7 @@ pref_double_get( const char * key )
 }
 
 void
-pref_double_set( const char * key,
-                 double       value )
+gtr_pref_double_set( const char * key, double value )
 {
     tr_bencDictAddReal( getPrefs( ), key, value );
 }
@@ -274,33 +266,15 @@ pref_double_set( const char * key,
 ***/
 
 gboolean
-pref_flag_get( const char * key )
+gtr_pref_flag_get( const char * key )
 {
-    tr_bool boolVal;
+    bool boolVal;
     tr_bencDictFindBool( getPrefs( ), key, &boolVal );
     return boolVal != 0;
 }
 
-gboolean
-pref_flag_eval( pref_flag_t  val,
-                const char * key )
-{
-    switch( val )
-    {
-        case PREF_FLAG_TRUE:
-            return TRUE;
-
-        case PREF_FLAG_FALSE:
-            return FALSE;
-
-        default:
-            return pref_flag_get( key );
-    }
-}
-
 void
-pref_flag_set( const char * key,
-               gboolean     value )
+gtr_pref_flag_set( const char * key, gboolean value )
 {
     tr_bencDictAddBool( getPrefs( ), key, value );
 }
@@ -310,7 +284,7 @@ pref_flag_set( const char * key,
 ***/
 
 const char*
-pref_string_get( const char * key )
+gtr_pref_string_get( const char * key )
 {
     const char * str = NULL;
     tr_bencDictFindStr( getPrefs( ), key, &str );
@@ -318,7 +292,7 @@ pref_string_get( const char * key )
 }
 
 void
-pref_string_set( const char * key, const char * value )
+gtr_pref_string_set( const char * key, const char * value )
 {
     tr_bencDictAddStr( getPrefs( ), key, value );
 }
@@ -328,7 +302,7 @@ pref_string_set( const char * key, const char * value )
 ***/
 
 void
-pref_save( tr_session * session )
+gtr_pref_save( tr_session * session )
 {
     tr_sessionSaveSettings( session, gl_confdir, getPrefs( ) );
 }
@@ -337,25 +311,6 @@ pref_save( tr_session * session )
 ****
 ***/
 
-#if !GLIB_CHECK_VERSION( 2, 8, 0 )
-static void
-tr_file_set_contents( const char *   filename,
-                      const void *   out,
-                      size_t         len,
-                      GError* unused UNUSED )
-{
-    FILE * fp = fopen( filename, "wb+" );
-
-    if( fp != NULL )
-    {
-        fwrite( out, 1, len, fp );
-        fclose( fp );
-    }
-}
-
- #define g_file_set_contents tr_file_set_contents
-#endif
-
 static char*
 getCompat090PrefsFilename( void )
 {
@@ -371,8 +326,7 @@ getCompat121PrefsFilename( void )
 }
 
 static void
-translate_keyfile_to_json( const char * old_file,
-                           const char * new_file )
+translate_keyfile_to_json( const char * old_file, const char * new_file )
 {
     tr_benc    dict;
     GKeyFile * keyfile;
