@@ -1,13 +1,13 @@
 /*
- * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: hig.c 8097 2009-03-28 16:47:01Z charles $
+ * $Id: hig.c 11709 2011-01-19 13:48:47Z jordan $
  */
 
 #include <gtk/gtk.h>
@@ -91,6 +91,21 @@ hig_workarea_add_wide_control( GtkWidget * t,
     ++ * row;
 }
 
+void
+hig_workarea_add_wide_tall_control( GtkWidget * t,
+                                    int *       row,
+                                    GtkWidget * w )
+{
+    GtkWidget * r = rowNew( w );
+
+    gtk_table_attach( GTK_TABLE( t ), r, 0, 2, *row, *row + 1,
+                      GTK_EXPAND | GTK_SHRINK | GTK_FILL,
+                      GTK_EXPAND | GTK_SHRINK | GTK_FILL,
+                      0, 0 );
+
+    ++ * row;
+}
+
 GtkWidget *
 hig_workarea_add_wide_checkbutton( GtkWidget *  t,
                                    int *        row,
@@ -125,7 +140,22 @@ hig_workarea_add_label( GtkWidget *  t,
     return l;
 }
 
-void
+static void
+hig_workarea_add_tall_control( GtkWidget * t,
+                               int         row,
+                               GtkWidget * control )
+{
+    if( GTK_IS_MISC( control ) )
+        gtk_misc_set_alignment( GTK_MISC( control ), 0.0f, 0.5f );
+
+    gtk_table_attach( GTK_TABLE( t ), control,
+                      1, 2, row, row + 1,
+                      GTK_EXPAND | GTK_SHRINK | GTK_FILL,
+                      GTK_EXPAND | GTK_SHRINK | GTK_FILL,
+                      0, 0 );
+}
+
+static void
 hig_workarea_add_control( GtkWidget * t,
                           int         row,
                           GtkWidget * control )
@@ -166,19 +196,33 @@ hig_workarea_add_row( GtkWidget *  t,
     return l;
 }
 
+GtkWidget*
+hig_workarea_add_tall_row( GtkWidget *  table,
+                           int *        row,
+                           const char * mnemonic_string,
+                           GtkWidget *  control,
+                           GtkWidget *  mnemonic )
+{
+    GtkWidget * l = gtk_label_new_with_mnemonic( mnemonic_string );
+    GtkWidget * h = gtk_hbox_new( FALSE, 0 );
+    GtkWidget * v = gtk_vbox_new( FALSE, 0 );
+    gtk_box_pack_start( GTK_BOX( h ), l, FALSE, FALSE, 0 );
+    gtk_box_pack_start( GTK_BOX( v ), h, FALSE, FALSE, GUI_PAD_SMALL );
+
+    hig_workarea_add_label_w( table, *row, v );
+    hig_workarea_add_tall_control( table, *row, control );
+
+    if( GTK_IS_LABEL( l ) )
+        gtk_label_set_mnemonic_widget( GTK_LABEL( l ),
+                                       mnemonic ? mnemonic : control );
+
+    ++ * row;
+    return l;
+}
+
 void
 hig_workarea_finish( GtkWidget * t,
                      int *       row )
 {
     gtk_table_resize( GTK_TABLE( t ), *row, 2 );
 }
-
-void
-hig_message_dialog_set_text( GtkMessageDialog * dialog,
-                             const char *       primary,
-                             const char *       secondary )
-{
-    gtk_message_dialog_set_markup ( dialog, primary );
-    gtk_message_dialog_format_secondary_text ( dialog, "%s", secondary );
-}
-
