@@ -1,17 +1,17 @@
 /*
- * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: hig.h 7658 2009-01-10 23:09:07Z charles $
+ * $Id: hig.h 11709 2011-01-19 13:48:47Z jordan $
  */
 
-#ifndef __HIG_H__
-#define __HIG_H__
+#ifndef GTR_HIG_H
+#define GTR_HIG_H
 
 #include <gtk/gtk.h>
 
@@ -33,6 +33,10 @@ void       hig_workarea_add_section_title( GtkWidget *  table,
                                            int *        row,
                                            const char * section_title );
 
+void       hig_workarea_add_wide_tall_control( GtkWidget * table,
+                                               int *       row,
+                                               GtkWidget * w );
+
 void       hig_workarea_add_wide_control( GtkWidget * table,
                                           int *       row,
                                           GtkWidget * w );
@@ -50,9 +54,11 @@ void       hig_workarea_add_label_w( GtkWidget * table,
                                      int         row,
                                      GtkWidget * label_widget );
 
-void       hig_workarea_add_control( GtkWidget * table,
-                                     int         row,
-                                     GtkWidget * control );
+GtkWidget* hig_workarea_add_tall_row( GtkWidget *  table,
+                                      int *        row,
+                                      const char * mnemonic_string,
+                                      GtkWidget *  control,
+                                      GtkWidget *  mnemonic_or_null_for_control );
 
 GtkWidget* hig_workarea_add_row( GtkWidget *  table,
                                  int *        row,
@@ -69,10 +75,6 @@ void       hig_workarea_add_row_w( GtkWidget * table,
 void       hig_workarea_finish( GtkWidget * table,
                                 int *       row );
 
-void       hig_message_dialog_set_text( GtkMessageDialog * dialog,
-                                        const char *       primary,
-                                        const char *       secondary );
-
 enum
 {
     GUI_PAD_SMALL = 3,
@@ -81,4 +83,4 @@ enum
     GUI_PAD_LARGE = 12
 };
 
-#endif /* __HIG_H__ */
+#endif /* GTR_HIG_H */
