@@ -1,17 +1,17 @@
 /*
- * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: torrent-cell-renderer.h 7658 2009-01-10 23:09:07Z charles $
+ * $Id: torrent-cell-renderer.h 11709 2011-01-19 13:48:47Z jordan $
  */
 
-#ifndef TORRENT_CELL_RENDERER_H
-#define TORRENT_CELL_RENDERER_H
+#ifndef GTR_TORRENT_CELL_RENDERER_H
+#define GTR_TORRENT_CELL_RENDERER_H
 
 #include <glib-object.h>
 #include <gtk/gtk.h>
@@ -42,4 +42,4 @@ GType             torrent_cell_renderer_get_type( void );
 
 GtkCellRenderer * torrent_cell_renderer_new( void );
 
-#endif
+#endif /* GTR_TORRENT_CELL_RENDERER_H */
