@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: tr-window.c 11465 2010-12-05 18:32:25Z charles $
+ * $Id: tr-window.c 12412 2011-05-02 17:58:27Z jordan $
  *
- * Copyright (c) 2005-2008 Transmission authors and contributors
+ * Copyright (c) Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,14 +27,8 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
-#if !GTK_CHECK_VERSION( 2,16,0 )
- /* FIXME: when 2.16 has been out long enough, it would be really nice to
-  * get rid of this libsexy usage because of its makefile strangeness */
- #define USE_SEXY
- #include "sexy-icon-entry.h"
-#endif
-
 #include <libtransmission/transmission.h>
+#include <libtransmission/utils.h> /* tr_formatter_speed_KBps() */
 
 #include "actions.h"
 #include "conf.h"
@@ -42,22 +36,9 @@
 #include "hig.h"
 #include "torrent-cell-renderer.h"
 #include "tr-prefs.h"
-#include "tr-torrent.h"
 #include "tr-window.h"
 #include "util.h"
 
-#if !GTK_CHECK_VERSION( 2, 8, 0 )
-static void
-gtk_tree_view_column_queue_resize( GtkTreeViewColumn * column ) /* yuck */
-{
-    const int spacing = gtk_tree_view_column_get_spacing( column );
-
-    gtk_tree_view_column_set_spacing( column, spacing + 1 );
-    gtk_tree_view_column_set_spacing( column, spacing );
-}
-
-#endif
-
 typedef struct
 {
     GtkWidget *           speedlimit_on_item[2];
@@ -86,12 +67,18 @@ typedef struct
 }
 PrivateData;
 
-#define PRIVATE_DATA_KEY "private-data"
+static GQuark
+get_private_data_key( void )
+{
+    static GQuark q = 0;
+    if( !q ) q = g_quark_from_static_string( "private-data" );
+    return q;
+}
 
 static PrivateData*
 get_private_data( TrWindow * w )
 {
-    return g_object_get_data ( G_OBJECT( w ), PRIVATE_DATA_KEY );
+    return g_object_get_qdata ( G_OBJECT( w ), get_private_data_key( ) );
 }
 
 /***
@@ -102,7 +89,7 @@ static void
 on_popup_menu( GtkWidget * self UNUSED,
                GdkEventButton * event )
 {
-    GtkWidget * menu = action_get_widget ( "/main-window-popup" );
+    GtkWidget * menu = gtr_action_get_widget ( "/main-window-popup" );
 
     gtk_menu_popup ( GTK_MENU( menu ), NULL, NULL, NULL, NULL,
                     ( event ? event->button : 0 ),
@@ -115,7 +102,7 @@ view_row_activated( GtkTreeView       * tree_view UNUSED,
                     GtkTreeViewColumn * column    UNUSED,
                     gpointer            user_data UNUSED )
 {
-    action_activate( "show-torrent-properties" );
+    gtr_action_activate( "show-torrent-properties" );
 }
 
 static GtkWidget*
@@ -140,7 +127,7 @@ makeview( PrivateData * p )
 
     p->renderer = r = torrent_cell_renderer_new( );
     gtk_tree_view_column_pack_start( col, r, FALSE );
-    gtk_tree_view_column_add_attribute( col, r, "torrent", MC_TORRENT_RAW );
+    gtk_tree_view_column_add_attribute( col, r, "torrent", MC_TORRENT );
     gtk_tree_view_column_add_attribute( col, r, "piece-upload-speed", MC_SPEED_UP );
     gtk_tree_view_column_add_attribute( col, r, "piece-download-speed", MC_SPEED_DOWN );
 
@@ -164,6 +151,7 @@ makeview( PrivateData * p )
 
 
     gtk_tree_view_set_model( GTK_TREE_VIEW( view ), p->filter_model );
+    g_object_unref( p->filter_model );
 
     return view;
 }
@@ -179,34 +167,34 @@ prefsChanged( TrCore * core UNUSED,
 
     if( !strcmp( key, PREF_KEY_COMPACT_VIEW ) )
     {
-        g_object_set( p->renderer, "compact", pref_flag_get( key ), NULL );
+        g_object_set( p->renderer, "compact", gtr_pref_flag_get( key ), NULL );
         /* since the cell size has changed, we need gtktreeview to revalidate
-         * its fixed-height mode values.  Unfortunately there's not an API call
+         * its fixed-height mode values. Unfortunately there's not an API call
          * for that, but it *does* revalidate when it thinks the style's been tweaked */
         g_signal_emit_by_name( p->view, "style-set", NULL, NULL );
     }
     else if( !strcmp( key, PREF_KEY_STATUSBAR ) )
     {
-        const gboolean isEnabled = pref_flag_get( key );
+        const gboolean isEnabled = gtr_pref_flag_get( key );
         g_object_set( p->status, "visible", isEnabled, NULL );
     }
     else if( !strcmp( key, PREF_KEY_FILTERBAR ) )
     {
-        const gboolean isEnabled = pref_flag_get( key );
+        const gboolean isEnabled = gtr_pref_flag_get( key );
         g_object_set( p->filter, "visible", isEnabled, NULL );
     }
     else if( !strcmp( key, PREF_KEY_TOOLBAR ) )
     {
-        const gboolean isEnabled = pref_flag_get( key );
+        const gboolean isEnabled = gtr_pref_flag_get( key );
         g_object_set( p->toolbar, "visible", isEnabled, NULL );
     }
     else if( !strcmp( key, PREF_KEY_STATUSBAR_STATS ) )
     {
-        tr_window_update( (TrWindow*)wind );
+        gtr_window_refresh( (TrWindow*)wind );
     }
     else if( !strcmp( key, TR_PREFS_KEY_ALT_SPEED_ENABLED ) ||
-             !strcmp( key, TR_PREFS_KEY_ALT_SPEED_UP ) ||
-             !strcmp( key, TR_PREFS_KEY_ALT_SPEED_DOWN ) )
+             !strcmp( key, TR_PREFS_KEY_ALT_SPEED_UP_KBps ) ||
+             !strcmp( key, TR_PREFS_KEY_ALT_SPEED_DOWN_KBps ) )
     {
         syncAltSpeedButton( p );
     }
@@ -251,7 +239,7 @@ status_menu_toggled_cb( GtkCheckMenuItem * menu_item,
         PrivateData * p = vprivate;
         const char *  val = g_object_get_data( G_OBJECT(
                                                    menu_item ), STATS_MODE );
-        tr_core_set_pref( p->core, PREF_KEY_STATUSBAR_STATS, val );
+        gtr_core_set_pref( p->core, PREF_KEY_STATUSBAR_STATS, val );
     }
 }
 
@@ -262,14 +250,14 @@ syncAltSpeedButton( PrivateData * p )
     char d[32];
     char * str;
     const char * fmt;
-    const gboolean b = pref_flag_get( TR_PREFS_KEY_ALT_SPEED_ENABLED );
+    const gboolean b = gtr_pref_flag_get( TR_PREFS_KEY_ALT_SPEED_ENABLED );
     const char * stock = b ? "alt-speed-on" : "alt-speed-off";
     GtkWidget * w = p->alt_speed_button;
 
-    tr_strlspeed( u, pref_int_get( TR_PREFS_KEY_ALT_SPEED_UP ), sizeof( u ) );
-    tr_strlspeed( d, pref_int_get( TR_PREFS_KEY_ALT_SPEED_DOWN ), sizeof( d ) );
-    fmt = b ? _( "Click to disable Temporary Speed Limits\n(%1$s down, %2$s up)" )
-            : _( "Click to enable Temporary Speed Limits\n(%1$s down, %2$s up)" );
+    tr_formatter_speed_KBps( u, gtr_pref_int_get( TR_PREFS_KEY_ALT_SPEED_UP_KBps ), sizeof( u ) );
+    tr_formatter_speed_KBps( d, gtr_pref_int_get( TR_PREFS_KEY_ALT_SPEED_DOWN_KBps ), sizeof( d ) );
+    fmt = b ? _( "Click to disable Alternative Speed Limits\n(%1$s down, %2$s up)" )
+            : _( "Click to enable Alternative Speed Limits\n(%1$s down, %2$s up)" );
     str = g_strdup_printf( fmt, d, u );
 
     gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( w ), b );
@@ -285,7 +273,7 @@ alt_speed_toggled_cb( GtkToggleButton * button, gpointer vprivate )
 {
     PrivateData * p = vprivate;
     const gboolean b = gtk_toggle_button_get_active( button );
-    tr_core_set_pref_bool( p->core, TR_PREFS_KEY_ALT_SPEED_ENABLED,  b );
+    gtr_core_set_pref_bool( p->core, TR_PREFS_KEY_ALT_SPEED_ENABLED,  b );
 }
 
 /***
@@ -304,7 +292,7 @@ findMaxAnnounceTime( GtkTreeModel *      model,
     const tr_stat * torStat;
     time_t *        maxTime = gmaxTime;
 
-    gtk_tree_model_get( model, iter, MC_TORRENT_RAW, &tor, -1 );
+    gtk_tree_model_get( model, iter, MC_TORRENT, &tor, -1 );
     torStat = tr_torrentStatCached( tor );
     *maxTime = MAX( *maxTime, torStat->manualAnnounceTime );
 }
@@ -348,14 +336,14 @@ static gboolean
 onAltSpeedToggledIdle( gpointer vp )
 {
     PrivateData * p = vp;
-    gboolean b = tr_sessionUsesAltSpeed( tr_core_session( p->core ) );
-    tr_core_set_pref_bool( p->core, TR_PREFS_KEY_ALT_SPEED_ENABLED, b );
+    gboolean b = tr_sessionUsesAltSpeed( gtr_core_session( p->core ) );
+    gtr_core_set_pref_bool( p->core, TR_PREFS_KEY_ALT_SPEED_ENABLED, b );
 
     return FALSE;
 }
 
 static void
-onAltSpeedToggled( tr_session * s UNUSED, tr_bool isEnabled UNUSED, tr_bool byUser UNUSED, void * p )
+onAltSpeedToggled( tr_session * s UNUSED, bool isEnabled UNUSED, bool byUser UNUSED, void * p )
 {
     gtr_idle_add( onAltSpeedToggledIdle, p );
 }
@@ -379,7 +367,7 @@ onSpeedToggled( GtkCheckMenuItem * check, gpointer vp )
                                     : TR_PREFS_KEY_DSPEED_ENABLED;
 
     if( gtk_check_menu_item_get_active( check ) )
-        tr_core_set_pref_bool( p->core, key, isEnabled );
+        gtr_core_set_pref_bool( p->core, key, isEnabled );
 }
 
 static void
@@ -388,14 +376,14 @@ onSpeedSet( GtkCheckMenuItem * check, gpointer vp )
     const char * key;
     PrivateData * p = vp;
     GObject * o = G_OBJECT( check );
-    const int speed = GPOINTER_TO_INT( g_object_get_data( o, SPEED_KEY ) );
+    const int KBps = GPOINTER_TO_INT( g_object_get_data( o, SPEED_KEY ) );
     tr_direction dir = GPOINTER_TO_INT( g_object_get_data( o, DIRECTION_KEY ) );
 
-    key = dir==TR_UP ? TR_PREFS_KEY_USPEED : TR_PREFS_KEY_DSPEED;
-    tr_core_set_pref_int( p->core, key, speed );
+    key = dir==TR_UP ? TR_PREFS_KEY_USPEED_KBps : TR_PREFS_KEY_DSPEED_KBps;
+    gtr_core_set_pref_int( p->core, key, KBps );
 
     key = dir==TR_UP ? TR_PREFS_KEY_USPEED_ENABLED : TR_PREFS_KEY_DSPEED_ENABLED;
-    tr_core_set_pref_bool( p->core, key, TRUE );
+    gtr_core_set_pref_bool( p->core, key, TRUE );
 }
 
 static GtkWidget*
@@ -403,7 +391,7 @@ createSpeedMenu( PrivateData * p, tr_direction dir )
 {
     int i, n;
     GtkWidget *w, *m;
-    const int speeds[] = { 5, 10, 20, 30, 40, 50, 75, 100, 150, 200, 250, 500, 750 };
+    const int speeds_KBps[] = { 5, 10, 20, 30, 40, 50, 75, 100, 150, 200, 250, 500, 750 };
 
     m = gtk_menu_new( );
 
@@ -424,13 +412,13 @@ createSpeedMenu( PrivateData * p, tr_direction dir )
     w = gtk_separator_menu_item_new( );
     gtk_menu_shell_append( GTK_MENU_SHELL( m ), w );
 
-    for( i=0, n=G_N_ELEMENTS(speeds); i<n; ++i )
+    for( i=0, n=G_N_ELEMENTS(speeds_KBps); i<n; ++i )
     {
         char buf[128];
-        tr_strlspeed( buf, speeds[i], sizeof( buf ) );
+        tr_formatter_speed_KBps( buf, speeds_KBps[i], sizeof( buf ) );
         w = gtk_menu_item_new_with_label( buf );
         g_object_set_data( G_OBJECT( w ), DIRECTION_KEY, GINT_TO_POINTER( dir ) );
-        g_object_set_data( G_OBJECT( w ), SPEED_KEY, GINT_TO_POINTER( speeds[i] ) );
+        g_object_set_data( G_OBJECT( w ), SPEED_KEY, GINT_TO_POINTER( speeds_KBps[i] ) );
         g_signal_connect( w, "activate", G_CALLBACK(onSpeedSet), p );
         gtk_menu_shell_append( GTK_MENU_SHELL( m ), w );
     }
@@ -453,7 +441,7 @@ onRatioToggled( GtkCheckMenuItem * check, gpointer vp )
     if( gtk_check_menu_item_get_active( check ) )
     {
         gboolean f = g_object_get_data( G_OBJECT( check ), ENABLED_KEY ) != 0;
-        tr_core_set_pref_bool( p->core, TR_PREFS_KEY_RATIO_ENABLED, f );
+        gtr_core_set_pref_bool( p->core, TR_PREFS_KEY_RATIO_ENABLED, f );
     }
 }
 static void
@@ -462,8 +450,8 @@ onRatioSet( GtkCheckMenuItem * check, gpointer vp )
     PrivateData * p = vp;
     int i = GPOINTER_TO_INT( g_object_get_data( G_OBJECT( check ), RATIO_KEY ) );
     const double ratio = stockRatios[i];
-    tr_core_set_pref_double( p->core, TR_PREFS_KEY_RATIO, ratio );
-    tr_core_set_pref_bool  ( p->core, TR_PREFS_KEY_RATIO_ENABLED, TRUE );
+    gtr_core_set_pref_double( p->core, TR_PREFS_KEY_RATIO, ratio );
+    gtr_core_set_pref_bool  ( p->core, TR_PREFS_KEY_RATIO_ENABLED, TRUE );
 }
 
 static GtkWidget*
@@ -541,26 +529,26 @@ onOptionsClicked( GtkButton * button UNUSED, gpointer vp )
     PrivateData * p = vp;
 
     w = p->speedlimit_on_item[TR_DOWN];
-    tr_strlspeed( buf1, pref_int_get( TR_PREFS_KEY_DSPEED ), sizeof( buf1 ) );
-    gtk_label_set_text( GTK_LABEL( gtk_bin_get_child( GTK_BIN( w ) ) ), buf1 );
+    tr_formatter_speed_KBps( buf1, gtr_pref_int_get( TR_PREFS_KEY_DSPEED_KBps ), sizeof( buf1 ) );
+    gtr_label_set_text( GTK_LABEL( gtk_bin_get_child( GTK_BIN( w ) ) ), buf1 );
 
-    b = pref_flag_get( TR_PREFS_KEY_DSPEED_ENABLED );
+    b = gtr_pref_flag_get( TR_PREFS_KEY_DSPEED_ENABLED );
     w = b ? p->speedlimit_on_item[TR_DOWN] : p->speedlimit_off_item[TR_DOWN];
     gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( w ), TRUE );
 
     w = p->speedlimit_on_item[TR_UP];
-    tr_strlspeed( buf1, pref_int_get( TR_PREFS_KEY_USPEED ), sizeof( buf1 ) );
-    gtk_label_set_text( GTK_LABEL( gtk_bin_get_child( GTK_BIN( w ) ) ), buf1 );
+    tr_formatter_speed_KBps( buf1, gtr_pref_int_get( TR_PREFS_KEY_USPEED_KBps ), sizeof( buf1 ) );
+    gtr_label_set_text( GTK_LABEL( gtk_bin_get_child( GTK_BIN( w ) ) ), buf1 );
 
-    b = pref_flag_get( TR_PREFS_KEY_USPEED_ENABLED );
+    b = gtr_pref_flag_get( TR_PREFS_KEY_USPEED_ENABLED );
     w = b ? p->speedlimit_on_item[TR_UP] : p->speedlimit_off_item[TR_UP];
     gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( w ), TRUE );
 
-    tr_strlratio( buf1, pref_double_get( TR_PREFS_KEY_RATIO ), sizeof( buf1 ) );
+    tr_strlratio( buf1, gtr_pref_double_get( TR_PREFS_KEY_RATIO ), sizeof( buf1 ) );
     g_snprintf( buf2, sizeof( buf2 ), _( "Stop at Ratio (%s)" ), buf1 );
-    gtk_label_set_text( GTK_LABEL( gtk_bin_get_child( GTK_BIN( p->ratio_on_item ) ) ), buf2 );
+    gtr_label_set_text( GTK_LABEL( gtk_bin_get_child( GTK_BIN( p->ratio_on_item ) ) ), buf2 );
 
-    b = pref_flag_get( TR_PREFS_KEY_RATIO_ENABLED );
+    b = gtr_pref_flag_get( TR_PREFS_KEY_RATIO_ENABLED );
     gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( b ? p->ratio_on_item : p->ratio_off_item ), TRUE );
 
     gtk_menu_popup ( GTK_MENU( p->options_menu ), NULL, NULL, NULL, NULL, 0, gtk_get_current_event_time( ) );
@@ -571,7 +559,7 @@ onOptionsClicked( GtkButton * button UNUSED, gpointer vp )
 ***/
 
 GtkWidget *
-tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
+gtr_window_new( GtkUIManager * ui_mgr, TrCore * core )
 {
     int           i, n;
     const char  * pch;
@@ -585,17 +573,16 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
 
     /* make the window */
     self = gtk_window_new ( GTK_WINDOW_TOPLEVEL );
-    g_object_set_data_full( G_OBJECT(
-                                self ), PRIVATE_DATA_KEY, p, privateFree );
+    g_object_set_qdata_full( G_OBJECT(self), get_private_data_key( ), p, privateFree );
     win = GTK_WINDOW( self );
     gtk_window_set_title( win, g_get_application_name( ) );
     gtk_window_set_role( win, "tr-main" );
     gtk_window_set_default_size( win,
-                                 pref_int_get( PREF_KEY_MAIN_WINDOW_WIDTH ),
-                                 pref_int_get( PREF_KEY_MAIN_WINDOW_HEIGHT ) );
-    gtk_window_move( win, pref_int_get( PREF_KEY_MAIN_WINDOW_X ),
-                          pref_int_get( PREF_KEY_MAIN_WINDOW_Y ) );
-    if( pref_flag_get( PREF_KEY_MAIN_WINDOW_IS_MAXIMIZED ) )
+                                 gtr_pref_int_get( PREF_KEY_MAIN_WINDOW_WIDTH ),
+                                 gtr_pref_int_get( PREF_KEY_MAIN_WINDOW_HEIGHT ) );
+    gtk_window_move( win, gtr_pref_int_get( PREF_KEY_MAIN_WINDOW_X ),
+                          gtr_pref_int_get( PREF_KEY_MAIN_WINDOW_Y ) );
+    if( gtr_pref_flag_get( PREF_KEY_MAIN_WINDOW_IS_MAXIMIZED ) )
         gtk_window_maximize( win );
     gtk_window_add_accel_group( win, gtk_ui_manager_get_accel_group( ui_mgr ) );
 
@@ -604,28 +591,28 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
     gtk_container_add ( GTK_CONTAINER( self ), vbox );
 
     /* main menu */
-    mainmenu = action_get_widget( "/main-window-menu" );
-    w = action_get_widget( "/main-window-menu/torrent-menu/update-tracker" );
+    mainmenu = gtr_action_get_widget( "/main-window-menu" );
+    w = gtr_action_get_widget( "/main-window-menu/torrent-menu/update-tracker" );
 #if GTK_CHECK_VERSION( 2, 12, 0 )
     g_signal_connect( w, "query-tooltip",
                       G_CALLBACK( onAskTrackerQueryTooltip ), p );
 #endif
 
     /* toolbar */
-    toolbar = p->toolbar = action_get_widget( "/main-window-toolbar" );
-    action_set_important( "add-torrent-toolbar", TRUE );
-    action_set_important( "show-torrent-properties", TRUE );
+    toolbar = p->toolbar = gtr_action_get_widget( "/main-window-toolbar" );
+    gtr_action_set_important( "open-torrent-toolbar", TRUE );
+    gtr_action_set_important( "show-torrent-properties", TRUE );
 
     /* filter */
-    h = filter = p->filter = gtr_filter_bar_new( tr_core_session( core ),
-                                                 tr_core_model( core ),
+    h = filter = p->filter = gtr_filter_bar_new( gtr_core_session( core ),
+                                                 gtr_core_model( core ),
                                                  &p->filter_model );
     gtk_container_set_border_width( GTK_CONTAINER( h ), GUI_PAD_SMALL );
 
     /* status menu */
     menu = p->status_menu = gtk_menu_new( );
     l = NULL;
-    pch = pref_string_get( PREF_KEY_STATUSBAR_STATS );
+    pch = gtr_pref_string_get( PREF_KEY_STATUSBAR_STATS );
     for( i = 0, n = G_N_ELEMENTS( stats_modes ); i < n; ++i )
     {
         const char * val = stats_modes[i].val;
@@ -647,7 +634,7 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
     gtk_container_set_border_width( GTK_CONTAINER( h ), GUI_PAD_SMALL );
 
         w = gtk_button_new( );
-        gtk_container_add( GTK_CONTAINER( w ), gtk_image_new_from_stock( "options", -1 ) );
+        gtk_container_add( GTK_CONTAINER( w ), gtk_image_new_from_stock( "utilities", -1 ) );
         gtr_widget_set_tooltip_text( w, _( "Options" ) );
         gtk_box_pack_start( GTK_BOX( h ), w, 0, 0, 0 );
         gtk_button_set_relief( GTK_BUTTON( w ), GTK_RELIEF_NONE );
@@ -662,6 +649,7 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
         gtk_box_pack_start( GTK_BOX( h ), w, 0, 0, 0 );
 
         w = p->gutter_lb = gtk_label_new( "N Torrents" );
+        gtk_label_set_single_line_mode( GTK_LABEL( w ), TRUE );
         gtk_box_pack_start( GTK_BOX( h ), w, 1, 1, GUI_PAD );
 
         hbox = gtk_hbox_new( FALSE, GUI_PAD );
@@ -669,6 +657,7 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
             gtk_widget_set_size_request( w, GUI_PAD, 0u );
             gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
             w = p->ul_lb = gtk_label_new( NULL );
+            gtk_label_set_single_line_mode( GTK_LABEL( w ), TRUE );
             gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
             w = gtk_image_new_from_stock( GTK_STOCK_GO_UP, GTK_ICON_SIZE_MENU );
             gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
@@ -679,6 +668,7 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
             gtk_widget_set_size_request( w, GUI_PAD, 0u );
             gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
             w = p->dl_lb = gtk_label_new( NULL );
+            gtk_label_set_single_line_mode( GTK_LABEL( w ), TRUE );
             gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
             w = gtk_image_new_from_stock( GTK_STOCK_GO_DOWN, GTK_ICON_SIZE_MENU );
             gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
@@ -692,6 +682,7 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
             g_signal_connect( w, "clicked", G_CALLBACK( onYinYangReleased ), p );
             gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
             w = p->stats_lb = gtk_label_new( NULL );
+            gtk_label_set_single_line_mode( GTK_LABEL( w ), TRUE );
             gtk_box_pack_end( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
         gtk_box_pack_end( GTK_BOX( h ), hbox, FALSE, FALSE, 0 );
 
@@ -705,33 +696,18 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
                                          GTK_SHADOW_IN );
     gtk_container_add( GTK_CONTAINER( w ), p->view );
 
-    /* layout the widgets */
-    {
-        const char * str = pref_string_get( PREF_KEY_MAIN_WINDOW_LAYOUT_ORDER );
-        char ** tokens = g_strsplit( str, ",", -1 );
-        for( i=0; tokens && tokens[i]; ++i )
-        {
-            const char * key = tokens[i];
-
-            if( !strcmp( key, "menu" ) )
-                gtk_box_pack_start( GTK_BOX( vbox ), mainmenu, FALSE, FALSE, 0 );
-            else if( !strcmp( key, "toolbar" ) )
-                gtk_box_pack_start( GTK_BOX( vbox ), toolbar, FALSE, FALSE, 0 );
-            else if( !strcmp( key, "filter" ) )
-                gtk_box_pack_start( GTK_BOX( vbox ), filter, FALSE, FALSE, 0 );
-            else if( !strcmp( key, "list" ) )
-                gtk_box_pack_start( GTK_BOX( vbox ), list, TRUE, TRUE, 0 );
-            else if( !strcmp( key, "statusbar" ) )
-                gtk_box_pack_start( GTK_BOX( vbox ), status, FALSE, FALSE, 0 );
-        }
-        g_strfreev( tokens );
-    }
+    /* lay out the widgets */
+    gtk_box_pack_start( GTK_BOX( vbox ), mainmenu, FALSE, FALSE, 0 );
+    gtk_box_pack_start( GTK_BOX( vbox ), toolbar, FALSE, FALSE, 0 );
+    gtk_box_pack_start( GTK_BOX( vbox ), filter, FALSE, FALSE, 0 );
+    gtk_box_pack_start( GTK_BOX( vbox ), list, TRUE, TRUE, 0 );
+    gtk_box_pack_start( GTK_BOX( vbox ), status, FALSE, FALSE, 0 );
 
     {
-        int w=0, h=0;
         /* this is to determine the maximum width/height for the label */
-        PangoLayout * pango_layout =
-            gtk_widget_create_pango_layout( p->ul_lb, _( "999.9 KiB/s" ) );
+        int w=0, h=0;
+        PangoLayout * pango_layout;
+        pango_layout = gtk_widget_create_pango_layout( p->ul_lb, "999.99 KiB/s" );
         pango_layout_get_pixel_size( pango_layout, &w, &h );
         gtk_widget_set_size_request( p->ul_lb, w, h );
         gtk_widget_set_size_request( p->dl_lb, w, h );
@@ -750,12 +726,11 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
     prefsChanged( core, PREF_KEY_STATUSBAR, self );
     prefsChanged( core, PREF_KEY_STATUSBAR_STATS, self );
     prefsChanged( core, PREF_KEY_TOOLBAR, self );
-    prefsChanged( core, PREF_KEY_FILTER_MODE, self );
     prefsChanged( core, TR_PREFS_KEY_ALT_SPEED_ENABLED, self );
     p->pref_handler_id = g_signal_connect( core, "prefs-changed",
                                            G_CALLBACK( prefsChanged ), self );
 
-    tr_sessionSetAltSpeedFunc( tr_core_session( core ), onAltSpeedToggled, p );
+    tr_sessionSetAltSpeedFunc( gtr_core_session( core ), onAltSpeedToggled, p );
 
     return self;
 }
@@ -766,10 +741,8 @@ updateTorrentCount( PrivateData * p )
     if( p && p->core )
     {
         char      buf[512];
-        const int torrentCount = gtk_tree_model_iter_n_children(
-            tr_core_model( p->core ), NULL );
-        const int visibleCount = gtk_tree_model_iter_n_children(
-            p->filter_model, NULL );
+        const int torrentCount = gtk_tree_model_iter_n_children( gtr_core_model( p->core ), NULL );
+        const int visibleCount = gtk_tree_model_iter_n_children( p->filter_model, NULL );
 
         if( !torrentCount )
             *buf = '\0';
@@ -783,7 +756,7 @@ updateTorrentCount( PrivateData * p )
             g_snprintf( buf, sizeof( buf ),
                         ngettext( "%'d Torrent", "%'d Torrents", torrentCount ),
                         torrentCount );
-        gtk_label_set_text( GTK_LABEL( p->gutter_lb ), buf );
+        gtr_label_set_text( GTK_LABEL( p->gutter_lb ), buf );
     }
 }
 
@@ -793,10 +766,10 @@ updateStats( PrivateData * p )
     const char *            pch;
     char                    up[32], down[32], ratio[32], buf[512];
     struct tr_session_stats stats;
-    tr_session *            session = tr_core_session( p->core );
+    tr_session *            session = gtr_core_session( p->core );
 
     /* update the stats */
-    pch = pref_string_get( PREF_KEY_STATUSBAR_STATS );
+    pch = gtr_pref_string_get( PREF_KEY_STATUSBAR_STATS );
     if( !strcmp( pch, "session-ratio" ) )
     {
         tr_sessionGetStats( session, &stats );
@@ -808,7 +781,7 @@ updateStats( PrivateData * p )
         tr_sessionGetStats( session, &stats );
         tr_strlsize( up, stats.uploadedBytes, sizeof( up ) );
         tr_strlsize( down, stats.downloadedBytes, sizeof( down ) );
-        /* Translators: "size|" is here for disambiguation.  Please remove it from your translation.
+        /* Translators: "size|" is here for disambiguation. Please remove it from your translation.
            %1$s is the size of the data we've downloaded
            %2$s is the size of the data we've uploaded */
         g_snprintf( buf, sizeof( buf ), Q_(
@@ -819,7 +792,7 @@ updateStats( PrivateData * p )
         tr_sessionGetCumulativeStats( session, &stats );
         tr_strlsize( up, stats.uploadedBytes, sizeof( up ) );
         tr_strlsize( down, stats.downloadedBytes, sizeof( down ) );
-        /* Translators: "size|" is here for disambiguation.  Please remove it from your translation.
+        /* Translators: "size|" is here for disambiguation. Please remove it from your translation.
            %1$s is the size of the data we've downloaded
            %2$s is the size of the data we've uploaded */
         g_snprintf( buf, sizeof( buf ), Q_(
@@ -831,22 +804,22 @@ updateStats( PrivateData * p )
         tr_strlratio( ratio, stats.ratio, sizeof( ratio ) );
         g_snprintf( buf, sizeof( buf ), _( "Ratio: %s" ), ratio );
     }
-    gtk_label_set_text( GTK_LABEL( p->stats_lb ), buf );
+    gtr_label_set_text( GTK_LABEL( p->stats_lb ), buf );
 }
 
 static void
 updateSpeeds( PrivateData * p )
 {
-    tr_session * session = tr_core_session( p->core );
+    tr_session * session = gtr_core_session( p->core );
 
     if( session != NULL )
     {
         char buf[128];
         double up=0, down=0;
         GtkTreeIter iter;
-        GtkTreeModel * model = tr_core_model( p->core );
+        GtkTreeModel * model = gtr_core_model( p->core );
 
-        if( gtk_tree_model_get_iter_first( model, &iter ) ) do
+        if( gtk_tree_model_iter_nth_child( model, &iter, NULL, 0 ) ) do
         {
             double u, d;
             gtk_tree_model_get( model, &iter, MC_SPEED_UP, &u,
@@ -857,31 +830,50 @@ updateSpeeds( PrivateData * p )
         }
         while( gtk_tree_model_iter_next( model, &iter ) );
 
-        tr_strlspeed( buf, down, sizeof( buf ) );
-        gtk_label_set_text( GTK_LABEL( p->dl_lb ), buf );
+        tr_formatter_speed_KBps( buf, down, sizeof( buf ) );
+        gtr_label_set_text( GTK_LABEL( p->dl_lb ), buf );
 
-        tr_strlspeed( buf, up, sizeof( buf ) );
-        gtk_label_set_text( GTK_LABEL( p->ul_lb ), buf );
+        tr_formatter_speed_KBps( buf, up, sizeof( buf ) );
+        gtr_label_set_text( GTK_LABEL( p->ul_lb ), buf );
     }
 }
 
 void
-tr_window_update( TrWindow * self )
+gtr_window_refresh( TrWindow * self )
 {
     PrivateData * p = get_private_data( self );
 
-    if( p && p->core && tr_core_session( p->core ) )
+    if( p && p->core && gtr_core_session( p->core ) )
     {
         updateSpeeds( p );
         updateTorrentCount( p );
         updateStats( p );
-        gtk_tree_model_filter_refilter( GTK_TREE_MODEL_FILTER( p->filter_model ) );
     }
 }
 
 GtkTreeSelection*
-tr_window_get_selection( TrWindow * w )
+gtr_window_get_selection( TrWindow * w )
 {
     return get_private_data( w )->selection;
 }
 
+void
+gtr_window_set_busy( TrWindow * w, gboolean isBusy )
+{
+    if( w && gtr_widget_get_realized( GTK_WIDGET( w ) ) )
+    {
+        GdkDisplay * display = gtk_widget_get_display( GTK_WIDGET( w ) );
+        GdkCursor * cursor = isBusy ? gdk_cursor_new_for_display( display, GDK_WATCH ) : NULL;
+
+        gdk_window_set_cursor( gtr_widget_get_window( GTK_WIDGET( w ) ), cursor );
+        gdk_display_flush( display );
+
+        if( cursor ) {
+#if GTK_CHECK_VERSION(3,0,0)
+            g_object_unref( cursor );
+#else
+            gdk_cursor_unref( cursor );
+#endif
+        }
+    }
+}
