@@ -1,22 +1,21 @@
-/* * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
+/*
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: crypto.c 8689 2009-06-14 14:39:51Z charles $
+ * $Id: crypto.c 12365 2011-04-17 05:22:50Z jordan $
  */
 
-#include <stdlib.h> /* for abs() */
-#include <limits.h> /* for INT_MAX */
-#include <sys/types.h> /* for event.h, as well as netinet/in.h on some platforms
-                         */
 #include <assert.h>
 #include <inttypes.h> /* uint8_t */
-#include <string.h> /* memcpy */
+#include <limits.h> /* INT_MAX */
 #include <stdarg.h>
+#include <stdlib.h> /* abs() */
+#include <string.h> /* memcpy(), memset(), strcmp() */
 
 #include <openssl/bn.h>
 #include <openssl/dh.h>
@@ -25,8 +24,7 @@
 #include <openssl/sha.h>
 #include <openssl/rand.h>
 
-#include <event.h>
-
+#include "transmission.h"
 #include "crypto.h"
 #include "utils.h"
 
@@ -37,27 +35,20 @@
 **/
 
 void
-tr_sha1( uint8_t *    setme,
-         const void * content1,
-         int          content1_len,
-         ... )
+tr_sha1( uint8_t * setme, const void * content1, int content1_len, ... )
 {
     va_list vl;
     SHA_CTX sha;
+    const void * content;
 
     SHA1_Init( &sha );
     SHA1_Update( &sha, content1, content1_len );
 
     va_start( vl, content1_len );
-    for( ; ; )
-    {
-        const void * content = (const void*) va_arg( vl, const void* );
-        const int    content_len = content ? (int) va_arg( vl, int ) : -1;
-        if( content == NULL || content_len < 1 )
-            break;
-        SHA1_Update( &sha, content, content_len );
-    }
+    while(( content = va_arg( vl, const void* )))
+        SHA1_Update( &sha, content, va_arg( vl, int ) );
     va_end( vl );
+
     SHA1_Final( setme, &sha );
 }
 
@@ -69,6 +60,9 @@ tr_sha1( uint8_t *    setme,
 
 #define PRIME_LEN 96
 
+#define DH_PRIVKEY_LEN_MIN 16
+#define DH_PRIVKEY_LEN 20
+
 static const uint8_t dh_P[PRIME_LEN] =
 {
     0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xC9, 0x0F, 0xDA, 0xA2,
@@ -83,18 +77,6 @@ static const uint8_t dh_P[PRIME_LEN] =
 
 static const uint8_t dh_G[] = { 2 };
 
-struct tr_crypto
-{
-    RC4_KEY         dec_key;
-    RC4_KEY         enc_key;
-    uint8_t         torrentHash[SHA_DIGEST_LENGTH];
-    tr_bool         isIncoming;
-    tr_bool         torrentHashIsSet;
-    tr_bool         mySecretIsSet;
-    uint8_t         myPublicKey[KEY_LEN];
-    uint8_t         mySecret[KEY_LEN];
-};
-
 /**
 ***
 **/
@@ -108,14 +90,13 @@ struct tr_crypto
         } \
     } while( 0 )
 
-static DH*
-getSharedDH( void )
+static void
+ensureKeyExists( tr_crypto * crypto)
 {
-    static DH * dh = NULL;
-
-    if( dh == NULL )
+    if( crypto->dh == NULL )
     {
-        dh = DH_new( );
+        int len, offset;
+        DH * dh = DH_new( );
 
         dh->p = BN_bin2bn( dh_P, sizeof( dh_P ), NULL );
         if( dh->p == NULL )
@@ -125,41 +106,44 @@ getSharedDH( void )
         if( dh->g == NULL )
             logErrorFromSSL( );
 
+        /* private DH value: strong random BN of DH_PRIVKEY_LEN*8 bits */
+        dh->priv_key = BN_new( );
+        do {
+            if( BN_rand( dh->priv_key, DH_PRIVKEY_LEN * 8, -1, 0 ) != 1 )
+                logErrorFromSSL( );
+        } while ( BN_num_bits( dh->priv_key ) < DH_PRIVKEY_LEN_MIN * 8 );
+
         if( !DH_generate_key( dh ) )
             logErrorFromSSL( );
+
+        /* DH can generate key sizes that are smaller than the size of
+           P with exponentially decreasing probability, in which case
+           the msb's of myPublicKey need to be zeroed appropriately. */
+        len = BN_num_bytes( dh->pub_key );
+        offset = KEY_LEN - len;
+        assert( len <= KEY_LEN );
+        memset( crypto->myPublicKey, 0, offset );
+        BN_bn2bin( dh->pub_key, crypto->myPublicKey + offset );
+
+        crypto->dh = dh;
     }
-
-    return dh;
-}
-
-tr_crypto *
-tr_cryptoNew( const uint8_t * torrentHash,
-              int             isIncoming )
-{
-    int         len, offset;
-    tr_crypto * crypto;
-    DH *        dh = getSharedDH( );
-
-    crypto = tr_new0( tr_crypto, 1 );
-    crypto->isIncoming = isIncoming ? 1 : 0;
-    tr_cryptoSetTorrentHash( crypto, torrentHash );
-
-    /* DH can generate key sizes that are smaller than the size of
-       P with exponentially decreasing probability, in which case
-       the msb's of myPublicKey need to be zeroed appropriately. */
-    len = DH_size( dh );
-    offset = KEY_LEN - len;
-    assert( len <= KEY_LEN );
-    memset( crypto->myPublicKey, 0, offset );
-    BN_bn2bin( dh->pub_key, crypto->myPublicKey + offset );
-
-    return crypto;
 }
 
 void
-tr_cryptoFree( tr_crypto * crypto )
+tr_cryptoConstruct( tr_crypto * crypto, const uint8_t * torrentHash, bool isIncoming )
 {
-    tr_free( crypto );
+    memset( crypto, 0, sizeof ( tr_crypto ) );
+
+    crypto->dh = NULL;
+    crypto->isIncoming = isIncoming;
+    tr_cryptoSetTorrentHash( crypto, torrentHash );
+}
+
+void
+tr_cryptoDestruct( tr_crypto * crypto )
+{
+    if( crypto->dh != NULL )
+        DH_free( crypto->dh );
 }
 
 /**
@@ -173,7 +157,10 @@ tr_cryptoComputeSecret( tr_crypto *     crypto,
     int      len;
     uint8_t  secret[KEY_LEN];
     BIGNUM * bn = BN_bin2bn( peerPublicKey, KEY_LEN, NULL );
-    DH *     dh = getSharedDH( );
+    DH *     dh;
+
+    ensureKeyExists( crypto );
+    dh = crypto->dh;
 
     assert( DH_size( dh ) == KEY_LEN );
 
@@ -197,6 +184,7 @@ const uint8_t*
 tr_cryptoGetMyPublicKey( const tr_crypto * crypto,
                          int *             setme_len )
 {
+    ensureKeyExists( (tr_crypto *) crypto );
     *setme_len = KEY_LEN;
     return crypto->myPublicKey;
 }
@@ -322,30 +310,23 @@ tr_cryptoRandInt( int upperBound )
         val = tr_cryptoWeakRandInt( upperBound );
     }
 
-    assert( val >= 0 );
-    assert( val < upperBound );
     return val;
 }
 
 int
 tr_cryptoWeakRandInt( int upperBound )
 {
-    int val;
-    static tr_bool init = FALSE;
+    static bool init = false;
 
     assert( upperBound > 0 );
 
     if( !init )
     {
-        srand( tr_date( ) );
-        init = TRUE;
+        srand( tr_time_msec( ) );
+        init = true;
     }
 
-    
-    val = rand( ) % upperBound;
-    assert( val >= 0 );
-    assert( val < upperBound );
-    return val;
+    return rand( ) % upperBound;
 }
 
 void
@@ -362,20 +343,21 @@ tr_cryptoRandBuf( void * buf, size_t len )
 char*
 tr_ssha1( const void * plaintext )
 {
+    enum { saltval_len = 8,
+           salter_len  = 64 };
     static const char * salter = "0123456789"
                                  "abcdefghijklmnopqrstuvwxyz"
                                  "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
                                  "./";
-    static const size_t salter_len = 64;
-    static const size_t saltval_len = 8;
 
     size_t i;
-    char salt[saltval_len];
+    unsigned char salt[saltval_len];
     uint8_t sha[SHA_DIGEST_LENGTH];
     char buf[2*SHA_DIGEST_LENGTH + saltval_len + 2];
 
+    tr_cryptoRandBuf( salt, saltval_len );
     for( i=0; i<saltval_len; ++i )
-        salt[i] = salter[ tr_cryptoRandInt( salter_len ) ];
+        salt[i] = salter[ salt[i] % salter_len ];
 
     tr_sha1( sha, plaintext, strlen( plaintext ), salt, saltval_len, NULL );
     tr_sha1_to_hex( &buf[1], sha );
@@ -387,14 +369,14 @@ tr_ssha1( const void * plaintext )
     return tr_strdup( &buf );
 }
 
-tr_bool
+bool
 tr_ssha1_matches( const char * source, const char * pass )
 {
     char * salt;
     size_t saltlen;
     char * hashed;
     uint8_t buf[SHA_DIGEST_LENGTH];
-    tr_bool result;
+    bool result;
 
     /* extract the salt */
     saltlen = strlen( source ) - 2*SHA_DIGEST_LENGTH-1;
@@ -409,10 +391,10 @@ tr_ssha1_matches( const char * source, const char * pass )
     hashed[1+2*SHA_DIGEST_LENGTH + saltlen] = '\0';
     hashed[0] = '{';
 
-    result = strcmp( source, hashed ) == 0 ? TRUE : FALSE;
+    result = strcmp( source, hashed ) == 0 ? true : false;
 
     tr_free( hashed );
     tr_free( salt );
-    
+
     return result;
 }
