@@ -1,26 +1,23 @@
 /*
- * This file Copyright (C) 2007-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: handshake.c 10506 2010-04-21 04:44:35Z charles $
+ * $Id: handshake.c 12365 2011-04-17 05:22:50Z jordan $
  */
 
 #include <assert.h>
 #include <errno.h>
-#include <inttypes.h>
-#include <limits.h> /* UCHAR_MAX */
-#include <string.h>
-#include <stdio.h>
+#include <string.h> /* strcmp(), strlen() */
 
-#include <event.h>
+#include <event2/buffer.h>
+#include <event2/event.h>
 
 #include "transmission.h"
-#include "bencode.h"
 #include "clients.h"
 #include "crypto.h"
 #include "handshake.h"
@@ -50,7 +47,6 @@ enum
     HANDSHAKE_NAME_LEN             = 20,
     HANDSHAKE_FLAGS_LEN            = 8,
     HANDSHAKE_SIZE                 = 68,
-    PEER_ID_LEN                    = 20,
     INCOMING_HANDSHAKE_LEN         = 48,
 
     /* Encryption Constants */
@@ -59,7 +55,6 @@ enum
     PadC_MAXLEN                    = 512,
     PadD_MAXLEN                    = 512,
     VC_LENGTH                      = 8,
-    KEY_LEN                        = 96,
     CRYPTO_PROVIDE_PLAINTEXT       = 1,
     CRYPTO_PROVIDE_CRYPTO          = 2,
 
@@ -99,16 +94,18 @@ enum
 #define HANDSHAKE_SET_EXTPREF( reserved, val ) ( ( reserved )[5] |= 0x03 &\
                                                                     ( val ) )
 
+typedef uint8_t handshake_state_t;
+
 struct tr_handshake
 {
-    tr_bool               haveReadAnythingFromPeer;
-    tr_bool               havePeerID;
-    tr_bool               haveSentBitTorrentHandshake;
+    bool                  haveReadAnythingFromPeer;
+    bool                  havePeerID;
+    bool                  haveSentBitTorrentHandshake;
     tr_peerIo *           io;
     tr_crypto *           crypto;
     tr_session *          session;
     uint8_t               mySecret[KEY_LEN];
-    uint8_t               state;
+    handshake_state_t     state;
     tr_encryption_mode    encryptionMode;
     uint16_t              pad_c_len;
     uint16_t              pad_d_len;
@@ -118,7 +115,7 @@ struct tr_handshake
     uint8_t               myReq1[SHA_DIGEST_LENGTH];
     handshakeDoneCB       doneCB;
     void *                doneUserData;
-    struct event          timeout_timer;
+    struct event        * timeout_timer;
 };
 
 /**
@@ -155,7 +152,7 @@ enum
     } while( 0 )
 
 static const char*
-getStateName( short state )
+getStateName( const handshake_state_t state )
 {
     const char * str = "f00!";
 
@@ -198,16 +195,14 @@ getStateName( short state )
 }
 
 static void
-setState( tr_handshake * handshake,
-          short          state )
+setState( tr_handshake * handshake, handshake_state_t state )
 {
     dbgmsg( handshake, "setting to state [%s]", getStateName( state ) );
     handshake->state = state;
 }
 
 static void
-setReadState( tr_handshake * handshake,
-              int            state )
+setReadState( tr_handshake * handshake, handshake_state_t state )
 {
     setState( handshake, state );
 }
@@ -218,7 +213,7 @@ buildHandshakeMessage( tr_handshake * handshake, uint8_t * buf )
     uint8_t          * walk = buf;
     const uint8_t    * torrentHash = tr_cryptoGetTorrentHash( handshake->crypto );
     const tr_torrent * tor = tr_torrentFindFromHash( handshake->session, torrentHash );
-    const uint8_t    * peer_id = tor && tor->peer_id ? tor->peer_id : tr_getPeerId( );
+    const uint8_t    * peer_id = tor && *tor->peer_id ? tor->peer_id : tr_getPeerId( handshake->session );
 
     memcpy( walk, HANDSHAKE_NAME, HANDSHAKE_NAME_LEN );
     walk += HANDSHAKE_NAME_LEN;
@@ -238,12 +233,11 @@ buildHandshakeMessage( tr_handshake * handshake, uint8_t * buf )
     memcpy( walk, peer_id, PEER_ID_LEN );
     walk += PEER_ID_LEN;
 
-    assert( strlen( ( const char* )peer_id ) == PEER_ID_LEN );
     assert( walk - buf == HANDSHAKE_SIZE );
 }
 
 static int tr_handshakeDone( tr_handshake * handshake,
-                             tr_bool        isConnected );
+                             bool           isConnected );
 
 enum
 {
@@ -265,9 +259,9 @@ parseHandshake( tr_handshake *    handshake,
     uint8_t peer_id[PEER_ID_LEN];
 
     dbgmsg( handshake, "payload: need %d, got %zu",
-            (int)HANDSHAKE_SIZE, EVBUFFER_LENGTH( inbuf ) );
+            HANDSHAKE_SIZE, evbuffer_get_length( inbuf ) );
 
-    if( EVBUFFER_LENGTH( inbuf ) < HANDSHAKE_SIZE )
+    if( evbuffer_get_length( inbuf ) < HANDSHAKE_SIZE )
         return READ_LATER;
 
     /* confirm the protocol */
@@ -294,11 +288,11 @@ parseHandshake( tr_handshake *    handshake,
     tr_peerIoSetPeersId( handshake->io, peer_id );
 
     /* peer id */
-    handshake->havePeerID = TRUE;
+    handshake->havePeerID = true;
     dbgmsg( handshake, "peer-id is [%*.*s]", PEER_ID_LEN, PEER_ID_LEN, peer_id );
 
     tor = tr_torrentFindFromHash( handshake->session, hash );
-    tor_peer_id = tor && tor->peer_id ? tor->peer_id : tr_getPeerId( );
+    tor_peer_id = tor ? tor->peer_id : tr_getPeerId( handshake->session );
     if( !memcmp( peer_id, tor_peer_id, PEER_ID_LEN ) )
     {
         dbgmsg( handshake, "streuth!  we've connected to ourselves." );
@@ -346,7 +340,7 @@ sendYa( tr_handshake * handshake )
 
     /* send it */
     setReadState( handshake, AWAITING_YB );
-    tr_peerIoWrite( handshake->io, outbuf, walk-outbuf, FALSE );
+    tr_peerIoWriteBytes( handshake->io, outbuf, walk - outbuf, false );
 }
 
 static uint32_t
@@ -401,8 +395,7 @@ getCryptoSelect( const tr_handshake * handshake,
 }
 
 static int
-readYb( tr_handshake *    handshake,
-        struct evbuffer * inbuf )
+readYb( tr_handshake * handshake, struct evbuffer * inbuf )
 {
     int               isEncrypted;
     const uint8_t *   secret;
@@ -410,14 +403,14 @@ readYb( tr_handshake *    handshake,
     struct evbuffer * outbuf;
     size_t            needlen = HANDSHAKE_NAME_LEN;
 
-    if( EVBUFFER_LENGTH( inbuf ) < needlen )
+    if( evbuffer_get_length( inbuf ) < needlen )
         return READ_LATER;
 
-    isEncrypted = memcmp( EVBUFFER_DATA( inbuf ), HANDSHAKE_NAME, HANDSHAKE_NAME_LEN );
+    isEncrypted = memcmp( evbuffer_pullup( inbuf, HANDSHAKE_NAME_LEN ), HANDSHAKE_NAME, HANDSHAKE_NAME_LEN );
     if( isEncrypted )
     {
         needlen = KEY_LEN;
-        if( EVBUFFER_LENGTH( inbuf ) < needlen )
+        if( evbuffer_get_length( inbuf ) < needlen )
             return READ_LATER;
     }
 
@@ -432,7 +425,7 @@ readYb( tr_handshake *    handshake,
         return READ_NOW;
     }
 
-    handshake->haveReadAnythingFromPeer = TRUE;
+    handshake->haveReadAnythingFromPeer = true;
 
     /* compute the secret */
     evbuffer_remove( inbuf, yb, KEY_LEN );
@@ -471,14 +464,13 @@ readYb( tr_handshake *    handshake,
     {
         uint8_t vc[VC_LENGTH] = { 0, 0, 0, 0, 0, 0, 0, 0 };
 
-        tr_peerIoWriteBuf( handshake->io, outbuf, FALSE );
+        tr_peerIoWriteBuf( handshake->io, outbuf, false );
         tr_cryptoEncryptInit( handshake->crypto );
         tr_peerIoSetEncryption( handshake->io, PEER_ENCRYPTION_RC4 );
 
-        tr_peerIoWriteBytes( handshake->io, outbuf, vc, VC_LENGTH );
-        tr_peerIoWriteUint32( handshake->io, outbuf,
-                             getCryptoProvide( handshake ) );
-        tr_peerIoWriteUint16( handshake->io, outbuf, 0 );
+        evbuffer_add        ( outbuf, vc, VC_LENGTH );
+        evbuffer_add_uint32 ( outbuf, getCryptoProvide( handshake ) );
+        evbuffer_add_uint16 ( outbuf, 0 );
     }
 
     /* ENCRYPT len(IA)), ENCRYPT(IA) */
@@ -486,8 +478,8 @@ readYb( tr_handshake *    handshake,
         uint8_t msg[HANDSHAKE_SIZE];
         buildHandshakeMessage( handshake, msg );
 
-        tr_peerIoWriteUint16( handshake->io, outbuf, sizeof( msg ) );
-        tr_peerIoWriteBytes( handshake->io, outbuf, msg, sizeof( msg ) );
+        evbuffer_add_uint16 ( outbuf, sizeof( msg ) );
+        evbuffer_add        ( outbuf, msg, sizeof( msg ) );
 
         handshake->haveSentBitTorrentHandshake = 1;
     }
@@ -495,7 +487,7 @@ readYb( tr_handshake *    handshake,
     /* send it */
     tr_cryptoDecryptInit( handshake->crypto );
     setReadState( handshake, AWAITING_VC );
-    tr_peerIoWriteBuf( handshake->io, outbuf, FALSE );
+    tr_peerIoWriteBuf( handshake->io, outbuf, false );
 
     /* cleanup */
     evbuffer_free( outbuf );
@@ -515,13 +507,13 @@ readVC( tr_handshake *    handshake,
      * it would be nice to make this cleaner. */
     for( ; ; )
     {
-        if( EVBUFFER_LENGTH( inbuf ) < VC_LENGTH )
+        if( evbuffer_get_length( inbuf ) < VC_LENGTH )
         {
             dbgmsg( handshake, "not enough bytes... returning read_more" );
             return READ_LATER;
         }
 
-        memcpy( tmp, EVBUFFER_DATA( inbuf ), key_len );
+        memcpy( tmp, evbuffer_pullup( inbuf, key_len ), key_len );
         tr_cryptoDecryptInit( handshake->crypto );
         tr_cryptoDecrypt( handshake->crypto, key_len, tmp, tmp );
         if( !memcmp( tmp, key, key_len ) )
@@ -544,7 +536,7 @@ readCryptoSelect( tr_handshake *    handshake,
     uint16_t     pad_d_len;
     const size_t needlen = sizeof( uint32_t ) + sizeof( uint16_t );
 
-    if( EVBUFFER_LENGTH( inbuf ) < needlen )
+    if( evbuffer_get_length( inbuf ) < needlen )
         return READ_LATER;
 
     tr_peerIoReadUint32( handshake->io, inbuf, &crypto_select );
@@ -554,7 +546,7 @@ readCryptoSelect( tr_handshake *    handshake,
     {
         dbgmsg( handshake,
                 "peer selected an encryption option we didn't provide" );
-        return tr_handshakeDone( handshake, FALSE );
+        return tr_handshakeDone( handshake, false );
     }
 
     tr_peerIoReadUint16( handshake->io, inbuf, &pad_d_len );
@@ -563,7 +555,7 @@ readCryptoSelect( tr_handshake *    handshake,
     if( pad_d_len > 512 )
     {
         dbgmsg( handshake, "encryption handshake: pad_d_len is too long" );
-        return tr_handshakeDone( handshake, FALSE );
+        return tr_handshakeDone( handshake, false );
     }
 
     handshake->pad_d_len = pad_d_len;
@@ -577,19 +569,15 @@ readPadD( tr_handshake *    handshake,
           struct evbuffer * inbuf )
 {
     const size_t needlen = handshake->pad_d_len;
-    uint8_t *    tmp;
 
     dbgmsg( handshake, "pad d: need %zu, got %zu",
-            needlen, EVBUFFER_LENGTH( inbuf ) );
-    if( EVBUFFER_LENGTH( inbuf ) < needlen )
+            needlen, evbuffer_get_length( inbuf ) );
+    if( evbuffer_get_length( inbuf ) < needlen )
         return READ_LATER;
 
-    tmp = tr_new( uint8_t, needlen );
-    tr_peerIoReadBytes( handshake->io, inbuf, tmp, needlen );
-    tr_free( tmp );
+    tr_peerIoDrain( handshake->io, inbuf, needlen );
 
-    tr_peerIoSetEncryption( handshake->io,
-                            handshake->crypto_select );
+    tr_peerIoSetEncryption( handshake->io, handshake->crypto_select );
 
     setState( handshake, AWAITING_HANDSHAKE );
     return READ_NOW;
@@ -606,20 +594,20 @@ readHandshake( tr_handshake *    handshake,
                struct evbuffer * inbuf )
 {
     uint8_t   pstrlen;
-    uint8_t * pstr;
+    uint8_t   pstr[20];
     uint8_t   reserved[HANDSHAKE_FLAGS_LEN];
     uint8_t   hash[SHA_DIGEST_LENGTH];
 
     dbgmsg( handshake, "payload: need %d, got %zu",
-            (int)INCOMING_HANDSHAKE_LEN, EVBUFFER_LENGTH( inbuf ) );
+            INCOMING_HANDSHAKE_LEN, evbuffer_get_length( inbuf ) );
 
-    if( EVBUFFER_LENGTH( inbuf ) < INCOMING_HANDSHAKE_LEN )
+    if( evbuffer_get_length( inbuf ) < INCOMING_HANDSHAKE_LEN )
         return READ_LATER;
 
-    handshake->haveReadAnythingFromPeer = TRUE;
+    handshake->haveReadAnythingFromPeer = true;
 
-    pstrlen = EVBUFFER_DATA( inbuf )[0]; /* peek, don't read.  We may be
-                                          handing inbuf to AWAITING_YA */
+    pstrlen = evbuffer_pullup( inbuf, 1 )[0]; /* peek, don't read. We may be
+                                                 handing inbuf to AWAITING_YA */
 
     if( pstrlen == 19 ) /* unencrypted */
     {
@@ -629,7 +617,7 @@ readHandshake( tr_handshake *    handshake,
         {
             dbgmsg( handshake,
                     "peer is unencrypted, and we're disallowing that" );
-            return tr_handshakeDone( handshake, FALSE );
+            return tr_handshakeDone( handshake, false );
         }
     }
     else /* encrypted or corrupt */
@@ -649,22 +637,18 @@ readHandshake( tr_handshake *    handshake,
         {
             dbgmsg( handshake,
                     "I think peer has sent us a corrupt handshake..." );
-            return tr_handshakeDone( handshake, FALSE );
+            return tr_handshakeDone( handshake, false );
         }
     }
 
     evbuffer_drain( inbuf, 1 );
 
     /* pstr (BitTorrent) */
-    pstr = tr_new( uint8_t, pstrlen + 1 );
+    assert( pstrlen == 19 );
     tr_peerIoReadBytes( handshake->io, inbuf, pstr, pstrlen );
     pstr[pstrlen] = '\0';
-    if( strcmp( (char*)pstr, "BitTorrent protocol" ) )
-    {
-        tr_free( pstr );
-        return tr_handshakeDone( handshake, FALSE );
-    }
-    tr_free( pstr );
+    if( memcmp( pstr, "BitTorrent protocol", 19 ) )
+        return tr_handshakeDone( handshake, false );
 
     /* reserved bytes */
     tr_peerIoReadBytes( handshake->io, inbuf, reserved, sizeof( reserved ) );
@@ -686,7 +670,7 @@ readHandshake( tr_handshake *    handshake,
         if( !tr_torrentExists( handshake->session, hash ) )
         {
             dbgmsg( handshake, "peer is trying to connect to us for a torrent we don't have." );
-            return tr_handshakeDone( handshake, FALSE );
+            return tr_handshakeDone( handshake, false );
         }
         else
         {
@@ -701,7 +685,7 @@ readHandshake( tr_handshake *    handshake,
                     SHA_DIGEST_LENGTH ) )
         {
             dbgmsg( handshake, "peer returned the wrong hash. wtf?" );
-            return tr_handshakeDone( handshake, FALSE );
+            return tr_handshakeDone( handshake, false );
         }
     }
 
@@ -713,7 +697,7 @@ readHandshake( tr_handshake *    handshake,
     {
         uint8_t msg[HANDSHAKE_SIZE];
         buildHandshakeMessage( handshake, msg );
-        tr_peerIoWrite( handshake->io, msg, sizeof( msg ), FALSE );
+        tr_peerIoWriteBytes( handshake->io, msg, sizeof( msg ), false );
         handshake->haveSentBitTorrentHandshake = 1;
     }
 
@@ -725,28 +709,28 @@ static int
 readPeerId( tr_handshake    * handshake,
             struct evbuffer * inbuf )
 {
-    int  peerIsGood;
+    bool peerIsGood;
     char client[128];
     tr_torrent * tor;
     const uint8_t * tor_peer_id;
     uint8_t peer_id[PEER_ID_LEN];
 
-    if( EVBUFFER_LENGTH( inbuf ) < PEER_ID_LEN )
+    if( evbuffer_get_length( inbuf ) < PEER_ID_LEN )
         return READ_LATER;
 
     /* peer id */
     tr_peerIoReadBytes( handshake->io, inbuf, peer_id, PEER_ID_LEN );
     tr_peerIoSetPeersId( handshake->io, peer_id );
-    handshake->havePeerID = TRUE;
+    handshake->havePeerID = true;
     tr_clientForId( client, sizeof( client ), peer_id );
     dbgmsg( handshake, "peer-id is [%s] ... isIncoming is %d", client,
             tr_peerIoIsIncoming( handshake->io ) );
 
     /* if we've somehow connected to ourselves, don't keep the connection */
     tor = tr_torrentFindFromHash( handshake->session, tr_peerIoGetTorrentHash( handshake->io ) );
-    tor_peer_id = tor && tor->peer_id ? tor->peer_id : tr_getPeerId( );
+    tor_peer_id = tor ? tor->peer_id : tr_getPeerId( handshake->session );
     peerIsGood = memcmp( peer_id, tor_peer_id, PEER_ID_LEN ) != 0;
-    dbgmsg( handshake, "isPeerGood == %d", peerIsGood );
+    dbgmsg( handshake, "isPeerGood == %d", (int)peerIsGood );
     return tr_handshakeDone( handshake, peerIsGood );
 }
 
@@ -760,8 +744,8 @@ readYa( tr_handshake *    handshake,
     int            len;
 
     dbgmsg( handshake, "in readYa... need %d, have %zu",
-            (int)KEY_LEN, EVBUFFER_LENGTH( inbuf ) );
-    if( EVBUFFER_LENGTH( inbuf ) < KEY_LEN )
+            KEY_LEN, evbuffer_get_length( inbuf ) );
+    if( evbuffer_get_length( inbuf ) < KEY_LEN )
         return READ_LATER;
 
     /* read the incoming peer's public key */
@@ -781,49 +765,30 @@ readYa( tr_handshake *    handshake,
     walk += len;
 
     setReadState( handshake, AWAITING_PAD_A );
-    tr_peerIoWrite( handshake->io, outbuf, walk - outbuf, FALSE );
+    tr_peerIoWriteBytes( handshake->io, outbuf, walk - outbuf, false );
     return READ_NOW;
 }
 
 static int
-readPadA( tr_handshake *    handshake,
-          struct evbuffer * inbuf )
+readPadA( tr_handshake * handshake, struct evbuffer * inbuf )
 {
-    uint8_t * pch;
+    /* resynchronizing on HASH('req1',S) */
+    struct evbuffer_ptr ptr = evbuffer_search( inbuf, (const char*)handshake->myReq1, SHA_DIGEST_LENGTH, NULL );
 
-    dbgmsg( handshake, "looking to get past pad a... & resync on hash('req',S) ... have %zu bytes",
-            EVBUFFER_LENGTH( inbuf ) );
-    /**
-    *** Resynchronizing on HASH('req1',S)
-    **/
-
-    pch = memchr( EVBUFFER_DATA( inbuf ),
-                 handshake->myReq1[0],
-                 EVBUFFER_LENGTH( inbuf ) );
-    if( pch == NULL )
+    if( ptr.pos != -1 ) /* match */
     {
-        dbgmsg( handshake, "no luck so far.. draining %zu bytes",
-                EVBUFFER_LENGTH( inbuf ) );
-        evbuffer_drain( inbuf, EVBUFFER_LENGTH( inbuf ) );
-        return READ_LATER;
-    }
-    dbgmsg( handshake, "looking for hash('req',S) ... draining %d bytes",
-           (int)( pch - EVBUFFER_DATA( inbuf ) ) );
-    evbuffer_drain( inbuf, pch - EVBUFFER_DATA( inbuf ) );
-    if( EVBUFFER_LENGTH( inbuf ) < SHA_DIGEST_LENGTH )
-        return READ_LATER;
-    if( memcmp( EVBUFFER_DATA( inbuf ), handshake->myReq1,
-                SHA_DIGEST_LENGTH ) )
-    {
-        dbgmsg( handshake, "draining one more byte" );
-        evbuffer_drain( inbuf, 1 );
+        evbuffer_drain( inbuf, ptr.pos );
+        dbgmsg( handshake, "found it... looking setting to awaiting_crypto_provide" );
+        setState( handshake, AWAITING_CRYPTO_PROVIDE );
         return READ_NOW;
     }
-
-    dbgmsg( handshake,
-            "found it... looking setting to awaiting_crypto_provide" );
-    setState( handshake, AWAITING_CRYPTO_PROVIDE );
-    return READ_NOW;
+    else
+    {
+        const size_t len = evbuffer_get_length( inbuf );
+        if( len > SHA_DIGEST_LENGTH )
+            evbuffer_drain( inbuf, len - SHA_DIGEST_LENGTH );
+        return READ_LATER;
+    }
 }
 
 static int
@@ -848,7 +813,7 @@ readCryptoProvide( tr_handshake *    handshake,
                            + sizeof( padc_len );
     tr_torrent * tor = NULL;
 
-    if( EVBUFFER_LENGTH( inbuf ) < needlen )
+    if( evbuffer_get_length( inbuf ) < needlen )
         return READ_LATER;
 
     /* TODO: confirm they sent HASH('req1',S) here? */
@@ -864,8 +829,8 @@ readCryptoProvide( tr_handshake *    handshake,
         obfuscatedTorrentHash[i] = req2[i] ^ req3[i];
     if(( tor = tr_torrentFindFromObfuscatedHash( handshake->session, obfuscatedTorrentHash )))
     {
-        const tr_bool clientIsSeed = tr_torrentIsSeed( tor );
-        const tr_bool peerIsSeed = tr_peerMgrPeerIsSeed( tor, tr_peerIoGetAddress( handshake->io, NULL ) );
+        const bool clientIsSeed = tr_torrentIsSeed( tor );
+        const bool peerIsSeed = tr_peerMgrPeerIsSeed( tor, tr_peerIoGetAddress( handshake->io, NULL ) );
         dbgmsg( handshake, "got INCOMING connection's encrypted handshake for torrent [%s]",
                 tr_torrentName( tor ) );
         tr_peerIoSetTorrentHash( handshake->io, tor->info.hash );
@@ -873,13 +838,13 @@ readCryptoProvide( tr_handshake *    handshake,
         if( clientIsSeed && peerIsSeed )
         {
             dbgmsg( handshake, "another seed tried to reconnect to us!" );
-            return tr_handshakeDone( handshake, FALSE );
+            return tr_handshakeDone( handshake, false );
         }
     }
     else
     {
         dbgmsg( handshake, "can't find that torrent..." );
-        return tr_handshakeDone( handshake, FALSE );
+        return tr_handshakeDone( handshake, false );
     }
 
     /* next part: ENCRYPT(VC, crypto_provide, len(PadC), */
@@ -906,7 +871,7 @@ readPadC( tr_handshake *    handshake,
     uint16_t     ia_len;
     const size_t needlen = handshake->pad_c_len + sizeof( uint16_t );
 
-    if( EVBUFFER_LENGTH( inbuf ) < needlen )
+    if( evbuffer_get_length( inbuf ) < needlen )
         return READ_LATER;
 
     evbuffer_drain( inbuf, handshake->pad_c_len );
@@ -927,8 +892,8 @@ readIA( tr_handshake *    handshake,
     uint32_t          crypto_select;
 
     dbgmsg( handshake, "reading IA... have %zu, need %zu",
-            EVBUFFER_LENGTH( inbuf ), needlen );
-    if( EVBUFFER_LENGTH( inbuf ) < needlen )
+            evbuffer_get_length( inbuf ), needlen );
+    if( evbuffer_get_length( inbuf ) < needlen )
         return READ_LATER;
 
     /**
@@ -943,7 +908,7 @@ readIA( tr_handshake *    handshake,
     {
         uint8_t vc[VC_LENGTH];
         memset( vc, 0, VC_LENGTH );
-        tr_peerIoWriteBytes( handshake->io, outbuf, vc, VC_LENGTH );
+        evbuffer_add( outbuf, vc, VC_LENGTH );
     }
 
     /* send crypto_select */
@@ -951,13 +916,13 @@ readIA( tr_handshake *    handshake,
     if( crypto_select )
     {
         dbgmsg( handshake, "selecting crypto mode '%d'", (int)crypto_select );
-        tr_peerIoWriteUint32( handshake->io, outbuf, crypto_select );
+        evbuffer_add_uint32( outbuf, crypto_select );
     }
     else
     {
         dbgmsg( handshake, "peer didn't offer an encryption mode we like." );
         evbuffer_free( outbuf );
-        return tr_handshakeDone( handshake, FALSE );
+        return tr_handshakeDone( handshake, false );
     }
 
     dbgmsg( handshake, "sending pad d" );
@@ -965,14 +930,14 @@ readIA( tr_handshake *    handshake,
      * PadD is reserved for future extensions to the handshake...
      * standard practice at this time is for it to be zero-length */
     {
-        const int len = 0;
-        tr_peerIoWriteUint16( handshake->io, outbuf, len );
+        const uint16_t len = 0;
+        evbuffer_add_uint16( outbuf, len );
     }
 
     /* maybe de-encrypt our connection */
     if( crypto_select == CRYPTO_PROVIDE_PLAINTEXT )
     {
-        tr_peerIoWriteBuf( handshake->io, outbuf, FALSE );
+        tr_peerIoWriteBuf( handshake->io, outbuf, false );
         tr_peerIoSetEncryption( handshake->io, PEER_ENCRYPTION_NONE );
     }
 
@@ -982,12 +947,12 @@ readIA( tr_handshake *    handshake,
         uint8_t msg[HANDSHAKE_SIZE];
         buildHandshakeMessage( handshake, msg );
 
-        tr_peerIoWriteBytes( handshake->io, outbuf, msg, sizeof( msg ) );
+        evbuffer_add( outbuf, msg, sizeof( msg ) );
         handshake->haveSentBitTorrentHandshake = 1;
     }
 
     /* send it out */
-    tr_peerIoWriteBuf( handshake->io, outbuf, FALSE );
+    tr_peerIoWriteBuf( handshake->io, outbuf, false );
     evbuffer_free( outbuf );
 
     /* now await the handshake */
@@ -1003,18 +968,18 @@ readPayloadStream( tr_handshake    * handshake,
     const size_t      needlen = HANDSHAKE_SIZE;
 
     dbgmsg( handshake, "reading payload stream... have %zu, need %zu",
-            EVBUFFER_LENGTH( inbuf ), needlen );
-    if( EVBUFFER_LENGTH( inbuf ) < needlen )
+            evbuffer_get_length( inbuf ), needlen );
+    if( evbuffer_get_length( inbuf ) < needlen )
         return READ_LATER;
 
     /* parse the handshake ... */
     i = parseHandshake( handshake, inbuf );
     dbgmsg( handshake, "parseHandshake returned %d", i );
     if( i != HANDSHAKE_OK )
-        return tr_handshakeDone( handshake, FALSE );
+        return tr_handshakeDone( handshake, false );
 
     /* we've completed the BT handshake... pass the work on to peer-msgs */
-    return tr_handshakeDone( handshake, TRUE );
+    return tr_handshakeDone( handshake, true );
 }
 
 /***
@@ -1029,7 +994,7 @@ canRead( struct tr_peerIo * io, void * arg, size_t * piece )
     tr_handshake    * handshake = arg;
     struct evbuffer * inbuf = tr_peerIoGetReadBuffer( io );
     ReadState         ret;
-    tr_bool           readyForMore = TRUE;
+    bool              readyForMore = true;
 
     assert( tr_isPeerIo( io ) );
 
@@ -1084,31 +1049,30 @@ canRead( struct tr_peerIo * io, void * arg, size_t * piece )
         }
 
         if( ret != READ_NOW )
-            readyForMore = FALSE;
+            readyForMore = false;
         else if( handshake->state == AWAITING_PAD_C )
-            readyForMore = EVBUFFER_LENGTH( inbuf ) >= handshake->pad_c_len;
+            readyForMore = evbuffer_get_length( inbuf ) >= handshake->pad_c_len;
         else if( handshake->state == AWAITING_PAD_D )
-            readyForMore = EVBUFFER_LENGTH( inbuf ) >= handshake->pad_d_len;
+            readyForMore = evbuffer_get_length( inbuf ) >= handshake->pad_d_len;
         else if( handshake->state == AWAITING_IA )
-            readyForMore = EVBUFFER_LENGTH( inbuf ) >= handshake->ia_len;
+            readyForMore = evbuffer_get_length( inbuf ) >= handshake->ia_len;
     }
 
     return ret;
 }
 
-static int
-fireDoneFunc( tr_handshake * handshake,
-              tr_bool        isConnected )
+static bool
+fireDoneFunc( tr_handshake * handshake, bool isConnected )
 {
     const uint8_t * peer_id = isConnected && handshake->havePeerID
                             ? tr_peerIoGetPeersId( handshake->io )
                             : NULL;
-    const int success = ( *handshake->doneCB )( handshake,
-                                                handshake->io,
-                                                handshake->haveReadAnythingFromPeer,
-                                                isConnected,
-                                                peer_id,
-                                                handshake->doneUserData );
+    const bool success = ( *handshake->doneCB )( handshake,
+                                                 handshake->io,
+                                                 handshake->haveReadAnythingFromPeer,
+                                                 isConnected,
+                                                 peer_id,
+                                                 handshake->doneUserData );
 
     return success;
 }
@@ -1119,16 +1083,14 @@ tr_handshakeFree( tr_handshake * handshake )
     if( handshake->io )
         tr_peerIoUnref( handshake->io ); /* balanced by the ref in tr_handshakeNew */
 
-    evtimer_del( &handshake->timeout_timer );
-
+    event_free( handshake->timeout_timer );
     tr_free( handshake );
 }
 
 static int
-tr_handshakeDone( tr_handshake * handshake,
-                  tr_bool        isOK )
+tr_handshakeDone( tr_handshake * handshake, bool isOK )
 {
-    tr_bool success;
+    bool success;
 
     dbgmsg( handshake, "handshakeDone: %s", isOK ? "connected" : "aborting" );
     tr_peerIoSetIOFuncs( handshake->io, NULL, NULL, NULL, NULL );
@@ -1144,16 +1106,42 @@ void
 tr_handshakeAbort( tr_handshake * handshake )
 {
     if( handshake != NULL )
-        tr_handshakeDone( handshake, FALSE );
+        tr_handshakeDone( handshake, false );
 }
 
 static void
-gotError( tr_peerIo  * io UNUSED,
+gotError( tr_peerIo  * io,
           short        what,
           void       * vhandshake )
 {
+    int errcode = errno;
     tr_handshake * handshake = vhandshake;
 
+    if( io->utp_socket && !io->isIncoming && handshake->state == AWAITING_YB ) {
+        /* This peer probably doesn't speak uTP. */
+        tr_torrent *tor =
+            tr_peerIoHasTorrentHash( io ) ?
+            tr_torrentFindFromHash( handshake->session,
+                                    tr_peerIoGetTorrentHash( io ) ) :
+            NULL;
+        /* Don't mark a peer as non-uTP unless it's really a connect failure. */
+        if( tor && ( errcode == ETIMEDOUT || errcode == ECONNREFUSED ) ) {
+            tr_torrentLock( tor );
+            tr_peerMgrSetUtpFailed( tor,
+                                    tr_peerIoGetAddress( io, NULL ),
+                                    true );
+            tr_torrentUnlock( tor );
+        }
+
+        if( !tr_peerIoReconnect( handshake->io ) ) {
+            uint8_t msg[HANDSHAKE_SIZE];
+            buildHandshakeMessage( handshake, msg );
+            handshake->haveSentBitTorrentHandshake = 1;
+            setReadState( handshake, AWAITING_HANDSHAKE );
+            tr_peerIoWriteBytes( handshake->io, msg, sizeof( msg ), false );
+        }
+    }
+
     /* if the error happened while we were sending a public key, we might
      * have encountered a peer that doesn't do encryption... reconnect and
      * try a plaintext handshake */
@@ -1168,13 +1156,13 @@ gotError( tr_peerIo  * io UNUSED,
         buildHandshakeMessage( handshake, msg );
         handshake->haveSentBitTorrentHandshake = 1;
         setReadState( handshake, AWAITING_HANDSHAKE );
-        tr_peerIoWrite( handshake->io, msg, sizeof( msg ), FALSE );
+        tr_peerIoWriteBytes( handshake->io, msg, sizeof( msg ), false );
     }
     else
     {
         dbgmsg( handshake, "libevent got an error what==%d, errno=%d (%s)",
                (int)what, errno, tr_strerror( errno ) );
-        tr_handshakeDone( handshake, FALSE );
+        tr_handshakeDone( handshake, false );
     }
 }
 
@@ -1189,12 +1177,13 @@ handshakeTimeout( int foo UNUSED, short bar UNUSED, void * handshake )
 }
 
 tr_handshake*
-tr_handshakeNew( tr_peerIo *        io,
-                 tr_encryption_mode encryptionMode,
-                 handshakeDoneCB    doneCB,
-                 void *             doneUserData )
+tr_handshakeNew( tr_peerIo           * io,
+                 tr_encryption_mode    encryptionMode,
+                 handshakeDoneCB       doneCB,
+                 void                * doneUserData )
 {
     tr_handshake * handshake;
+    tr_session * session = tr_peerIoGetSession( io );
 
     handshake = tr_new0( tr_handshake, 1 );
     handshake->io = io;
@@ -1202,10 +1191,9 @@ tr_handshakeNew( tr_peerIo *        io,
     handshake->encryptionMode = encryptionMode;
     handshake->doneCB = doneCB;
     handshake->doneUserData = doneUserData;
-    handshake->session = tr_peerIoGetSession( io );
-
-    evtimer_set( &handshake->timeout_timer, handshakeTimeout, handshake );
-    tr_timerAdd( &handshake->timeout_timer, HANDSHAKE_TIMEOUT_SEC, 0 );
+    handshake->session = session;
+    handshake->timeout_timer = evtimer_new( session->event_base, handshakeTimeout, handshake );
+    tr_timerAdd( handshake->timeout_timer, HANDSHAKE_TIMEOUT_SEC, 0 );
 
     tr_peerIoRef( io ); /* balanced by the unref in tr_handshakeFree */
     tr_peerIoSetIOFuncs( handshake->io, canRead, NULL, gotError, handshake );
@@ -1222,7 +1210,7 @@ tr_handshakeNew( tr_peerIo *        io,
 
         handshake->haveSentBitTorrentHandshake = 1;
         setReadState( handshake, AWAITING_HANDSHAKE );
-        tr_peerIoWrite( handshake->io, msg, sizeof( msg ), FALSE );
+        tr_peerIoWriteBytes( handshake->io, msg, sizeof( msg ), false );
     }
 
     return handshake;
