@@ -1,59 +1,61 @@
-/******************************************************************************
- * $Id: metainfo.c 9829 2009-12-28 00:35:27Z charles $
+/*
+ * This file Copyright (C) Mnemosyne LLC
  *
- * Copyright (c) 2005-2008 Transmission authors and contributors
+ * This file is licensed by the GPL version 2. Works owned by the
+ * Transmission project are granted a special exemption to clause 2(b)
+ * so that the bulk of its code can remain under the MIT license.
+ * This exemption does not extend to derived works not owned by
+ * the Transmission project.
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- *****************************************************************************/
+ * $Id: metainfo.c 12416 2011-05-05 03:10:51Z jordan $
+ */
 
 #include <assert.h>
 #include <errno.h>
-#include <stdio.h>
-#include <string.h>
+#include <stdio.h> /* fopen(), fwrite(), fclose() */
+#include <string.h> /* strlen() */
 
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h> /* unlink, stat */
 
-#include <event.h> /* struct evbuffer */
+#include <event2/buffer.h>
 
 #include "transmission.h"
 #include "session.h"
 #include "bencode.h"
 #include "crypto.h" /* tr_sha1 */
 #include "metainfo.h"
-#include "platform.h"
+#include "platform.h" /* tr_getTorrentDir() */
 #include "utils.h"
 
 /***
 ****
 ***/
 
-static char*
-getTorrentFilename( const tr_session * session,
-                    const tr_info *   inf )
+char*
+tr_metainfoGetBasename( const tr_info * inf )
 {
-    return tr_strdup_printf( "%s%c%s.%16.16s.torrent",
-                             tr_getTorrentDir( session ),
-                             TR_PATH_DELIMITER,
-                             inf->name,
-                             inf->hashString );
+    size_t i;
+    const size_t name_len = strlen( inf->name );
+    char * ret = tr_strdup_printf( "%s.%16.16s", inf->name, inf->hashString );
+
+    for( i=0; i<name_len; ++i )
+        if( ret[i] == '/' )
+            ret[i] = '_';
+
+
+    return ret;
+}
+
+static char*
+getTorrentFilename( const tr_session * session, const tr_info * inf )
+{
+    char * base = tr_metainfoGetBasename( inf );
+    char * filename = tr_strdup_printf( "%s" TR_PATH_DELIMITER_STR "%s.torrent",
+                                        tr_getTorrentDir( session ), base );
+    tr_free( base );
+    return filename;
 }
 
 static char*
@@ -135,24 +137,24 @@ tr_metainfoMigrate( tr_session * session,
 ****
 ***/
 
-static tr_bool
+static bool
 path_is_suspicious( const char * path )
 {
     return ( path == NULL )
         || ( strstr( path, "../" ) != NULL );
 }
 
-static tr_bool
-getfile( char ** setme, const char * root, tr_benc * path )
+static bool
+getfile( char ** setme, const char * root, tr_benc * path, struct evbuffer * buf )
 {
-    tr_bool success = FALSE;
+    bool success = false;
 
     if( tr_bencIsList( path ) )
     {
         int i;
         const int n = tr_bencListSize( path );
-        struct evbuffer * buf = evbuffer_new( );
 
+        evbuffer_drain( buf, evbuffer_get_length( buf ) );
         evbuffer_add( buf, root, strlen( root ) );
         for( i = 0; i < n; ++i )
         {
@@ -164,26 +166,23 @@ getfile( char ** setme, const char * root, tr_benc * path )
             }
         }
 
-        *setme = tr_utf8clean( (char*)EVBUFFER_DATA( buf ), EVBUFFER_LENGTH( buf ), NULL );
+        *setme = tr_utf8clean( (char*)evbuffer_pullup( buf, -1 ), evbuffer_get_length( buf ) );
         /* fprintf( stderr, "[%s]\n", *setme ); */
-        evbuffer_free( buf );
-        success = TRUE;
+        success = true;
     }
 
     if( ( *setme != NULL ) && path_is_suspicious( *setme ) )
     {
         tr_free( *setme );
         *setme = NULL;
-        success = FALSE;
+        success = false;
     }
 
     return success;
 }
 
 static const char*
-parseFiles( tr_info *       inf,
-            tr_benc *       files,
-            const tr_benc * length )
+parseFiles( tr_info * inf, tr_benc * files, const tr_benc * length )
 {
     int64_t len;
 
@@ -192,6 +191,7 @@ parseFiles( tr_info *       inf,
     if( tr_bencIsList( files ) ) /* multi-file mode */
     {
         tr_file_index_t i;
+        struct evbuffer * buf = evbuffer_new( );
 
         inf->isMultifile = 1;
         inf->fileCount   = tr_bencListSize( files );
@@ -210,7 +210,7 @@ parseFiles( tr_info *       inf,
                 if( !tr_bencDictFindList( file, "path", &path ) )
                     return "path";
 
-            if( !getfile( &inf->files[i].name, inf->name, path ) )
+            if( !getfile( &inf->files[i].name, inf->name, path, buf ) )
                 return "path";
 
             if( !tr_bencDictFindInt( file, "length", &len ) )
@@ -219,6 +219,8 @@ parseFiles( tr_info *       inf,
             inf->files[i].length = len;
             inf->totalSize      += len;
         }
+
+        evbuffer_free( buf );
     }
     else if( tr_bencGetInt( length, &len ) ) /* single-file mode */
     {
@@ -241,7 +243,7 @@ parseFiles( tr_info *       inf,
 }
 
 static char *
-announceToScrape( const char * announce )
+tr_convertAnnounceToScrape( const char * announce )
 {
     char *       scrape = NULL;
     const char * s;
@@ -251,7 +253,7 @@ announceToScrape( const char * announce )
      * If the text immediately following that '/' isn't 'announce'
      * it will be taken as a sign that that tracker doesn't support
      * the scrape convention. If it does, substitute 'scrape' for
-     * 'announce' to find the scrape page.  */
+     * 'announce' to find the scrape page. */
     if( ( ( s = strrchr( announce, '/' ) ) ) && !strncmp( ++s, "announce", 8 ) )
     {
         const char * prefix = announce;
@@ -271,8 +273,7 @@ announceToScrape( const char * announce )
 }
 
 static const char*
-getannounce( tr_info * inf,
-             tr_benc * meta )
+getannounce( tr_info * inf, tr_benc * meta )
 {
     const char *      str;
     tr_tracker_info * trackers = NULL;
@@ -291,28 +292,29 @@ getannounce( tr_info * inf,
             n += tr_bencListSize( tr_bencListChild( tiers, i ) );
 
         trackers = tr_new0( tr_tracker_info, n );
-        trackerCount = 0;
 
         for( i = 0, validTiers = 0; i < numTiers; ++i )
         {
             tr_benc * tier = tr_bencListChild( tiers, i );
             const int tierSize = tr_bencListSize( tier );
-            tr_bool anyAdded = FALSE;
+            bool anyAdded = false;
             for( j = 0; j < tierSize; ++j )
             {
                 if( tr_bencGetStr( tr_bencListChild( tier, j ), &str ) )
                 {
                     char * url = tr_strstrip( tr_strdup( str ) );
-                    if( tr_httpIsValidURL( url ) )
-                    {
-                        tr_tracker_info * t = trackers + trackerCount++;
+                    if( !tr_urlIsValidTracker( url ) )
+                        tr_free( url );
+                    else {
+                        tr_tracker_info * t = trackers + trackerCount;
                         t->tier = validTiers;
-                        t->announce = tr_strdup( url );
-                        t->scrape = announceToScrape( url );
+                        t->announce = url;
+                        t->scrape = tr_convertAnnounceToScrape( url );
+                        t->id = trackerCount;
 
-                        anyAdded = TRUE;
+                        anyAdded = true;
+                        ++trackerCount;
                     }
-                    tr_free( url );
                 }
             }
 
@@ -333,21 +335,23 @@ getannounce( tr_info * inf,
       && tr_bencDictFindStr( meta, "announce", &str ) )
     {
         char * url = tr_strstrip( tr_strdup( str ) );
-        if( tr_httpIsValidURL( url ) )
-        {
+        if( !tr_urlIsValidTracker( url ) )
+            tr_free( url );
+        else {
             trackers = tr_new0( tr_tracker_info, 1 );
             trackers[trackerCount].tier = 0;
-            trackers[trackerCount].announce = tr_strdup( url );
-            trackers[trackerCount++].scrape = announceToScrape( url );
+            trackers[trackerCount].announce = url;
+            trackers[trackerCount].scrape = tr_convertAnnounceToScrape( url );
+            trackers[trackerCount].id = 0;
+            trackerCount++;
             /*fprintf( stderr, "single announce: [%s]\n", url );*/
         }
-        tr_free( url );
     }
 
     inf->trackers = trackers;
     inf->trackerCount = trackerCount;
 
-    return inf->trackerCount ? NULL : "announce";
+    return NULL;
 }
 
 static void
@@ -355,71 +359,129 @@ geturllist( tr_info * inf,
             tr_benc * meta )
 {
     tr_benc * urls;
+    const char * url;
 
     if( tr_bencDictFindList( meta, "url-list", &urls ) )
     {
         int          i;
-        const char * url;
         const int    n = tr_bencListSize( urls );
 
         inf->webseedCount = 0;
         inf->webseeds = tr_new0( char*, n );
 
         for( i = 0; i < n; ++i )
+        {
             if( tr_bencGetStr( tr_bencListChild( urls, i ), &url ) )
-                inf->webseeds[inf->webseedCount++] = tr_strdup( url );
+            {
+                const size_t len = strlen( url );
+
+                if( tr_urlIsValid( url, len  ) )
+                    inf->webseeds[inf->webseedCount++] = tr_strndup( url, len );
+            }
+        }
+    }
+    else if( tr_bencDictFindStr( meta, "url-list", &url ) ) /* handle single items in webseeds */
+    {
+        const size_t len = strlen( url );
+
+        if( tr_urlIsValid( url, len  ) )
+        {
+            inf->webseedCount = 1;
+            inf->webseeds = tr_new0( char*, 1 );
+            inf->webseeds[0] = tr_strndup( url, len );
+        }
     }
 }
 
 static const char*
-tr_metainfoParseImpl( const tr_session * session,
-                      tr_info *         inf,
-                      const tr_benc *   meta_in )
+tr_metainfoParseImpl( const tr_session  * session,
+                      tr_info           * inf,
+                      bool              * hasInfoDict,
+                      int               * infoDictLength,
+                      const tr_benc     * meta_in )
 {
     int64_t         i;
     size_t          raw_len;
     const char *    str;
     const uint8_t * raw;
-    tr_benc *       beInfo;
+    tr_benc *       d;
+    tr_benc *       infoDict = NULL;
     tr_benc *       meta = (tr_benc *) meta_in;
-    tr_bool         err;
+    bool            b;
+    bool            isMagnet = false;
 
     /* info_hash: urlencoded 20-byte SHA1 hash of the value of the info key
      * from the Metainfo file. Note that the value will be a bencoded
      * dictionary, given the definition of the info key above. */
-    if( !tr_bencDictFindDict( meta, "info", &beInfo ) )
-        return "info";
+    b = tr_bencDictFindDict( meta, "info", &infoDict );
+    if( hasInfoDict != NULL )
+        *hasInfoDict = b;
+    if( !b )
+    {
+        /* no info dictionary... is this a magnet link? */
+        if( tr_bencDictFindDict( meta, "magnet-info", &d ) )
+        {
+            isMagnet = true;
+
+            /* get the info-hash */
+            if( !tr_bencDictFindRaw( d, "info_hash", &raw, &raw_len ) )
+                return "info_hash";
+            if( raw_len != SHA_DIGEST_LENGTH )
+                return "info_hash";
+            memcpy( inf->hash, raw, raw_len );
+            tr_sha1_to_hex( inf->hashString, inf->hash );
+
+            /* maybe get the display name */
+            if( tr_bencDictFindStr( d, "display-name", &str ) ) {
+                tr_free( inf->name );
+                inf->name = tr_strdup( str );
+            }
+
+            if( !inf->name )
+                inf->name = tr_strdup( inf->hashString );
+        }
+        else /* not a magnet link and has no info dict... */
+        {
+            return "info";
+        }
+    }
     else
     {
         int len;
-        char * bstr = tr_bencToStr( beInfo, TR_FMT_BENC, &len );
+        char * bstr = tr_bencToStr( infoDict, TR_FMT_BENC, &len );
         tr_sha1( inf->hash, bstr, len, NULL );
         tr_sha1_to_hex( inf->hashString, inf->hash );
+
+        if( infoDictLength != NULL )
+            *infoDictLength = len;
+
         tr_free( bstr );
     }
 
     /* name */
-    if( !tr_bencDictFindStr( beInfo, "name.utf-8", &str ) )
-        if( !tr_bencDictFindStr( beInfo, "name", &str ) )
-            str = "";
-    if( !str || !*str )
-        return "name";
-    tr_free( inf->name );
-    inf->name = tr_utf8clean( str, -1, &err );
+    if( !isMagnet ) {
+        if( !tr_bencDictFindStr( infoDict, "name.utf-8", &str ) )
+            if( !tr_bencDictFindStr( infoDict, "name", &str ) )
+                str = "";
+        if( !str || !*str )
+            return "name";
+        tr_free( inf->name );
+        inf->name = tr_utf8clean( str, -1 );
+    }
 
     /* comment */
     if( !tr_bencDictFindStr( meta, "comment.utf-8", &str ) )
         if( !tr_bencDictFindStr( meta, "comment", &str ) )
             str = "";
     tr_free( inf->comment );
-    inf->comment = tr_utf8clean( str, -1, &err );
+    inf->comment = tr_utf8clean( str, -1 );
 
     /* created by */
     if( !tr_bencDictFindStr( meta, "created by.utf-8", &str ) )
         if( !tr_bencDictFindStr( meta, "created by", &str ) )
             str = "";
     tr_free( inf->creator );
-    inf->creator = tr_utf8clean( str, -1, &err );
+    inf->creator = tr_utf8clean( str, -1 );
 
     /* creation date */
     if( !tr_bencDictFindInt( meta, "creation date", &i ) )
@@ -427,35 +489,42 @@ tr_metainfoParseImpl( const tr_session * session,
     inf->dateCreated = i;
 
     /* private */
-    if( !tr_bencDictFindInt( beInfo, "private", &i ) )
+    if( !tr_bencDictFindInt( infoDict, "private", &i ) )
         if( !tr_bencDictFindInt( meta, "private", &i ) )
             i = 0;
     inf->isPrivate = i != 0;
 
     /* piece length */
-    if( !tr_bencDictFindInt( beInfo, "piece length", &i ) || ( i < 1 ) )
-        return "piece length";
-    inf->pieceSize = i;
+    if( !isMagnet ) {
+        if( !tr_bencDictFindInt( infoDict, "piece length", &i ) || ( i < 1 ) )
+            return "piece length";
+        inf->pieceSize = i;
+    }
 
     /* pieces */
-    if( !tr_bencDictFindRaw( beInfo, "pieces", &raw,
-                             &raw_len ) || ( raw_len % SHA_DIGEST_LENGTH ) )
-        return "pieces";
-    inf->pieceCount = raw_len / SHA_DIGEST_LENGTH;
-    inf->pieces = tr_new0( tr_piece, inf->pieceCount );
-    for( i = 0; i < inf->pieceCount; ++i )
-        memcpy( inf->pieces[i].hash, &raw[i * SHA_DIGEST_LENGTH],
-                SHA_DIGEST_LENGTH );
+    if( !isMagnet ) {
+        if( !tr_bencDictFindRaw( infoDict, "pieces", &raw, &raw_len ) )
+            return "pieces";
+        if( raw_len % SHA_DIGEST_LENGTH )
+            return "pieces";
+        inf->pieceCount = raw_len / SHA_DIGEST_LENGTH;
+        inf->pieces = tr_new0( tr_piece, inf->pieceCount );
+        for( i = 0; i < inf->pieceCount; ++i )
+            memcpy( inf->pieces[i].hash, &raw[i * SHA_DIGEST_LENGTH],
+                    SHA_DIGEST_LENGTH );
+    }
 
     /* files */
-    if( ( str = parseFiles( inf, tr_bencDictFind( beInfo, "files" ),
-                                 tr_bencDictFind( beInfo, "length" ) ) ) )
-        return str;
-    if( !inf->fileCount || !inf->totalSize )
-        return "files";
-    if( (uint64_t) inf->pieceCount !=
-       ( inf->totalSize + inf->pieceSize - 1 ) / inf->pieceSize )
-        return "files";
+    if( !isMagnet ) {
+        if( ( str = parseFiles( inf, tr_bencDictFind( infoDict, "files" ),
+                                     tr_bencDictFind( infoDict, "length" ) ) ) )
+            return str;
+        if( !inf->fileCount || !inf->totalSize )
+            return "files";
+        if( (uint64_t) inf->pieceCount !=
+           ( inf->totalSize + inf->pieceSize - 1 ) / inf->pieceSize )
+            return "files";
+    }
 
     /* get announce or announce-list */
     if( ( str = getannounce( inf, meta ) ) )
@@ -471,13 +540,19 @@ tr_metainfoParseImpl( const tr_session * session,
     return NULL;
 }
 
-tr_bool
+bool
 tr_metainfoParse( const tr_session * session,
+                  const tr_benc    * meta_in,
                   tr_info          * inf,
-                  const tr_benc    * meta_in )
+                  bool             * hasInfoDict,
+                  int              * infoDictLength )
 {
-    const char * badTag = tr_metainfoParseImpl( session, inf, meta_in );
-    const tr_bool success = badTag == NULL;
+    const char * badTag = tr_metainfoParseImpl( session,
+                                                inf,
+                                                hasInfoDict,
+                                                infoDictLength,
+                                                meta_in );
+    const bool success = badTag == NULL;
 
     if( badTag )
     {
@@ -491,8 +566,8 @@ tr_metainfoParse( const tr_session * session,
 void
 tr_metainfoFree( tr_info * inf )
 {
+    int i;
     tr_file_index_t ff;
-    int             i;
 
     for( i = 0; i < inf->webseedCount; ++i )
         tr_free( inf->webseeds[i] );
@@ -519,8 +594,7 @@ tr_metainfoFree( tr_info * inf )
 }
 
 void
-tr_metainfoRemoveSaved( const tr_session * session,
-                        const tr_info *   inf )
+tr_metainfoRemoveSaved( const tr_session * session, const tr_info * inf )
 {
     char * filename;
 
@@ -532,4 +606,3 @@ tr_metainfoRemoveSaved( const tr_session * session,
     unlink( filename );
     tr_free( filename );
 }
-
