@@ -1,26 +1,27 @@
 /*
- * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: natpmp.c 9870 2010-01-04 22:15:01Z charles $
+ * $Id: natpmp.c 12225 2011-03-24 22:57:39Z jordan $
  */
 
-#include <assert.h>
 #include <errno.h>
 #include <time.h>
 #include <inttypes.h>
 
+#include <event2/util.h> /* evutil_inet_ntop() */
+
 #define ENABLE_STRNATPMPERR
 #include <libnatpmp/natpmp.h>
 
 #include "transmission.h"
 #include "natpmp.h"
-#include "net.h" /* inet_ntoa() */
+#include "net.h" /* tr_netCloseSocket */
 #include "port-forwarding.h"
 #include "utils.h"
 
@@ -45,13 +46,16 @@ tr_natpmp_state;
 
 struct tr_natpmp
 {
-    tr_bool            isMapped;
-    tr_bool            hasDiscovered;
-    int                port;
-    time_t             renewTime;
-    time_t             commandTime;
-    tr_natpmp_state    state;
-    natpmp_t           natpmp;
+    bool              has_discovered;
+    bool              is_mapped;
+
+    tr_port           public_port;
+    tr_port           private_port;
+
+    time_t            renew_time;
+    time_t            command_time;
+    tr_natpmp_state   state;
+    natpmp_t          natpmp;
 };
 
 /**
@@ -69,7 +73,7 @@ logVal( const char * func,
     else
         tr_ndbg(
              getKey( ),
-            "%s failed.  natpmp returned %d (%s); errno is %d (%s)",
+            "%s failed. Natpmp returned %d (%s); errno is %d (%s)",
             func, ret, strnatpmperr( ret ), errno, tr_strerror( errno ) );
 }
 
@@ -80,7 +84,8 @@ tr_natpmpInit( void )
 
     nat = tr_new0( struct tr_natpmp, 1 );
     nat->state = TR_NATPMP_DISCOVER;
-    nat->port = -1;
+    nat->public_port = 0;
+    nat->private_port = 0;
     nat->natpmp.s = -1; /* socket */
     return nat;
 }
@@ -91,7 +96,7 @@ tr_natpmpClose( tr_natpmp * nat )
     if( nat )
     {
         if( nat->natpmp.s >= 0 )
-            tr_netClose( nat->natpmp.s );
+            tr_netCloseSocket( nat->natpmp.s );
         tr_free( nat );
     }
 }
@@ -99,44 +104,41 @@ tr_natpmpClose( tr_natpmp * nat )
 static int
 canSendCommand( const struct tr_natpmp * nat )
 {
-    return time( NULL ) >= nat->commandTime;
+    return tr_time( ) >= nat->command_time;
 }
 
 static void
 setCommandTime( struct tr_natpmp * nat )
 {
-    nat->commandTime = time( NULL ) + COMMAND_WAIT_SECS;
+    nat->command_time = tr_time( ) + COMMAND_WAIT_SECS;
 }
 
 int
-tr_natpmpPulse( struct tr_natpmp * nat,
-                int                port,
-                int                isEnabled )
+tr_natpmpPulse( struct tr_natpmp * nat, tr_port private_port, bool is_enabled, tr_port * public_port )
 {
     int ret;
 
-    if( isEnabled && ( nat->state == TR_NATPMP_DISCOVER ) )
+    if( is_enabled && ( nat->state == TR_NATPMP_DISCOVER ) )
     {
         int val = initnatpmp( &nat->natpmp );
         logVal( "initnatpmp", val );
         val = sendpublicaddressrequest( &nat->natpmp );
         logVal( "sendpublicaddressrequest", val );
         nat->state = val < 0 ? TR_NATPMP_ERR : TR_NATPMP_RECV_PUB;
-        nat->hasDiscovered = 1;
+        nat->has_discovered = true;
         setCommandTime( nat );
     }
 
     if( ( nat->state == TR_NATPMP_RECV_PUB ) && canSendCommand( nat ) )
     {
         natpmpresp_t response;
-        const int    val = readnatpmpresponseorretry( &nat->natpmp,
-                                                      &response );
+        const int val = readnatpmpresponseorretry( &nat->natpmp, &response );
         logVal( "readnatpmpresponseorretry", val );
         if( val >= 0 )
         {
-            tr_ninf( getKey( ), _(
-                        "Found public address \"%s\"" ),
-                    inet_ntoa( response.pnu.publicaddress.addr ) );
+            char str[128];
+            evutil_inet_ntop( AF_INET, &response.pnu.publicaddress.addr, str, sizeof( str ) );
+            tr_ninf( getKey( ), _( "Found public address \"%s\"" ), str );
             nat->state = TR_NATPMP_IDLE;
         }
         else if( val != NATPMP_TRYAGAIN )
@@ -147,16 +149,16 @@ tr_natpmpPulse( struct tr_natpmp * nat,
 
     if( ( nat->state == TR_NATPMP_IDLE ) || ( nat->state == TR_NATPMP_ERR ) )
     {
-        if( nat->isMapped && ( !isEnabled || ( nat->port != port ) ) )
+        if( nat->is_mapped && ( !is_enabled || ( nat->private_port != private_port ) ) )
             nat->state = TR_NATPMP_SEND_UNMAP;
     }
 
     if( ( nat->state == TR_NATPMP_SEND_UNMAP ) && canSendCommand( nat ) )
     {
-        const int val =
-            sendnewportmappingrequest( &nat->natpmp, NATPMP_PROTOCOL_TCP,
-                                       nat->port, nat->port,
-                                       0 );
+        const int val = sendnewportmappingrequest( &nat->natpmp, NATPMP_PROTOCOL_TCP,
+                                                   nat->private_port,
+                                                   nat->public_port,
+                                                   0 );
         logVal( "sendnewportmappingrequest", val );
         nat->state = val < 0 ? TR_NATPMP_ERR : TR_NATPMP_RECV_UNMAP;
         setCommandTime( nat );
@@ -165,17 +167,20 @@ tr_natpmpPulse( struct tr_natpmp * nat,
     if( nat->state == TR_NATPMP_RECV_UNMAP )
     {
         natpmpresp_t resp;
-        const int    val = readnatpmpresponseorretry( &nat->natpmp, &resp );
+        const int val = readnatpmpresponseorretry( &nat->natpmp, &resp );
         logVal( "readnatpmpresponseorretry", val );
         if( val >= 0 )
         {
-            const int p = resp.pnu.newportmapping.privateport;
-            tr_ninf( getKey( ), _( "no longer forwarding port %d" ), p );
-            if( nat->port == p )
+            const int private_port = resp.pnu.newportmapping.privateport;
+
+            tr_ninf( getKey( ), _( "no longer forwarding port %d" ), private_port );
+
+            if( nat->private_port == private_port )
             {
-                nat->port = -1;
+                nat->private_port = 0;
+                nat->public_port = 0;
                 nat->state = TR_NATPMP_IDLE;
-                nat->isMapped = 0;
+                nat->is_mapped = false;
             }
         }
         else if( val != NATPMP_TRYAGAIN )
@@ -186,20 +191,16 @@ tr_natpmpPulse( struct tr_natpmp * nat,
 
     if( nat->state == TR_NATPMP_IDLE )
     {
-        if( isEnabled && !nat->isMapped && nat->hasDiscovered )
+        if( is_enabled && !nat->is_mapped && nat->has_discovered )
             nat->state = TR_NATPMP_SEND_MAP;
 
-        else if( nat->isMapped && time( NULL ) >= nat->renewTime )
+        else if( nat->is_mapped && tr_time( ) >= nat->renew_time )
             nat->state = TR_NATPMP_SEND_MAP;
     }
 
     if( ( nat->state == TR_NATPMP_SEND_MAP ) && canSendCommand( nat ) )
     {
-        const int val =
-            sendnewportmappingrequest( &nat->natpmp, NATPMP_PROTOCOL_TCP,
-                                       port,
-                                       port,
-                                       LIFETIME_SECS );
+        const int val = sendnewportmappingrequest( &nat->natpmp, NATPMP_PROTOCOL_TCP, private_port, private_port, LIFETIME_SECS );
         logVal( "sendnewportmappingrequest", val );
         nat->state = val < 0 ? TR_NATPMP_ERR : TR_NATPMP_RECV_MAP;
         setCommandTime( nat );
@@ -213,11 +214,11 @@ tr_natpmpPulse( struct tr_natpmp * nat,
         if( val >= 0 )
         {
             nat->state = TR_NATPMP_IDLE;
-            nat->isMapped = 1;
-            nat->renewTime = time( NULL ) + LIFETIME_SECS;
-            nat->port = resp.pnu.newportmapping.privateport;
-            tr_ninf( getKey( ), _(
-                         "Port %d forwarded successfully" ), nat->port );
+            nat->is_mapped = true;
+            nat->renew_time = tr_time( ) + ( resp.pnu.newportmapping.lifetime / 2 );
+            nat->private_port = resp.pnu.newportmapping.privateport;
+            nat->public_port = resp.pnu.newportmapping.mappedpublicport;
+            tr_ninf( getKey( ), _( "Port %d forwarded successfully" ), nat->private_port );
         }
         else if( val != NATPMP_TRYAGAIN )
         {
@@ -228,7 +229,9 @@ tr_natpmpPulse( struct tr_natpmp * nat,
     switch( nat->state )
     {
         case TR_NATPMP_IDLE:
-            ret = nat->isMapped ? TR_PORT_MAPPED : TR_PORT_UNMAPPED; break;
+            *public_port = nat->public_port;
+            return nat->is_mapped ? TR_PORT_MAPPED : TR_PORT_UNMAPPED;
+            break;
 
         case TR_NATPMP_DISCOVER:
             ret = TR_PORT_UNMAPPED; break;
