@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: net.c 10753 2010-06-14 23:14:58Z charles $
+ * $Id: net.c 12300 2011-04-02 16:43:17Z jch $
  *
- * Copyright (c) 2005-2008 Transmission authors and contributors
+ * Copyright (c) Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -24,37 +24,29 @@
  *****************************************************************************/
 
 #include <errno.h>
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 #include <assert.h>
 
 #include <sys/types.h>
 
 #ifdef WIN32
- #include <winsock2.h> /* inet_addr */
- #include <WS2tcpip.h>
+ #define _WIN32_WINNT   0x0501
+ #include <ws2tcpip.h>
 #else
- #include <sys/socket.h>
- #include <netinet/in.h>
- #include <netinet/tcp.h>
- #include <arpa/inet.h> /* inet_addr */
- #include <netdb.h>
- #include <fcntl.h>
+ #include <netinet/tcp.h>       /* TCP_CONGESTION */
 #endif
-#include <unistd.h>
 
-#include <stdarg.h> /* some 1.4.x versions of evutil.h need this */
-#include <evutil.h>
+#include <event2/util.h>
+
+#include <libutp/utp.h>
 
 #include "transmission.h"
-#include "fdlimit.h"
-#include "natpmp.h"
+#include "fdlimit.h" /* tr_fdSocketClose() */
 #include "net.h"
-#include "peer-io.h"
-#include "platform.h"
-#include "session.h"
-#include "utils.h"
+#include "peer-io.h" /* tr_peerIoAddrStr() FIXME this should be moved to net.h */
+#include "session.h" /* tr_sessionGetPublicAddress() */
+#include "tr-utp.h" /* tr_utpSendTo() */
+#include "utils.h" /* tr_time(), tr_dbg() */
 
 #ifndef IN_MULTICAST
 #define IN_MULTICAST( a ) ( ( ( a ) & 0xf0000000 ) == 0xe0000000 )
@@ -63,65 +55,10 @@
 const tr_address tr_in6addr_any = { TR_AF_INET6, { IN6ADDR_ANY_INIT } };
 const tr_address tr_inaddr_any = { TR_AF_INET, { { { { INADDR_ANY, 0x00, 0x00, 0x00 } } } } };
 
-#ifdef WIN32
-static const char *
-inet_ntop( int af, const void *src, char *dst, socklen_t cnt )
-{
-    if (af == AF_INET)
-    {
-        struct sockaddr_in in;
-        memset( &in, 0, sizeof( in ) );
-        in.sin_family = AF_INET;
-        memcpy( &in.sin_addr, src, sizeof( struct in_addr ) );
-        getnameinfo((struct sockaddr *)&in, sizeof(struct sockaddr_in),
-                    dst, cnt, NULL, 0, NI_NUMERICHOST);
-        return dst;
-    }
-    else if (af == AF_INET6)
-    {
-        struct sockaddr_in6 in;
-        memset( &in, 0, sizeof( in ) );
-        in.sin6_family = AF_INET6;
-        memcpy( &in.sin6_addr, src, sizeof( struct in_addr6 ) );
-        getnameinfo((struct sockaddr *)&in, sizeof(struct sockaddr_in6),
-                    dst, cnt, NULL, 0, NI_NUMERICHOST);
-        return dst;
-    }
-    return NULL;
-}
-
-static int
-inet_pton(int af, const char *src, void *dst)
-{
-    struct addrinfo hints;
-    struct addrinfo *res;
-    struct addrinfo *ressave;
-
-    memset(&hints, 0, sizeof(struct addrinfo));
-    hints.ai_family = af;
-
-    if (getaddrinfo(src, NULL, &hints, &res) != 0)
-        return -1;
-
-    ressave = res;
-
-    while (res)
-    {
-        memcpy(dst, res->ai_addr, res->ai_addrlen);
-        res = res->ai_next;
-    }
-
-    freeaddrinfo(ressave);
-    return 0;
-}
-
-#endif
-
-
 void
 tr_netInit( void )
 {
-    static int initialized = FALSE;
+    static int initialized = false;
 
     if( !initialized )
     {
@@ -129,52 +66,59 @@ tr_netInit( void )
         WSADATA wsaData;
         WSAStartup( MAKEWORD( 2, 2 ), &wsaData );
 #endif
-        initialized = TRUE;
+        initialized = true;
     }
 }
 
-const char *
-tr_ntop( const tr_address * src, char * dst, int size )
+char *
+tr_net_strerror( char * buf, size_t buflen, int err )
 {
-    assert( tr_isAddress( src ) );
+    *buf = '\0';
+#ifdef WIN32
+    FormatMessage( FORMAT_MESSAGE_FROM_SYSTEM, NULL, err, 0, buf, buflen, NULL );
+#else
+    tr_strlcpy( buf, tr_strerror( err ), buflen );
+#endif
+    return buf;
+}
 
-    if( src->type == TR_AF_INET )
-        return inet_ntop( AF_INET, &src->addr, dst, size );
+const char *
+tr_address_to_string_with_buf( const tr_address * addr, char * buf, size_t buflen )
+{
+    assert( tr_address_is_valid( addr ) );
+
+    if( addr->type == TR_AF_INET )
+        return evutil_inet_ntop( AF_INET, &addr->addr, buf, buflen );
     else
-        return inet_ntop( AF_INET6, &src->addr, dst, size );
+        return evutil_inet_ntop( AF_INET6, &addr->addr, buf, buflen );
 }
 
 /*
- * Non-threadsafe version of tr_ntop, which uses a static memory area for a buffer.
+ * Non-threadsafe version of tr_address_to_string_with_buf()
+ * and uses a static memory area for a buffer.
  * This function is suitable to be called from libTransmission's networking code,
  * which is single-threaded.
  */
 const char *
-tr_ntop_non_ts( const tr_address * src )
+tr_address_to_string( const tr_address * addr )
 {
     static char buf[INET6_ADDRSTRLEN];
-    return tr_ntop( src, buf, sizeof( buf ) );
+    return tr_address_to_string_with_buf( addr, buf, sizeof( buf ) );
 }
 
-tr_address *
-tr_pton( const char * src, tr_address * dst )
+bool
+tr_address_from_string( tr_address * dst, const char * src )
 {
-    int retval = inet_pton( AF_INET, src, &dst->addr );
-    assert( dst );
-    if( retval < 0 )
-        return NULL;
-    else if( retval == 0 )
-        retval = inet_pton( AF_INET6, src, &dst->addr );
-    else
-    {
-        dst->type = TR_AF_INET;
-        return dst;
-    }
+    bool ok;
 
-    if( retval < 1 )
-        return NULL;
-    dst->type = TR_AF_INET6;
-    return dst;
+    if(( ok = evutil_inet_pton( AF_INET, src, &dst->addr ) == 1 ))
+        dst->type = TR_AF_INET;
+
+    if( !ok ) /* try IPv6 */
+        if(( ok = evutil_inet_pton( AF_INET6, src, &dst->addr ) == 1 ))
+            dst->type = TR_AF_INET6;
+
+    return ok;
 }
 
 /*
@@ -185,7 +129,7 @@ tr_pton( const char * src, tr_address * dst )
  * 0  if a == b
  */
 int
-tr_compareAddresses( const tr_address * a, const tr_address * b)
+tr_address_compare( const tr_address * a, const tr_address * b)
 {
     static const int sizes[2] = { sizeof(struct in_addr), sizeof(struct in6_addr) };
 
@@ -211,7 +155,7 @@ tr_netSetTOS( int s, int tos )
 }
 
 int
-tr_netSetCongestionControl( int s, const char *algorithm )
+tr_netSetCongestionControl( int s UNUSED, const char *algorithm UNUSED )
 {
 #ifdef TCP_CONGESTION
     return setsockopt( s, IPPROTO_TCP, TCP_CONGESTION,
@@ -222,12 +166,38 @@ tr_netSetCongestionControl( int s, const char *algorithm )
 #endif
 }
 
+bool
+tr_address_from_sockaddr_storage( tr_address                     * setme_addr,
+                                  tr_port                        * setme_port,
+                                  const struct sockaddr_storage  * from )
+{
+    if( from->ss_family == AF_INET )
+    {
+        struct sockaddr_in * sin = (struct sockaddr_in *)from;
+        setme_addr->type = TR_AF_INET;
+        setme_addr->addr.addr4.s_addr = sin->sin_addr.s_addr;
+        *setme_port = sin->sin_port;
+        return true;
+    }
+
+    if( from->ss_family == AF_INET6 )
+    {
+        struct sockaddr_in6 *sin6 = (struct sockaddr_in6*) from;
+        setme_addr->type = TR_AF_INET6;
+        setme_addr->addr.addr6 = sin6->sin6_addr;
+        *setme_port = sin6->sin6_port;
+        return true;
+    }
+
+    return false;
+}
+
 static socklen_t
 setup_sockaddr( const tr_address        * addr,
                 tr_port                   port,
                 struct sockaddr_storage * sockaddr)
 {
-    assert( tr_isAddress( addr ) );
+    assert( tr_address_is_valid( addr ) );
 
     if( addr->type == TR_AF_INET )
     {
@@ -256,7 +226,7 @@ int
 tr_netOpenPeerSocket( tr_session        * session,
                       const tr_address  * addr,
                       tr_port             port,
-                      tr_bool             clientIsSeed )
+                      bool                clientIsSeed )
 {
     static const int domains[NUM_TR_AF_INET_TYPES] = { AF_INET, AF_INET6 };
     int                     s;
@@ -266,9 +236,9 @@ tr_netOpenPeerSocket( tr_session        * session,
     socklen_t               sourcelen;
     struct sockaddr_storage source_sock;
 
-    assert( tr_isAddress( addr ) );
+    assert( tr_address_is_valid( addr ) );
 
-    if( !tr_isValidPeerAddress( addr, port ) )
+    if( !tr_address_is_valid_for_peers( addr, port ) )
         return -EINVAL;
 
     s = tr_fdSocketCreate( session, domains[addr->type], SOCK_STREAM );
@@ -290,13 +260,13 @@ tr_netOpenPeerSocket( tr_session        * session,
     addrlen = setup_sockaddr( addr, port, &sock );
 
     /* set source address */
-    source_addr = tr_sessionGetPublicAddress( session, addr->type );
+    source_addr = tr_sessionGetPublicAddress( session, addr->type, NULL );
     assert( source_addr );
     sourcelen = setup_sockaddr( source_addr, 0, &source_sock );
     if( bind( s, ( struct sockaddr * ) &source_sock, sourcelen ) )
     {
         tr_err( _( "Couldn't set source address %s on %d: %s" ),
-                tr_ntop_non_ts( source_addr ), s, tr_strerror( errno ) );
+                tr_address_to_string( source_addr ), s, tr_strerror( errno ) );
         return -errno;
     }
 
@@ -312,7 +282,7 @@ tr_netOpenPeerSocket( tr_session        * session,
         if( ( tmperrno != ENETUNREACH && tmperrno != EHOSTUNREACH )
                 || addr->type == TR_AF_INET )
             tr_err( _( "Couldn't connect socket %d to %s, port %d (errno %d - %s)" ),
-                    s, tr_ntop_non_ts( addr ), (int)port, tmperrno,
+                    s, tr_address_to_string( addr ), (int)ntohs( port ), tmperrno,
                     tr_strerror( tmperrno ) );
         tr_netClose( session, s );
         s = -tmperrno;
@@ -324,8 +294,22 @@ tr_netOpenPeerSocket( tr_session        * session,
     return s;
 }
 
+struct UTPSocket *
+tr_netOpenPeerUTPSocket( tr_session        * session,
+                         const tr_address  * addr,
+                         tr_port             port,
+                         bool                clientIsSeed UNUSED )
+{
+    struct sockaddr_storage ss;
+    socklen_t sslen;
+    sslen = setup_sockaddr( addr, port, &ss );
+
+    return UTP_Create( tr_utpSendTo, (void*)session,
+                       (struct sockaddr*)&ss, sslen );
+}
+
 static int
-tr_netBindTCPImpl( const tr_address * addr, tr_port port, tr_bool suppressMsgs, int * errOut )
+tr_netBindTCPImpl( const tr_address * addr, tr_port port, bool suppressMsgs, int * errOut )
 {
     static const int domains[NUM_TR_AF_INET_TYPES] = { AF_INET, AF_INET6 };
     struct sockaddr_storage sock;
@@ -333,7 +317,7 @@ tr_netBindTCPImpl( const tr_address * addr, tr_port port, tr_bool suppressMsgs,
     int addrlen;
     int optval;
 
-    assert( tr_isAddress( addr ) );
+    assert( tr_address_is_valid( addr ) );
 
     fd = socket( domains[addr->type], SOCK_STREAM, 0 );
     if( fd < 0 ) {
@@ -378,7 +362,7 @@ tr_netBindTCPImpl( const tr_address * addr, tr_port port, tr_bool suppressMsgs,
             else
                 fmt = _( "Couldn't bind port %d on %s: %s (%s)" );
 
-            tr_err( fmt, port, tr_ntop_non_ts( addr ), tr_strerror( err ), hint );
+            tr_err( fmt, port, tr_address_to_string( addr ), tr_strerror( err ), hint );
         }
         tr_netCloseSocket( fd );
         *errOut = err;
@@ -386,7 +370,7 @@ tr_netBindTCPImpl( const tr_address * addr, tr_port port, tr_bool suppressMsgs,
     }
 
     if( !suppressMsgs )
-        tr_dbg( "Bound socket %d to port %d on %s", fd, port, tr_ntop_non_ts( addr ) );
+        tr_dbg( "Bound socket %d to port %d on %s", fd, port, tr_address_to_string( addr ) );
 
     if( listen( fd, 128 ) == -1 ) {
         *errOut = sockerrno;
@@ -398,27 +382,27 @@ tr_netBindTCPImpl( const tr_address * addr, tr_port port, tr_bool suppressMsgs,
 }
 
 int
-tr_netBindTCP( const tr_address * addr, tr_port port, tr_bool suppressMsgs )
+tr_netBindTCP( const tr_address * addr, tr_port port, bool suppressMsgs )
 {
     int unused;
     return tr_netBindTCPImpl( addr, port, suppressMsgs, &unused );
 }
 
-tr_bool
+bool
 tr_net_hasIPv6( tr_port port )
 {
-    static tr_bool result = FALSE;
-    static tr_bool alreadyDone = FALSE;
+    static bool result = false;
+    static bool alreadyDone = false;
 
     if( !alreadyDone )
     {
         int err;
-        int fd = tr_netBindTCPImpl( &tr_in6addr_any, port, TRUE, &err );
+        int fd = tr_netBindTCPImpl( &tr_in6addr_any, port, true, &err );
         if( fd >= 0 || err != EAFNOSUPPORT ) /* we support ipv6 */
-            result = TRUE;
+            result = true;
         if( fd >= 0 )
             tr_netCloseSocket( fd );
-        alreadyDone = TRUE;
+        alreadyDone = true;
     }
 
     return result;
@@ -443,7 +427,7 @@ tr_netAccept( tr_session  * session,
 void
 tr_netCloseSocket( int fd )
 {
-    EVUTIL_CLOSESOCKET( fd );
+    evutil_closesocket( fd );
 }
 
 void
@@ -458,7 +442,7 @@ tr_netClose( tr_session * session, int s )
    Please feel free to copy them into your software if it can help
    unbreaking the double-stack Internet. */
 
-/* Get the source address used for a given destination address.  Since
+/* Get the source address used for a given destination address. Since
    there is no official interface to get this information, we create
    a connected UDP socket (connected UDP... hmm...) and check its source
    address. */
@@ -483,13 +467,13 @@ get_source_address( const struct sockaddr  * dst,
     if(rc < 0)
         goto fail;
 
-    EVUTIL_CLOSESOCKET( s );
+    evutil_closesocket( s );
 
     return rc;
 
  fail:
     save = errno;
-    EVUTIL_CLOSESOCKET( s );
+    evutil_closesocket( s );
     errno = save;
     return -1;
 }
@@ -532,7 +516,7 @@ tr_globalAddress( int af, void *addr, int *addr_len )
     case AF_INET:
         memset(&sin, 0, sizeof(sin));
         sin.sin_family = AF_INET;
-        inet_pton(AF_INET, "91.121.74.28", &sin.sin_addr);
+        evutil_inet_pton(AF_INET, "91.121.74.28", &sin.sin_addr);
         sin.sin_port = htons(6969);
         sa = (struct sockaddr*)&sin;
         salen = sizeof(sin);
@@ -542,7 +526,7 @@ tr_globalAddress( int af, void *addr, int *addr_len )
         sin6.sin6_family = AF_INET6;
         /* In order for address selection to work right, this should be
            a native IPv6 address, not Teredo or 6to4. */
-        inet_pton(AF_INET6, "2001:1890:1112:1::20", &sin6.sin6_addr);
+        evutil_inet_pton(AF_INET6, "2001:1890:1112:1::20", &sin6.sin6_addr);
         sin6.sin6_port = htons(6969);
         sa = (struct sockaddr*)&sin6;
         salen = sizeof(sin6);
@@ -604,13 +588,13 @@ tr_globalIPv6( void )
 ****
 ***/
 
-static tr_bool
+static bool
 isIPv4MappedAddress( const tr_address * addr )
 {
     return ( addr->type == TR_AF_INET6 ) && IN6_IS_ADDR_V4MAPPED( &addr->addr.addr6 );
 }
 
-static tr_bool
+static bool
 isIPv6LinkLocalAddress( const tr_address * addr )
 {
     return ( ( addr->type == TR_AF_INET6 )
@@ -619,13 +603,13 @@ isIPv6LinkLocalAddress( const tr_address * addr )
 
 /* isMartianAddr was written by Juliusz Chroboczek,
    and is covered under the same license as third-party/dht/dht.c. */
-static tr_bool
+static bool
 isMartianAddr( const struct tr_address * a )
 {
     static const unsigned char zeroes[16] =
         { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
 
-    assert( tr_isAddress( a ) );
+    assert( tr_address_is_valid( a ) );
 
     switch( a->type )
     {
@@ -643,7 +627,7 @@ isMartianAddr( const struct tr_address * a )
                    (memcmp(address, zeroes, 15) == 0 &&
                     (address[15] == 0 || address[15] == 1)) ||
                    /* Addresses outside of 2000::/3 are currently reserved,
-                      but might be allocated at some future time.  Since
+                      but might be allocated at some future time. Since
                       there are a lot of buggy peers pushing around such
                       addresses over PEX, we reject them until the end of
                       the 13th Baktun. */
@@ -652,15 +636,15 @@ isMartianAddr( const struct tr_address * a )
         }
 
         default:
-            return TRUE;
+            return true;
     }
 }
 
-tr_bool
-tr_isValidPeerAddress( const tr_address * addr, tr_port port )
+bool
+tr_address_is_valid_for_peers( const tr_address * addr, tr_port port )
 {
     return ( port != 0 )
-        && ( tr_isAddress( addr ) )
+        && ( tr_address_is_valid( addr ) )
         && ( !isIPv6LinkLocalAddress( addr ) )
         && ( !isIPv4MappedAddress( addr ) )
         && ( !isMartianAddr( addr ) );
