@@ -1,16 +1,18 @@
 /*
- * This file Copyright (C) 2008-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: platform.c 8929 2009-08-14 12:53:08Z charles $
+ * $Id: platform.c 12478 2011-05-30 18:17:16Z jordan $
  */
 
 #ifdef WIN32
+ #include <w32api.h>
+ #define WINVER  WindowsXP
  #include <windows.h>
  #include <shlobj.h> /* for CSIDL_APPDATA, CSIDL_MYDOCUMENTS */
 #else
@@ -32,8 +34,16 @@
 #include <stdlib.h>
 #include <string.h>
 
+#ifdef SYS_DARWIN
+ #define HAVE_SYS_STATVFS_H
+ #define HAVE_STATVFS
+#endif
+
 #include <sys/stat.h>
 #include <sys/types.h>
+#ifdef HAVE_SYS_STATVFS_H
+ #include <sys/statvfs.h>
+#endif
 #ifdef WIN32
 #include <libgen.h>
 #endif
@@ -67,17 +77,17 @@ tr_getCurrentThread( void )
 #endif
 }
 
-static int
-tr_areThreadsEqual( tr_thread_id a,
-                    tr_thread_id b )
+static bool
+tr_areThreadsEqual( tr_thread_id a, tr_thread_id b )
 {
 #ifdef WIN32
     return a == b;
 #else
-    return pthread_equal( a, b );
+    return pthread_equal( a, b ) != 0;
 #endif
 }
 
+/** @brief portability wrapper around OS-dependent threads */
 struct tr_thread
 {
     void            ( * func )( void * );
@@ -88,7 +98,7 @@ struct tr_thread
 #endif
 };
 
-int
+bool
 tr_amInThread( const tr_thread * t )
 {
     return tr_areThreadsEqual( tr_getCurrentThread( ), t->thread );
@@ -107,6 +117,7 @@ ThreadFunc( void * _t )
 
     t->func( t->arg );
 
+    tr_free( t );
 #ifdef WIN32
     _endthreadex( 0 );
     return 0;
@@ -131,8 +142,9 @@ tr_threadNew( void   ( *func )(void *),
         t->thread = (DWORD) id;
     }
 #else
-    pthread_create( &t->thread, NULL, ( void * ( * )(
-                                           void * ) )ThreadFunc, t );
+    pthread_create( &t->thread, NULL, (void*(*)(void*))ThreadFunc, t );
+    pthread_detach( t->thread );
+
 #endif
 
     return t;
@@ -142,6 +154,7 @@ tr_threadNew( void   ( *func )(void *),
 ****  LOCKS
 ***/
 
+/** @brief portability wrapper around OS-dependent thread mutexes */
 struct tr_lock
 {
     int                 depth;
@@ -274,7 +287,7 @@ getOldConfigDir( void )
         SHGetFolderPath( NULL, CSIDL_APPDATA, NULL, 0, appdata );
         path = tr_buildPath( appdata, "Transmission", NULL );
 #elif defined( __HAIKU__ )
-        char buf[MAX_PATH_LENGTH];
+        char buf[TR_PATH_MAX];
         find_directory( B_USER_SETTINGS_DIRECTORY, -1, true, buf, sizeof(buf) );
         path = tr_buildPath( buf, "Transmission", NULL );
 #else
@@ -336,11 +349,11 @@ moveFiles( const char * oldDir,
             struct dirent * dirp;
             while( ( dirp = readdir( dirh ) ) )
             {
-                if( strcmp( dirp->d_name,
-                            "." ) && strcmp( dirp->d_name, ".." ) )
+                const char * name = dirp->d_name;
+                if( name && strcmp( name, "." ) && strcmp( name, ".." ) )
                 {
-                    char * o = tr_buildPath( oldDir, dirp->d_name, NULL );
-                    char * n = tr_buildPath( newDir, dirp->d_name, NULL );
+                    char * o = tr_buildPath( oldDir, name, NULL );
+                    char * n = tr_buildPath( newDir, name, NULL );
                     rename( o, n );
                     ++count;
                     tr_free( n );
@@ -356,16 +369,22 @@ moveFiles( const char * oldDir,
     }
 }
 
+/**
+ * This function is for transmission-gtk users to migrate the config files
+ * from $HOME/.transmission/ (where they were kept before Transmission 1.30)
+ * to $HOME/.config/$appname as per the XDG directory spec.
+ */
 static void
 migrateFiles( const tr_session * session )
 {
-    static int migrated = FALSE;
+    static int migrated = false;
+    const bool should_migrate = strstr( getOldConfigDir(), ".transmission" ) != NULL;
 
-    if( !migrated )
+    if( !migrated && should_migrate )
     {
         const char * oldDir;
         const char * newDir;
-        migrated = TRUE;
+        migrated = true;
 
         oldDir = getOldTorrentsDir( );
         newDir = tr_getTorrentDir( session );
@@ -378,8 +397,7 @@ migrateFiles( const tr_session * session )
 }
 
 void
-tr_setConfigDir( tr_session * session,
-                 const char * configDir )
+tr_setConfigDir( tr_session * session, const char * configDir )
 {
     char * path;
 
@@ -434,11 +452,11 @@ tr_getDefaultConfigDir( const char * appname )
             s = tr_buildPath( getHomeDir( ), "Library", "Application Support",
                               appname, NULL );
 #elif defined( WIN32 )
-            char appdata[MAX_PATH]; /* SHGetFolderPath() requires MAX_PATH */
+            char appdata[TR_PATH_MAX]; /* SHGetFolderPath() requires MAX_PATH */
             SHGetFolderPath( NULL, CSIDL_APPDATA, NULL, 0, appdata );
             s = tr_buildPath( appdata, appname, NULL );
 #elif defined( __HAIKU__ )
-            char buf[MAX_PATH_LENGTH];
+            char buf[TR_PATH_MAX];
             find_directory( B_USER_SETTINGS_DIRECTORY, -1, true, buf, sizeof(buf) );
             s = tr_buildPath( buf, appname, NULL );
 #else
@@ -514,10 +532,10 @@ tr_getDefaultDownloadDir( void )
 ***/
 
 static int
-isClutchDir( const char * path )
+isWebClientDir( const char * path )
 {
     struct stat sb;
-    char * tmp = tr_buildPath( path, "javascript", "transmission.js", NULL );
+    char * tmp = tr_buildPath( path, "index.html", NULL );
     const int ret = !stat( tmp, &sb );
     tr_inf( _( "Searching for web interface file \"%s\"" ), tmp );
     tr_free( tmp );
@@ -525,7 +543,7 @@ isClutchDir( const char * path )
 }
 
 const char *
-tr_getClutchDir( const tr_session * session UNUSED )
+tr_getWebClientDir( const tr_session * session UNUSED )
 {
     static char * s = NULL;
 
@@ -542,27 +560,38 @@ tr_getClutchDir( const tr_session * session UNUSED )
         else
         {
 
-#ifdef SYS_DARWIN /* on Mac, look in the app package first, then the Application Support folder (for daemon, etc) */
+#ifdef SYS_DARWIN /* on Mac, look in the Application Support folder first, then in the app bundle. */
 
-            CFURLRef appURL = CFBundleCopyBundleURL( CFBundleGetMainBundle( ) );
-            CFStringRef appRef = CFURLCopyFileSystemPath( appURL,
-                                                         kCFURLPOSIXPathStyle );
-            const char * appString = CFStringGetCStringPtr( appRef,
-                                         CFStringGetFastestEncoding( appRef ) );
-            CFRelease( appURL );
-            CFRelease( appRef );
+            /* Look in the Application Support folder */
+            s = tr_buildPath( tr_sessionGetConfigDir( session ), "web", NULL );
 
-            s = tr_buildPath( appString, "Contents", "Resources", "web", NULL );
-
-            if( !isClutchDir( s ) ) {
+            if( !isWebClientDir( s ) ) {
                 tr_free( s );
 
-                /* Fallback to the Application Support folder */
-                s = tr_buildPath( tr_sessionGetConfigDir( session ), "web", NULL );
-                if( !isClutchDir( s ) ) {
+                CFURLRef appURL = CFBundleCopyBundleURL( CFBundleGetMainBundle( ) );
+                CFStringRef appRef = CFURLCopyFileSystemPath( appURL,
+                                                              kCFURLPOSIXPathStyle );
+                CFIndex appLength = CFStringGetMaximumSizeForEncoding( CFStringGetLength(appRef),
+                                                                       CFStringGetFastestEncoding( appRef ));
+
+                char * appString = tr_malloc( appLength + 1 );
+                bool success = CFStringGetCString( appRef,
+                                              appString,
+                                              appLength + 1,
+                                              CFStringGetFastestEncoding( appRef ));
+                assert( success );
+
+                CFRelease( appURL );
+                CFRelease( appRef );
+
+                /* Fallback to the app bundle */
+                s = tr_buildPath( appString, "Contents", "Resources", "web", NULL );
+                if( !isWebClientDir( s ) ) {
                     tr_free( s );
                     s = NULL;
                 }
+
+                tr_free( appString );
             }
 
 #elif defined( WIN32 )
@@ -577,7 +606,7 @@ tr_getClutchDir( const tr_session * session UNUSED )
                 /* First, we should check personal AppData/Transmission/Web */
                 SHGetFolderPath( NULL, CSIDL_COMMON_APPDATA, NULL, 0, dir );
                 s = tr_buildPath( dir, "Transmission", "Web", NULL );
-                if( !isClutchDir( s ) ) {
+                if( !isWebClientDir( s ) ) {
                     tr_free( s );
                     s = NULL;
                 }
@@ -587,7 +616,7 @@ tr_getClutchDir( const tr_session * session UNUSED )
                 /* check personal AppData */
                 SHGetFolderPath( NULL, CSIDL_APPDATA, NULL, 0, dir );
                 s = tr_buildPath( dir, "Transmission", "Web", NULL );
-                if( !isClutchDir( s ) ) {
+                if( !isWebClientDir( s ) ) {
                     tr_free( s );
                     s = NULL;
                 }
@@ -597,7 +626,7 @@ tr_getClutchDir( const tr_session * session UNUSED )
                 /* check calling module place */
                 GetModuleFileName( GetModuleHandle( NULL ), dir, sizeof( dir ) );
                 s = tr_buildPath( dirname( dir ), "Web", NULL );
-                if( !isClutchDir( s ) ) {
+                if( !isWebClientDir( s ) ) {
                     tr_free( s );
                     s = NULL;
                 }
@@ -641,7 +670,7 @@ tr_getClutchDir( const tr_session * session UNUSED )
             /* walk through the candidates & look for a match */
             for( l=candidates; l; l=l->next ) {
                 char * path = tr_buildPath( l->data, "transmission", "web", NULL );
-                const int found = isClutchDir( path );
+                const int found = isWebClientDir( path );
                 if( found ) {
                     s = path;
                     break;
@@ -663,6 +692,27 @@ tr_getClutchDir( const tr_session * session UNUSED )
 ****
 ***/
 
+int64_t
+tr_getFreeSpace( const char * path )
+{
+#ifdef WIN32
+    uint64_t freeBytesAvailable = 0;
+    return GetDiskFreeSpaceEx( path, &freeBytesAvailable, NULL, NULL)
+        ? (int64_t)freeBytesAvailable
+        : -1;
+#elif defined(HAVE_STATVFS)
+    struct statvfs buf;
+    return statvfs( path, &buf ) ? -1 : (int64_t)buf.f_bavail * (int64_t)buf.f_frsize;
+#else
+    #warning FIXME: not implemented
+    return -1;
+#endif
+}
+
+/***
+****
+***/
+
 #ifdef WIN32
 
 /* The following mmap functions are by Joerg Walter, and were taken from
@@ -782,4 +832,3 @@ munmap_exit:
 }
 
 #endif
-
