@@ -1,40 +1,51 @@
 /*
-Copyright (c) 2009 by Juliusz Chroboczek
-
-Permission is hereby granted, free of charge, to any person obtaining a copy
-of this software and associated documentation files (the "Software"), to deal
-in the Software without restriction, including without limitation the rights
-to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-copies of the Software, and to permit persons to whom the Software is
-furnished to do so, subject to the following conditions:
-
-The above copyright notice and this permission notice shall be included in
-all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
-AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-THE SOFTWARE.
-*/
+ * Copyright (c) 2009-2010 by Juliusz Chroboczek
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+ * THE SOFTWARE.
+ *
+ * $Id: tr-dht.c 12543 2011-07-12 12:26:24Z jordan $
+ *
+ */
 
 /* ansi */
 #include <errno.h>
 #include <stdio.h>
+#include <string.h> /* memcpy(), memset(), memchr(), strlen() */
+#include <stdlib.h> /* atoi() */
 
 /* posix */
-#include <netinet/in.h> /* sockaddr_in */
 #include <signal.h> /* sig_atomic_t */
 #include <sys/time.h>
-#include <sys/types.h>
-#include <sys/socket.h> /* socket(), bind() */
-#include <netdb.h>
 #include <unistd.h> /* close() */
+#ifdef WIN32
+  #include <inttypes.h>
+  #define _WIN32_WINNT  0x0501	/* freeaddrinfo(),getaddrinfo(),getnameinfo() */
+  #include <ws2tcpip.h>
+#else
+  #include <sys/types.h>
+  #include <sys/socket.h> /* socket(), bind() */
+  #include <netdb.h>
+  #include <netinet/in.h> /* sockaddr_in */
+#endif
 
 /* third party */
-#include <event.h>
+#include <event2/event.h>
 #include <dht/dht.h>
 
 /* libT */
@@ -50,13 +61,11 @@ THE SOFTWARE.
 #include "trevent.h" /* tr_runInEventThread() */
 #include "utils.h"
 
-static int dht_socket = -1, dht6_socket = -1;
-static struct event dht_event, dht6_event;
-static tr_port dht_port;
+static struct event *dht_timer = NULL;
 static unsigned char myid[20];
 static tr_session *session = NULL;
 
-static void event_callback(int s, short type, void *ignore);
+static void timer_callback(int s, short type, void *ignore);
 
 struct bootstrap_closure {
     tr_session *session;
@@ -80,12 +89,11 @@ bootstrap_done( tr_session *session, int af )
 }
 
 static void
-nap( int roughly )
+nap( int roughly_sec )
 {
-    struct timeval tv;
-    tv.tv_sec = roughly / 2 + tr_cryptoWeakRandInt( roughly );
-    tv.tv_usec = tr_cryptoWeakRandInt( 1000000 );
-    select( 0, NULL, NULL, NULL, &tv );
+    const int roughly_msec = roughly_sec * 1000;
+    const int msec = roughly_msec/2 + tr_cryptoWeakRandInt(roughly_msec);
+    tr_wait_msec( msec );
 }
 
 static int
@@ -173,7 +181,7 @@ dht_bootstrap(void *closure)
         }
 
         /* Our DHT code is able to take up to 9 nodes in a row without
-           dropping any.  After that, it takes some time to split buckets.
+           dropping any. After that, it takes some time to split buckets.
            So ping the first 8 nodes quickly, then slow down. */
         if(i < 8)
             nap(2);
@@ -192,7 +200,7 @@ dht_bootstrap(void *closure)
             tr_buildPath(cl->session->configDir, "dht.bootstrap", NULL);
 
         if(bootstrap_file)
-            f = fopen(bootstrap_file, "r");
+            f = fopen(bootstrap_file, "rb");
         if(f != NULL) {
             tr_ninf("DHT", "Attempting manual bootstrap");
             for(;;) {
@@ -219,20 +227,27 @@ dht_bootstrap(void *closure)
                 if(bootstrap_done(cl->session, 0))
                     break;
             }
+            fclose( f );
         }
 
         tr_free( bootstrap_file );
     }
 
-    /* We really don't want to abuse our bootstrap nodes.
-       Be glacially slow. */
-    if(!bootstrap_done(cl->session, 0))
-        nap(30);
-
     if(!bootstrap_done(cl->session, 0)) {
-        tr_ninf("DHT", "Attempting bootstrap from dht.transmissionbt.com");
-        bootstrap_from_name( "dht.transmissionbt.com", 6881,
-                             bootstrap_af(session) );
+        for(i = 0; i < 6; i++) {
+            /* We don't want to abuse our bootstrap nodes, so be very
+               slow.  The initial wait is to give other nodes a chance
+               to contact us before we attempt to contact a bootstrap
+               node, for example because we've just been restarted. */
+            nap(40);
+            if(bootstrap_done(cl->session, 0))
+                break;
+            if(i == 0)
+                tr_ninf("DHT",
+                        "Attempting bootstrap from dht.transmissionbt.com");
+            bootstrap_from_name( "dht.transmissionbt.com", 6881,
+                                 bootstrap_af(session) );
+        }
     }
 
     if( cl->nodes )
@@ -243,75 +258,12 @@ dht_bootstrap(void *closure)
     tr_ndbg( "DHT", "Finished bootstrapping" );
 }
 
-/* BEP-32 has a rather nice explanation of why we need to bind to one
-   IPv6 address, if I may say so myself. */
-
-static int
-rebind_ipv6(tr_bool force)
-{
-    struct sockaddr_in6 sin6;
-    const unsigned char *ipv6 = tr_globalIPv6();
-    static unsigned char *last_bound = NULL;
-    int s, rc;
-    int one = 1;
-
-    /* We currently have no way to enable or disable IPv6 once the DHT has
-       been initialised.  Oh, well. */
-    if(ipv6 == NULL || (!force && dht6_socket < 0)) {
-        if(last_bound) {
-            free(last_bound);
-            last_bound = NULL;
-        }
-        return 0;
-    }
-
-    if(last_bound != NULL && memcmp(ipv6, last_bound, 16) == 0)
-        return 0;
-
-    s = socket(PF_INET6, SOCK_DGRAM, 0);
-    if(s < 0)
-        return -1;
-
-#ifdef IPV6_V6ONLY
-        /* Since we always open an IPv4 socket on the same port, this
-           shouldn't matter.  But I'm superstitious. */
-        setsockopt(s, IPPROTO_IPV6, IPV6_V6ONLY, &one, sizeof(one));
-#endif
-
-    memset(&sin6, 0, sizeof(sin6));
-    sin6.sin6_family = AF_INET6;
-    if(ipv6)
-        memcpy(&sin6.sin6_addr, ipv6, 16);
-    sin6.sin6_port = htons(dht_port);
-    rc = bind(s, (struct sockaddr*)&sin6, sizeof(sin6));
-
-    if(rc < 0)
-        return -1;
-
-    if(dht6_socket < 0) {
-        dht6_socket = s;
-    } else {
-        rc = dup2(s, dht6_socket);
-        close(s);
-        if(rc < 0)
-            return -1;
-    }
-
-    if(last_bound == NULL)
-        last_bound = malloc(16);
-    if(last_bound)
-        memcpy(last_bound, ipv6, 16);
-
-    return 1;
-}
-
 int
-tr_dhtInit(tr_session *ss, const tr_address * tr_addr)
+tr_dhtInit(tr_session *ss)
 {
-    struct sockaddr_in sin;
     tr_benc benc;
     int rc;
-    tr_bool have_id = FALSE;
+    bool have_id = false;
     char * dat_file;
     uint8_t * nodes = NULL, * nodes6 = NULL;
     const uint8_t * raw;
@@ -321,27 +273,8 @@ tr_dhtInit(tr_session *ss, const tr_address * tr_addr)
     if( session ) /* already initialized */
         return -1;
 
-    dht_port = tr_sessionGetPeerPort(ss);
-    if(dht_port <= 0)
-        return -1;
-
     tr_ndbg( "DHT", "Initializing DHT" );
 
-    dht_socket = socket(PF_INET, SOCK_DGRAM, 0);
-    if(dht_socket < 0)
-        goto fail;
-
-    memset(&sin, 0, sizeof(sin));
-    sin.sin_family = AF_INET;
-    memcpy(&sin.sin_addr, &tr_addr->addr.addr4, sizeof (struct in_addr));
-    sin.sin_port = htons(dht_port);
-    rc = bind(dht_socket, (struct sockaddr*)&sin, sizeof(sin));
-    if(rc < 0)
-        goto fail;
-
-    if(tr_globalIPv6())
-        rebind_ipv6(TRUE);
-
     if( getenv( "TR_DHT_VERBOSE" ) != NULL )
         dht_debug = stderr;
 
@@ -352,11 +285,11 @@ tr_dhtInit(tr_session *ss, const tr_address * tr_addr)
         have_id = tr_bencDictFindRaw(&benc, "id", &raw, &len);
         if( have_id && len==20 )
             memcpy( myid, raw, len );
-        if( dht_socket >= 0 &&
+        if( ss->udp_socket >= 0 &&
             tr_bencDictFindRaw( &benc, "nodes", &raw, &len ) && !(len%6) ) {
                 nodes = tr_memdup( raw, len );
         }
-        if( dht6_socket > 0 &&
+        if( ss->udp6_socket > 0 &&
             tr_bencDictFindRaw( &benc, "nodes6", &raw, &len6 ) && !(len6%18) ) {
             nodes6 = tr_memdup( raw, len6 );
         }
@@ -377,8 +310,8 @@ tr_dhtInit(tr_session *ss, const tr_address * tr_addr)
         tr_cryptoRandBuf( myid, 20 );
     }
 
-    rc = dht_init( dht_socket, dht6_socket, myid, NULL );
-    if(rc < 0)
+    rc = dht_init( ss->udp_socket, ss->udp6_socket, myid, NULL );
+    if( rc < 0 )
         goto fail;
 
     session = ss;
@@ -391,31 +324,16 @@ tr_dhtInit(tr_session *ss, const tr_address * tr_addr)
     cl->len6 = len6;
     tr_threadNew( dht_bootstrap, cl );
 
-    event_set( &dht_event, dht_socket, EV_READ, event_callback, NULL );
-    tr_timerAdd( &dht_event, 0, tr_cryptoWeakRandInt( 1000000 ) );
-
-    if( dht6_socket >= 0 )
-    {
-        event_set( &dht6_event, dht6_socket, EV_READ, event_callback, NULL );
-        tr_timerAdd( &dht6_event, 0, tr_cryptoWeakRandInt( 1000000 ) );
-    }
+    dht_timer = evtimer_new( session->event_base, timer_callback, session );
+    tr_timerAdd( dht_timer, 0, tr_cryptoWeakRandInt( 1000000 ) );
 
     tr_ndbg( "DHT", "DHT initialized" );
 
     return 1;
 
-    fail:
-    {
-        const int save = errno;
-        close(dht_socket);
-        if( dht6_socket >= 0 )
-            close(dht6_socket);
-        dht_socket = dht6_socket = -1;
-        session = NULL;
-        tr_ndbg( "DHT", "DHT initialization failed (errno = %d)", save );
-        errno = save;
-    }
-
+ fail:
+    tr_ndbg( "DHT", "DHT initialization failed (errno = %d)", errno );
+    session = NULL;
     return -1;
 }
 
@@ -427,10 +345,10 @@ tr_dhtUninit(tr_session *ss)
 
     tr_ndbg( "DHT", "Uninitializing DHT" );
 
-    event_del( &dht_event );
-
-    if( dht6_socket >= 0 )
-        event_del( &dht6_event );
+    if( dht_timer != NULL ) {
+        event_free( dht_timer );
+        dht_timer = NULL;
+    }
 
     /* Since we only save known good nodes, avoid erasing older data if we
        don't know enough nodes. */
@@ -471,20 +389,13 @@ tr_dhtUninit(tr_session *ss)
         tr_free( dat_file );
     }
 
-    dht_uninit( 1 );
-    tr_netCloseSocket( dht_socket );
-    dht_socket = -1;
-    if(dht6_socket > 0) {
-        tr_netCloseSocket( dht6_socket );
-        dht6_socket = -1;
-    }
-
+    dht_uninit();
     tr_ndbg("DHT", "Done uninitializing DHT");
 
     session = NULL;
 }
 
-tr_bool
+bool
 tr_dhtEnabled( const tr_session * ss )
 {
     return ss && ( ss == session );
@@ -523,8 +434,8 @@ tr_dhtStatus( tr_session * session, int af, int * nodes_return )
     struct getstatus_closure closure = { af, -1, -1 };
 
     if( !tr_dhtEnabled( session ) ||
-        (af == AF_INET && dht_socket < 0) ||
-        (af == AF_INET6 && dht6_socket < 0) ) {
+        (af == AF_INET && session->udp_socket < 0) ||
+        (af == AF_INET6 && session->udp6_socket < 0) ) {
         if( nodes_return )
             *nodes_return = 0;
         return TR_DHT_STOPPED;
@@ -532,7 +443,7 @@ tr_dhtStatus( tr_session * session, int af, int * nodes_return )
 
     tr_runInEventThread( session, getstatus, &closure );
     while( closure.status < 0 )
-        tr_wait_msec( 10 /*msec*/ );
+        tr_wait_msec( 50 /*msec*/ );
 
     if( nodes_return )
         *nodes_return = closure.count;
@@ -543,14 +454,14 @@ tr_dhtStatus( tr_session * session, int af, int * nodes_return )
 tr_port
 tr_dhtPort( tr_session *ss )
 {
-    return tr_dhtEnabled( ss ) ? dht_port : 0;
+    return tr_dhtEnabled( ss ) ? ss->udp_port : 0;
 }
 
 int
 tr_dhtAddNode( tr_session       * ss,
                const tr_address * address,
                tr_port            port,
-               tr_bool            bootstrap )
+               bool            bootstrap )
 {
     int af = address->type == TR_AF_INET ? AF_INET : AF_INET6;
 
@@ -638,8 +549,8 @@ callback( void *ignore UNUSED, int event,
     }
 }
 
-int
-tr_dhtAnnounce(tr_torrent *tor, int af, tr_bool announce)
+static int
+tr_dhtAnnounce(tr_torrent *tor, int af, bool announce)
 {
     int rc, status, numnodes, ret = 0;
 
@@ -662,14 +573,15 @@ tr_dhtAnnounce(tr_torrent *tor, int af, tr_bool announce)
                       af == AF_INET6 ? " IPv6" : "",
                       tr_dhtPrintableStatus(status), numnodes);
             if(af == AF_INET)
-                tor->dhtAnnounceInProgress = TRUE;
+                tor->dhtAnnounceInProgress = true;
             else
-                tor->dhtAnnounce6InProgress = TRUE;
+                tor->dhtAnnounce6InProgress = true;
             ret = 1;
         } else {
-            tr_torerr(tor, "%sDHT announce failed, errno = %d (%s, %d nodes)",
+            tr_torerr(tor, "%sDHT announce failed (%s, %d nodes): %s",
                       af == AF_INET6 ? "IPv6 " : "",
-                      errno, tr_dhtPrintableStatus(status), numnodes);
+                      tr_dhtPrintableStatus(status), numnodes,
+                      tr_strerror( errno ) );
         }
     } else {
         tr_tordbg(tor, "%sDHT not ready (%s, %d nodes)",
@@ -680,37 +592,75 @@ tr_dhtAnnounce(tr_torrent *tor, int af, tr_bool announce)
     return ret;
 }
 
-static void
-event_callback(int s, short type, void *ignore UNUSED )
+void
+tr_dhtUpkeep( tr_session * session )
 {
-    struct event *event = (s == dht_socket) ? &dht_event : &dht6_event;
-    time_t tosleep;
-    static int count = 0;
+    tr_torrent * tor = NULL;
+    const time_t now = tr_time( );
 
-    if( dht_periodic( type == EV_READ, &tosleep, callback, NULL) < 0 ) {
+    while(( tor = tr_torrentNext( session, tor )))
+    {
+        if( !tor->isRunning || !tr_torrentAllowsDHT( tor ) )
+            continue;
+
+        if( tor->dhtAnnounceAt <= now )
+        {
+            const int rc = tr_dhtAnnounce(tor, AF_INET, 1);
+
+            tor->dhtAnnounceAt = now + ((rc == 0)
+                                     ? 5 + tr_cryptoWeakRandInt( 5 )
+                                     : 25 * 60 + tr_cryptoWeakRandInt( 3*60 ));
+        }
+
+        if( tor->dhtAnnounce6At <= now )
+        {
+            const int rc = tr_dhtAnnounce(tor, AF_INET6, 1);
+
+            tor->dhtAnnounce6At = now + ((rc == 0)
+                                      ? 5 + tr_cryptoWeakRandInt( 5 )
+                                      : 25 * 60 + tr_cryptoWeakRandInt( 3*60 ));
+        }
+    }
+}
+
+void
+tr_dhtCallback(unsigned char *buf, int buflen,
+               struct sockaddr *from, socklen_t fromlen,
+               void *sv )
+{
+    time_t tosleep;
+    int rc;
+
+    assert(tr_isSession(sv));
+
+    if(sv != session)
+        return;
+
+    rc = dht_periodic( buf, buflen, from, fromlen,
+                       &tosleep, callback, NULL);
+    if(rc < 0) {
         if(errno == EINTR) {
             tosleep = 0;
         } else {
-            tr_nerr("DHT", "dht_periodic failed (errno = %d)", errno);
+            tr_nerr( "DHT", "dht_periodic failed: %s", tr_strerror( errno ) );
             if(errno == EINVAL || errno == EFAULT)
                     abort();
             tosleep = 1;
         }
     }
 
-    /* Only do this once in a while.  Counting rather than measuring time
-       avoids a system call. */
-    count++;
-    if(count >= 20) {
-        rebind_ipv6(FALSE);
-        count = 0;
-    }
-
     /* Being slightly late is fine,
        and has the added benefit of adding some jitter. */
-    tr_timerAdd( event, tosleep, tr_cryptoWeakRandInt( 1000000 ) );
+    tr_timerAdd( dht_timer, tosleep, tr_cryptoWeakRandInt( 1000000 ) );
 }
 
+static void
+timer_callback(int s UNUSED, short type UNUSED, void *session )
+{
+    tr_dhtCallback(NULL, 0, NULL, 0, session);
+}
+
+
 void
 dht_hash(void *hash_return, int hash_size,
          const void *v1, int len1,
