@@ -1,15 +1,16 @@
 /*
- * This file Copyright (C) 2008-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: tr-getopt.c 8071 2009-03-17 18:08:02Z charles $
+ * $Id: tr-getopt.c 11709 2011-01-19 13:48:47Z jordan $
  */
 
+#include <ctype.h> /* isspace() */
 #include <stdio.h>
 #include <stdlib.h> /* exit() */
 #include <string.h>
@@ -37,21 +38,52 @@ getArgName( const tr_option * opt )
     return arg;
 }
 
+static int
+get_next_line_len( const char * description, int maxlen )
+{
+    int end;
+    int len = strlen( description );
+
+    if( len < maxlen )
+        return len;
+
+    end = maxlen < len ? maxlen : len;
+    while( ( end > 0 ) && !isspace( description[end] ) )
+        --end;
+
+    return end ? end : len;
+}
+
 static void
 getopts_usage_line( const tr_option * opt,
                     int               longWidth,
                     int               shortWidth,
                     int               argWidth )
 {
+    int len;
     const char * longName   = opt->longName ? opt->longName : "";
     const char * shortName  = opt->shortName ? opt->shortName : "";
     const char * arg        = getArgName( opt );
 
-    printf( "  %s%-*s %s%-*s %-*s  %s\n",
+    const int d_indent = shortWidth + longWidth + argWidth + 7;
+    const int d_width = 80 - d_indent;
+    const char * d = opt->description;
+
+    printf( " %s%-*s %s%-*s %-*s ",
             (shortName && *shortName ? "-" : " "), shortWidth, shortName,
             (longName && *longName ? "--" : "  "), longWidth, longName,
-            argWidth, arg,
-            opt->description );
+            argWidth, arg );
+    len = get_next_line_len( d, d_width );
+    printf( "%*.*s\n", len, len, d );
+
+    d += len;
+    while( isspace( *d ) ) ++d;
+
+    while(( len = get_next_line_len( d, d_width ))) {
+        printf( "%*.*s%*.*s\n", d_indent, d_indent, "", len, len, d );
+        d += len;
+        while( isspace( *d ) ) ++d;
+    }
 }
 
 static void
