@@ -1,13 +1,13 @@
 /*
- * This file Copyright (C) 2009-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: utils.c 11474 2010-12-05 18:50:18Z charles $
+ * $Id: utils.c 12551 2011-07-17 14:15:02Z jordan $
  */
 
 #ifdef HAVE_MEMMEM
@@ -22,55 +22,57 @@
 #endif
 
 #include <assert.h>
-#include <ctype.h> /* isalpha, tolower */
+#include <ctype.h> /* isdigit(), isalpha(), tolower() */
 #include <errno.h>
 #include <float.h> /* DBL_EPSILON */
 #include <locale.h> /* localeconv() */
-#include <math.h> /* pow */
+#include <math.h> /* pow(), fabs(), floor() */
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h> /* strerror, memset, memmem */
+#include <string.h> /* strerror(), memset(), memmem() */
 #include <time.h> /* nanosleep() */
 
 #ifdef HAVE_ICONV_OPEN
  #include <iconv.h>
 #endif
-#include <libgen.h> /* basename */
+#include <libgen.h> /* basename() */
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <unistd.h> /* stat(), getcwd(), getpagesize() */
+#include <unistd.h> /* stat(), getcwd(), getpagesize(), unlink() */
 
-#include "event.h"
+#include <event2/buffer.h>
+#include <event2/event.h>
 
 #ifdef WIN32
- #include <direct.h> /* _getcwd */
- #include <windows.h> /* Sleep */
+ #include <w32api.h>
+ #define WINVER WindowsXP /* freeaddrinfo(), getaddrinfo(), getnameinfo() */
+ #include <direct.h> /* _getcwd() */
+ #include <windows.h> /* Sleep() */
 #endif
 
 #include "transmission.h"
+#include "bencode.h"
 #include "fdlimit.h"
 #include "ConvertUTF.h"
 #include "list.h"
-#include "net.h"
 #include "utils.h"
-#include "platform.h"
+#include "platform.h" /* tr_lockLock(), TR_PATH_MAX */
 #include "version.h"
 
 
-time_t transmission_now = 0;
+time_t       __tr_current_time   = 0;
+tr_msg_level __tr_message_level  = TR_MSG_ERR;
 
-int                   messageLevel = TR_MSG_INF;
-static tr_lock *      messageLock = NULL;
-static tr_bool        messageQueuing = FALSE;
+static bool           messageQueuing = false;
 static tr_msg_list *  messageQueue = NULL;
 static tr_msg_list ** messageQueueTail = &messageQueue;
 static int            messageQueueCount = 0;
 
 #ifndef WIN32
     /* make null versions of these win32 functions */
-    static inline int IsDebuggerPresent( void ) { return FALSE; }
+    static inline int IsDebuggerPresent( void ) { return false; }
     static inline void OutputDebugString( const void * unused UNUSED ) { }
 #endif
 
@@ -78,21 +80,21 @@ static int            messageQueueCount = 0;
 ****
 ***/
 
-void
-tr_msgInit( void )
+static tr_lock*
+getMessageLock( void )
 {
-    const char * env = getenv( "TR_DEBUG" );
-    messageLevel = ( env ? atoi( env ) : 0 ) + 1;
-    messageLevel = MAX( 1, messageLevel );
+    static tr_lock * l = NULL;
 
-    if( messageLock == NULL )
-        messageLock = tr_lockNew( );
+    if( !l )
+        l = tr_lockNew( );
+
+    return l;
 }
 
-FILE*
+void*
 tr_getLog( void )
 {
-    static tr_bool initialized = FALSE;
+    static bool initialized = false;
     static FILE * file = NULL;
 
     if( !initialized )
@@ -112,61 +114,35 @@ tr_getLog( void )
             default:
                 file = NULL; break;
         }
-        initialized = TRUE;
+        initialized = true;
     }
 
     return file;
 }
 
 void
-tr_setMessageLevel( int level )
+tr_setMessageLevel( tr_msg_level level )
 {
-    tr_lockLock( messageLock );
-
-    messageLevel = MAX( 0, level );
-
-    tr_lockUnlock( messageLock );
-}
-
-int
-tr_getMessageLevel( void )
-{
-    int ret;
-    tr_lockLock( messageLock );
-
-    ret = messageLevel;
-
-    tr_lockUnlock( messageLock );
-    return ret;
+    __tr_message_level = level;
 }
 
 void
-tr_setMessageQueuing( tr_bool enabled )
+tr_setMessageQueuing( bool enabled )
 {
-    tr_lockLock( messageLock );
-
     messageQueuing = enabled;
-
-    tr_lockUnlock( messageLock );
 }
 
-tr_bool
+bool
 tr_getMessageQueuing( void )
 {
-    int ret;
-    tr_lockLock( messageLock );
-
-    ret = messageQueuing;
-
-    tr_lockUnlock( messageLock );
-    return ret;
+    return messageQueuing != 0;
 }
 
 tr_msg_list *
 tr_getQueuedMessages( void )
 {
     tr_msg_list * ret;
-    tr_lockLock( messageLock );
+    tr_lockLock( getMessageLock( ) );
 
     ret = messageQueue;
     messageQueue = NULL;
@@ -174,7 +150,7 @@ tr_getQueuedMessages( void )
 
     messageQueueCount = 0;
 
-    tr_lockUnlock( messageLock );
+    tr_lockUnlock( getMessageLock( ) );
     return ret;
 }
 
@@ -224,13 +200,13 @@ tr_getLogTimeStr( char * buf, int buflen )
     seconds = tv.tv_sec;
     tr_localtime_r( &seconds, &now_tm );
     strftime( tmp, sizeof( tmp ), "%H:%M:%S", &now_tm );
-    milliseconds = (int)( tv.tv_usec / 1000 );
+    milliseconds = tv.tv_usec / 1000;
     tr_snprintf( buf, buflen, "%s.%03d", tmp, milliseconds );
 
     return buf;
 }
 
-tr_bool
+bool
 tr_deepLoggingIsActive( void )
 {
     static int8_t deepLoggingIsActive = -1;
@@ -265,9 +241,9 @@ tr_deepLog( const char  * file,
         va_end( args );
         evbuffer_add_printf( buf, " (%s:%d)\n", base, line );
         /* FIXME(libevent2) ifdef this out for nonwindows platforms */
-        OutputDebugString( EVBUFFER_DATA( buf ) );
-        if(fp) /* FIXME(libevent2) tr_getLog() should return an fd, then use evbuffer_write() here ) */
-            (void) fwrite( EVBUFFER_DATA( buf ), 1, EVBUFFER_LENGTH( buf ), fp );
+        OutputDebugString( evbuffer_pullup( buf, -1 ) );
+        if( fp )
+            fputs( (const char*)evbuffer_pullup( buf, -1 ), fp );
 
         tr_free( base );
         evbuffer_free( buf );
@@ -280,15 +256,14 @@ tr_deepLog( const char  * file,
 
 void
 tr_msg( const char * file, int line,
-        int level, const char * name,
+        tr_msg_level level,
+        const char * name,
         const char * fmt, ... )
 {
     const int err = errno; /* message logging shouldn't affect errno */
     char buf[1024];
     va_list ap;
-
-    if( messageLock != NULL )
-        tr_lockLock( messageLock );
+    tr_lockLock( getMessageLock( ) );
 
     /* build the text message */
     *buf = '\0';
@@ -346,9 +321,7 @@ tr_msg( const char * file, int line,
         }
     }
 
-    if( messageLock != NULL )
-        tr_lockUnlock( messageLock );
-
+    tr_lockUnlock( getMessageLock( ) );
     errno = err;
 }
 
@@ -356,100 +329,72 @@ tr_msg( const char * file, int line,
 ****
 ***/
 
-void
-tr_set_compare( const void * va,
-                size_t aCount,
-                const void * vb,
-                size_t bCount,
-                int compare( const void * a, const void * b ),
-                size_t elementSize,
-                tr_set_func in_a_cb,
-                tr_set_func in_b_cb,
-                tr_set_func in_both_cb,
-                void * userData )
+void*
+tr_malloc( size_t size )
 {
-    const uint8_t * a = (const uint8_t *) va;
-    const uint8_t * b = (const uint8_t *) vb;
-    const uint8_t * aend = a + elementSize * aCount;
-    const uint8_t * bend = b + elementSize * bCount;
+    return size ? malloc( size ) : NULL;
+}
 
-    while( a != aend || b != bend )
-    {
-        if( a == aend )
-        {
-            ( *in_b_cb )( (void*)b, userData );
-            b += elementSize;
-        }
-        else if( b == bend )
-        {
-            ( *in_a_cb )( (void*)a, userData );
-            a += elementSize;
-        }
-        else
-        {
-            const int val = ( *compare )( a, b );
+void*
+tr_malloc0( size_t size )
+{
+    return size ? calloc( 1, size ) : NULL;
+}
 
-            if( !val )
-            {
-                ( *in_both_cb )( (void*)a, userData );
-                a += elementSize;
-                b += elementSize;
-            }
-            else if( val < 0 )
-            {
-                ( *in_a_cb )( (void*)a, userData );
-                a += elementSize;
-            }
-            else if( val > 0 )
-            {
-                ( *in_b_cb )( (void*)b, userData );
-                b += elementSize;
-            }
-        }
-    }
+void
+tr_free( void * p )
+{
+    if( p != NULL )
+        free( p );
+}
+
+void*
+tr_memdup( const void * src, size_t byteCount )
+{
+    return memcpy( tr_malloc( byteCount ), src, byteCount );
 }
 
 /***
 ****
 ***/
 
-#ifdef DISABLE_GETTEXT
-
 const char*
 tr_strip_positional_args( const char* str )
 {
     const char * in = str;
     static size_t bufsize = 0;
     static char * buf = NULL;
-    const size_t  len = strlen( str );
+    const size_t  len = str ? strlen( str ) : 0;
     char *        out;
 
-    if( bufsize < len )
+    if( !buf || ( bufsize < len ) )
     {
-        bufsize = len * 2;
+        bufsize = len * 2 + 1;
         buf = tr_renew( char, buf, bufsize );
     }
 
-    for( out = buf; *str; ++str )
+    for( out = buf; str && *str; ++str )
     {
         *out++ = *str;
+
         if( ( *str == '%' ) && isdigit( str[1] ) )
         {
             const char * tmp = str + 1;
             while( isdigit( *tmp ) )
                 ++tmp;
-
             if( *tmp == '$' )
-                str = tmp;
+                str = tmp[1]=='\'' ? tmp+1 : tmp;
         }
+
+        if( ( *str == '%' ) && ( str[1] == '\'' ) )
+            str = str + 1;
+
     }
     *out = '\0';
 
-    return strcmp( buf, in ) ? buf : in;
+    return !in || strcmp( buf, in ) ? buf : in;
 }
 
-#endif
-
 /**
 ***
 **/
@@ -488,7 +433,7 @@ tr_loadFile( const char * path,
     struct stat  sb;
     int fd;
     ssize_t n;
-    const char * err_fmt = _( "Couldn't read \"%1$s\": %2$s" );
+    const char * const err_fmt = _( "Couldn't read \"%1$s\": %2$s" );
 
     /* try to stat the file */
     errno = 0;
@@ -516,7 +461,7 @@ tr_loadFile( const char * path,
         errno = err;
         return NULL;
     }
-    buf = malloc( sb.st_size + 1 );
+    buf = tr_malloc( sb.st_size + 1 );
     if( !buf )
     {
         const int err = errno;
@@ -525,7 +470,7 @@ tr_loadFile( const char * path,
         errno = err;
         return NULL;
     }
-    n = read( fd, buf, sb.st_size );
+    n = read( fd, buf, (size_t)sb.st_size );
     if( n == -1 )
     {
         const int err = errno;
@@ -652,7 +597,7 @@ tr_buildPath( const char *first_element, ... )
     element = first_element;
     while( element ) {
         bufLen += strlen( element ) + 1;
-        element = (const char*) va_arg( vl, const char* );
+        element = va_arg( vl, const char* );
     }
     pch = buf = tr_new( char, bufLen );
     va_end( vl );
@@ -665,11 +610,11 @@ tr_buildPath( const char *first_element, ... )
         memcpy( pch, element, elementLen );
         pch += elementLen;
         *pch++ = TR_PATH_DELIMITER;
-        element = (const char*) va_arg( vl, const char* );
+        element = va_arg( vl, const char* );
     }
     va_end( vl );
 
-    /* terminate the string.  if nonempty, eat the unwanted trailing slash */
+    /* terminate the string. if nonempty, eat the unwanted trailing slash */
     if( pch != buf )
         --pch;
     *pch++ = '\0';
@@ -683,6 +628,23 @@ tr_buildPath( const char *first_element, ... )
 *****
 ****/
 
+char*
+evbuffer_free_to_str( struct evbuffer * buf )
+{
+    const size_t n = evbuffer_get_length( buf );
+    char * ret = tr_new( char, n + 1 );
+    evbuffer_copyout( buf, ret, n );
+    evbuffer_free( buf );
+    ret[n] = '\0';
+    return ret;
+}
+
+char*
+tr_strdup( const void * in )
+{
+    return tr_strndup( in, in ? (int)strlen((const char *)in) : 0 );
+}
+
 char*
 tr_strndup( const void * in, int len )
 {
@@ -754,10 +716,50 @@ tr_strerror( int i )
     return ret;
 }
 
+int
+tr_strcmp0( const char * str1, const char * str2 )
+{
+    if( str1 && str2 ) return strcmp( str1, str2 );
+    if( str1 ) return 1;
+    if( str2 ) return -1;
+    return 0;
+}
+
 /****
 *****
 ****/
 
+/* https://bugs.launchpad.net/percona-patches/+bug/526863/+attachment/1160199/+files/solaris_10_fix.patch */
+char*
+tr_strsep( char ** str, const char * delims )
+{
+#ifdef HAVE_STRSEP
+    return strsep( str, delims );
+#else
+    char *token;
+
+    if (*str == NULL) {
+        /* No more tokens */
+        return NULL;
+    }
+
+    token = *str;
+    while (**str != '\0') {
+        if (strchr(delims, **str) != NULL) {
+            **str = '\0';
+            (*str)++;
+            return token;
+        }
+        (*str)++;
+    }
+
+    /* There is not another token */
+    *str = NULL;
+
+    return token;
+#endif
+}
+
 char*
 tr_strstrip( char * str )
 {
@@ -780,23 +782,23 @@ tr_strstrip( char * str )
     return str;
 }
 
-tr_bool
+bool
 tr_str_has_suffix( const char *str, const char *suffix )
 {
     size_t str_len;
     size_t suffix_len;
 
     if( !str )
-        return FALSE;
+        return false;
     if( !suffix )
-        return TRUE;
+        return true;
 
     str_len = strlen( str );
     suffix_len = strlen( suffix );
     if( str_len < suffix_len )
-        return FALSE;
+        return false;
 
-    return !strncasecmp( str + str_len - suffix_len, suffix, suffix_len );
+    return !evutil_ascii_strncasecmp( str + str_len - suffix_len, suffix, suffix_len );
 }
 
 /****
@@ -804,7 +806,7 @@ tr_str_has_suffix( const char *str, const char *suffix )
 ****/
 
 uint64_t
-tr_date( void )
+tr_time_msec( void )
 {
     struct timeval tv;
 
@@ -842,14 +844,12 @@ tr_snprintf( char * buf, size_t buflen, const char * fmt, ... )
 }
 
 /*
- * Copy src to string dst of size siz.  At most siz-1 characters
- * will be copied.  Always NUL terminates (unless siz == 0).
+ * Copy src to string dst of size siz. At most siz-1 characters
+ * will be copied. Always NUL terminates (unless siz == 0).
  * Returns strlen(src); if retval >= siz, truncation occurred.
  */
 size_t
-tr_strlcpy( char *       dst,
-            const void * src,
-            size_t       siz )
+tr_strlcpy( char * dst, const void * src, size_t siz )
 {
 #ifdef HAVE_STRLCPY
     return strlcpy( dst, src, siz );
@@ -889,14 +889,13 @@ tr_strlcpy( char *       dst,
 ***/
 
 double
-tr_getRatio( double numerator,
-             double denominator )
+tr_getRatio( uint64_t numerator, uint64_t denominator )
 {
     double ratio;
 
-    if( denominator )
-        ratio = numerator / denominator;
-    else if( numerator )
+    if( denominator > 0 )
+        ratio = numerator / (double)denominator;
+    else if( numerator > 0 )
         ratio = TR_RATIO_INF;
     else
         ratio = TR_RATIO_NA;
@@ -938,10 +937,11 @@ tr_hex_to_sha1( uint8_t * out, const char * in )
 ****
 ***/
 
-static tr_bool
-isValidURLChars( const char * url )
+static bool
+isValidURLChars( const char * url, int url_len )
 {
     const char * c;
+    const char * end;
     static const char * rfc2396_valid_chars =
         "abcdefghijklmnopqrstuvwxyz" /* lowalpha */
         "ABCDEFGHIJKLMNOPQRSTUVWXYZ" /* upalpha */
@@ -952,52 +952,49 @@ isValidURLChars( const char * url )
         "{}|\\^[]`";                 /* unwise */
 
     if( url == NULL )
-        return FALSE;
+        return false;
 
-    for( c = url; c && *c; ++c )
+    for( c=url, end=c+url_len; c && *c && c!=end; ++c )
         if( !strchr( rfc2396_valid_chars, *c ) )
-            return FALSE;
+            return false;
 
-    return TRUE;
+    return true;
 }
 
-/** @brief return TRUE if the url is a http or https url that Transmission understands */
-tr_bool
+/** @brief return true if the URL is a http or https or UDP one that Transmission understands */
+bool
 tr_urlIsValidTracker( const char * url )
 {
-    tr_bool valid;
-    char * scheme = NULL;
+    bool valid;
+    const int len = url ? strlen(url) : 0;
 
-    valid = isValidURLChars( url )
-         && !tr_urlParse( url, -1, &scheme, NULL, NULL, NULL )
-         && ( scheme != NULL )
-         && ( !strcmp(scheme,"http") || !strcmp(scheme,"https") );
+    valid = isValidURLChars( url, len )
+         && !tr_urlParse( url, len, NULL, NULL, NULL, NULL )
+         && ( !memcmp(url,"http://",7) || !memcmp(url,"https://",8) || !memcmp(url,"udp://",6) );
 
-    tr_free( scheme );
     return valid;
 }
 
-/** @brief return TRUE if the url is a http or https or ftp or sftp url that Transmission understands */
-tr_bool
-tr_urlIsValid( const char * url )
+/** @brief return true if the URL is a http or https or ftp or sftp one that Transmission understands */
+bool
+tr_urlIsValid( const char * url, int url_len )
 {
-    tr_bool valid;
-    char * scheme = NULL;
+    bool valid;
+    if( ( url_len < 0 ) && ( url != NULL ) )
+        url_len = strlen( url );
 
-    valid = isValidURLChars( url )
-         && !tr_urlParse( url, -1, &scheme, NULL, NULL, NULL )
-         && ( scheme != NULL )
-         && ( !strcmp(scheme,"http") || !strcmp(scheme,"https") || !strcmp(scheme,"ftp") || !strcmp(scheme,"sftp") );
+    valid = isValidURLChars( url, url_len )
+         && !tr_urlParse( url, url_len, NULL, NULL, NULL, NULL )
+         && ( !memcmp(url,"http://",7) || !memcmp(url,"https://",8) || !memcmp(url,"ftp://",6) || !memcmp(url,"sftp://",7) );
 
-    tr_free( scheme );
     return valid;
 }
 
-tr_bool
-tr_addressIsIP( const char * address )
+bool
+tr_addressIsIP( const char * str )
 {
-    tr_address tempAddr;
-    return tr_pton(address, &tempAddr) != NULL;
+    tr_address tmp;
+    return tr_address_from_string( &tmp, str );
 }
 
 int
@@ -1013,8 +1010,10 @@ tr_urlParse( const char * url_in,
     int          n;
     char *       tmp;
     char *       pch;
-    const char * protocol = NULL;
+    size_t       host_len;
+    size_t       protocol_len;
     const char * host = NULL;
+    const char * protocol = NULL;
     const char * path = NULL;
 
     tmp = tr_strndup( url_in, len );
@@ -1022,14 +1021,17 @@ tr_urlParse( const char * url_in,
     {
         *pch = '\0';
         protocol = tmp;
+        protocol_len = pch - protocol;
         pch += 3;
 /*fprintf( stderr, "protocol is [%s]... what's left is [%s]\n", protocol, pch);*/
         if( ( n = strcspn( pch, ":/" ) ) )
         {
             const int havePort = pch[n] == ':';
             host = pch;
+            host_len = n;
             pch += n;
-            *pch++ = '\0';
+            if( pch && *pch )
+                *pch++ = '\0';
 /*fprintf( stderr, "host is [%s]... what's left is [%s]\n", host, pch );*/
             if( havePort )
             {
@@ -1047,19 +1049,21 @@ tr_urlParse( const char * url_in,
 
     if( !err && !port )
     {
-        if( !strcmp( protocol, "ftp" ) ) port = 21;
-        if( !strcmp( protocol, "sftp" ) ) port = 22;
-        if( !strcmp( protocol, "http" ) ) port = 80;
-        if( !strcmp( protocol, "https" ) ) port = 443;
+        if( !strcmp( protocol, "udp" ) ) port = 80;
+        else if( !strcmp( protocol, "ftp" ) ) port = 21;
+        else if( !strcmp( protocol, "sftp" ) ) port = 22;
+        else if( !strcmp( protocol, "http" ) ) port = 80;
+        else if( !strcmp( protocol, "https" ) ) port = 443;
     }
 
     if( !err )
     {
-        if( setme_protocol ) *setme_protocol = tr_strdup( protocol );
+        if( setme_protocol ) *setme_protocol = tr_strndup( protocol, protocol_len );
 
         if( setme_host ){ ( (char*)host )[-3] = ':'; *setme_host =
-                              tr_strdup( host ); }
-        if( setme_path ){ if( path[0] == '/' ) *setme_path = tr_strdup( path );
+                              tr_strndup( host, host_len ); }
+        if( setme_path ){ if( !*path ) *setme_path = tr_strdup( "/" );
+                          else if( path[0] == '/' ) *setme_path = tr_strdup( path );
                           else { ( (char*)path )[-1] = '/'; *setme_path = tr_strdup( path - 1 ); } }
         if( setme_port ) *setme_port = port;
     }
@@ -1089,10 +1093,11 @@ tr_base64_encode( const void * input, int length, int * setme_len )
         BUF_MEM * bptr;
 
         if( length < 1 )
-            length = strlen( input );
+            length = (int)strlen( input );
 
         bmem = BIO_new( BIO_s_mem( ) );
         b64 = BIO_new( BIO_f_base64( ) );
+        BIO_set_flags( b64, BIO_FLAGS_BASE64_NO_NL );
         b64 = BIO_push( b64, bmem );
         BIO_write( b64, input, length );
         (void) BIO_flush( b64 );
@@ -1148,17 +1153,30 @@ tr_base64_decode( const void * input,
 ****
 ***/
 
+void
+tr_removeElementFromArray( void         * array,
+                           unsigned int   index_to_remove,
+                           size_t         sizeof_element,
+                           size_t         nmemb )
+{
+    char * a = array;
+
+    memmove( a + sizeof_element * index_to_remove,
+             a + sizeof_element * ( index_to_remove  + 1 ),
+             sizeof_element * ( --nmemb - index_to_remove ) );
+}
+
 int
 tr_lowerBound( const void * key,
                const void * base,
                size_t       nmemb,
                size_t       size,
                int       (* compar)(const void* key, const void* arrayMember),
-               tr_bool    * exact_match )
+               bool       * exact_match )
 {
     size_t first = 0;
     const char * cbase = base;
-    tr_bool exact = FALSE;
+    bool exact = false;
 
     while( nmemb != 0 )
     {
@@ -1168,7 +1186,7 @@ tr_lowerBound( const void * key,
 
         if( c <= 0 ) {
             if( c == 0 )
-                exact = TRUE;
+                exact = true;
             nmemb = half;
         } else {
             first = middle + 1;
@@ -1188,11 +1206,10 @@ tr_lowerBound( const void * key,
 static char*
 strip_non_utf8( const char * in, size_t inlen )
 {
-    char * ret;
     const char * end;
     const char zero = '\0';
     struct evbuffer * buf = evbuffer_new( );
- 
+
     while( !tr_utf8_validate( in, inlen, &end ) )
     {
         const int good_len = end - in;
@@ -1202,12 +1219,10 @@ strip_non_utf8( const char * in, size_t inlen )
         in += ( good_len + 1 );
         evbuffer_add( buf, "?", 1 );
     }
- 
+
     evbuffer_add( buf, in, inlen );
     evbuffer_add( buf, &zero, 1 );
-    ret = tr_memdup( EVBUFFER_DATA( buf ), EVBUFFER_LENGTH( buf ) );
-    evbuffer_free( buf );
-    return ret;
+    return evbuffer_free_to_str( buf );
 }
 
 static char*
@@ -1237,6 +1252,8 @@ to_utf8( const char * in, size_t inlen )
             iconv_close( cd );
         }
     }
+
+    tr_free( out );
 #endif
 
     if( ret == NULL )
@@ -1277,11 +1294,11 @@ struct number_range
  * This should be a single number (ex. "6") or a range (ex. "6-9").
  * Anything else is an error and will return failure.
  */
-static tr_bool
+static bool
 parseNumberSection( const char * str, int len, struct number_range * setme )
 {
     long a, b;
-    tr_bool success;
+    bool success;
     char * end;
     const int error = errno;
     char * tmp = tr_strndup( str, len );
@@ -1289,19 +1306,18 @@ parseNumberSection( const char * str, int len, struct number_range * setme )
     errno = 0;
     a = b = strtol( tmp, &end, 10 );
     if( errno || ( end == tmp ) ) {
-        success = FALSE;
+        success = false;
     } else if( *end != '-' ) {
-        b = a;
-        success = TRUE;
+        success = true;
     } else {
         const char * pch = end + 1;
         b = strtol( pch, &end, 10 );
         if( errno || ( pch == end ) )
-            success = FALSE;
+            success = false;
         else if( *end ) /* trailing data */
-            success = FALSE;
+            success = false;
         else
-            success = TRUE;
+            success = true;
     }
     tr_free( tmp );
 
@@ -1312,7 +1328,7 @@ parseNumberSection( const char * str, int len, struct number_range * setme )
     return success;
 }
 
-static int
+int
 compareInt( const void * va, const void * vb )
 {
     const int a = *(const int *)va;
@@ -1335,7 +1351,7 @@ tr_parseNumberRange( const char * str_in, int len, int * setmeCount )
     char * str = tr_strndup( str_in, len );
     const char * walk;
     tr_list * ranges = NULL;
-    tr_bool success = TRUE;
+    bool success = true;
 
     walk = str;
     while( walk && *walk && success ) {
@@ -1414,6 +1430,26 @@ tr_truncd( double x, int precision )
     return atof(buf);
 }
 
+/* return a truncated double as a string */
+static char*
+tr_strtruncd( char * buf, double x, int precision, size_t buflen )
+{
+    tr_snprintf( buf, buflen, "%.*f", precision, tr_truncd( x, precision ) );
+    return buf;
+}
+
+char*
+tr_strpercent( char * buf, double x, size_t buflen )
+{
+    if( x < 10.0 )
+        tr_strtruncd( buf, x, 2, buflen );
+    else if( x < 100.0 )
+        tr_strtruncd( buf, x, 1, buflen );
+    else
+        tr_strtruncd( buf, x, 0, buflen );
+    return buf;
+}
+
 char*
 tr_strratio( char * buf, size_t buflen, double ratio, const char * infinity )
 {
@@ -1421,12 +1457,8 @@ tr_strratio( char * buf, size_t buflen, double ratio, const char * infinity )
         tr_strlcpy( buf, _( "None" ), buflen );
     else if( (int)ratio == TR_RATIO_INF )
         tr_strlcpy( buf, infinity, buflen );
-    else if( ratio < 10.0 )
-        tr_snprintf( buf, buflen, "%.2f", tr_truncd( ratio, 2 ) );
-    else if( ratio < 100.0 )
-        tr_snprintf( buf, buflen, "%.1f", tr_truncd( ratio, 1 ) );
     else
-        tr_snprintf( buf, buflen, "%'.0f", ratio );
+        tr_strpercent( buf, ratio, buflen );
     return buf;
 }
 
@@ -1435,14 +1467,14 @@ tr_strratio( char * buf, size_t buflen, double ratio, const char * infinity )
 ***/
 
 int
-tr_moveFile( const char * oldpath, const char * newpath, tr_bool * renamed )
+tr_moveFile( const char * oldpath, const char * newpath, bool * renamed )
 {
     int in;
     int out;
     char * buf;
     struct stat st;
     off_t bytesLeft;
-    const off_t buflen = 1024 * 128; /* 128 KiB buffer */
+    const size_t buflen = 1024 * 128; /* 128 KiB buffer */
 
     /* make sure the old file exists */
     if( stat( oldpath, &st ) ) {
@@ -1481,7 +1513,7 @@ tr_moveFile( const char * oldpath, const char * newpath, tr_bool * renamed )
     while( bytesLeft > 0 )
     {
         ssize_t bytesWritten;
-        const off_t bytesThisPass = MIN( bytesLeft, buflen );
+        const off_t bytesThisPass = MIN( bytesLeft, (off_t)buflen );
         const int numRead = read( in, buf, bytesThisPass );
         if( numRead < 0 )
             break;
@@ -1502,6 +1534,17 @@ tr_moveFile( const char * oldpath, const char * newpath, tr_bool * renamed )
     return 0;
 }
 
+bool
+tr_is_same_file( const char * filename1, const char * filename2 )
+{
+    struct stat sb1, sb2;
+
+    return !stat( filename1, &sb1 )
+        && !stat( filename2, &sb2 )
+        && ( sb1.st_dev == sb2.st_dev )
+        && ( sb1.st_ino == sb2.st_ino );
+}
+
 /***
 ****
 ***/
@@ -1515,7 +1558,7 @@ tr_valloc( size_t bufLen )
 
     if( !pageSize ) {
 #ifdef HAVE_GETPAGESIZE
-        pageSize = getpagesize();
+        pageSize = (size_t) getpagesize();
 #else /* guess */
         pageSize = 4096;
 #endif
@@ -1527,15 +1570,219 @@ tr_valloc( size_t bufLen )
 
 #ifdef HAVE_POSIX_MEMALIGN
     if( !buf )
-        posix_memalign( &buf, pageSize, allocLen );
+        if( posix_memalign( &buf, pageSize, allocLen ) )
+            buf = NULL; /* just retry with valloc/malloc */
 #endif
 #ifdef HAVE_VALLOC
     if( !buf )
         buf = valloc( allocLen );
 #endif
     if( !buf )
-        buf = malloc( allocLen );
+        buf = tr_malloc( allocLen );
 
-    tr_dbg( "tr_valloc(%zu) allocating %zu bytes", bufLen, allocLen );
     return buf;
 }
+
+char *
+tr_realpath( const char * path, char * resolved_path )
+{
+#ifdef WIN32
+    /* From a message to the Mingw-msys list, Jun 2, 2005 by Mark Junker. */
+    if( GetFullPathNameA( path, TR_PATH_MAX, resolved_path, NULL ) == 0 )
+        return NULL;
+    return resolved_path;
+#else
+    return realpath( path, resolved_path );
+#endif
+}
+
+/***
+****
+***/
+
+uint64_t
+tr_htonll( uint64_t x )
+{
+#ifdef HAVE_HTONLL
+    return htonll( x );
+#else
+    /* fallback code by Runner and Juan Carlos Cobas at
+     * http://www.codeproject.com/KB/cpp/endianness.aspx */
+    return (((uint64_t)(htonl((int)((x << 32) >> 32))) << 32) |
+                     (unsigned int)htonl(((int)(x >> 32))));
+#endif
+}
+
+uint64_t
+tr_ntohll( uint64_t x )
+{
+#ifdef HAVE_NTOHLL
+    return ntohll( x );
+#else
+    /* fallback code by Runner and Juan Carlos Cobas at
+     * http://www.codeproject.com/KB/cpp/endianness.aspx */
+    return (((uint64_t)(ntohl((int)((x << 32) >> 32))) << 32) |
+                     (unsigned int)ntohl(((int)(x >> 32))));
+#endif
+}
+
+/***
+****
+****
+****
+***/
+
+struct formatter_unit
+{
+    char * name;
+    int64_t value;
+};
+
+struct formatter_units
+{
+    struct formatter_unit units[4];
+};
+
+enum { TR_FMT_KB, TR_FMT_MB, TR_FMT_GB, TR_FMT_TB };
+
+static void
+formatter_init( struct formatter_units * units,
+                unsigned int kilo,
+                const char * kb, const char * mb,
+                const char * gb, const char * tb )
+{
+    uint64_t value = kilo;
+    units->units[TR_FMT_KB].name = tr_strdup( kb );
+    units->units[TR_FMT_KB].value = value;
+
+    value *= kilo;
+    units->units[TR_FMT_MB].name = tr_strdup( mb );
+    units->units[TR_FMT_MB].value = value;
+
+    value *= kilo;
+    units->units[TR_FMT_GB].name = tr_strdup( gb );
+    units->units[TR_FMT_GB].value = value;
+
+    value *= kilo;
+    units->units[TR_FMT_TB].name = tr_strdup( tb );
+    units->units[TR_FMT_TB].value = value;
+}
+
+static char*
+formatter_get_size_str( const struct formatter_units * u,
+                        char * buf, int64_t bytes, size_t buflen )
+{
+    int precision;
+    double value;
+    const char * units;
+    const struct formatter_unit * unit;
+
+         if( bytes < u->units[1].value ) unit = &u->units[0];
+    else if( bytes < u->units[2].value ) unit = &u->units[1];
+    else if( bytes < u->units[3].value ) unit = &u->units[2];
+    else                                 unit = &u->units[3];
+
+    value = (double)bytes / unit->value;
+    units = unit->name;
+    if( unit->value == 1 )
+        precision = 0;
+    else if( value < 100 )
+        precision = 2;
+    else
+        precision = 1;
+    tr_snprintf( buf, buflen, "%.*f %s", precision, value, units );
+    return buf;
+}
+
+static struct formatter_units size_units;
+
+void
+tr_formatter_size_init( unsigned int kilo,
+                        const char * kb, const char * mb,
+                        const char * gb, const char * tb )
+{
+    formatter_init( &size_units, kilo, kb, mb, gb, tb );
+}
+
+char*
+tr_formatter_size_B( char * buf, int64_t bytes, size_t buflen )
+{
+    return formatter_get_size_str( &size_units, buf, bytes, buflen );
+}
+
+static struct formatter_units speed_units;
+
+unsigned int tr_speed_K = 0u;
+
+void
+tr_formatter_speed_init( unsigned int kilo,
+                         const char * kb, const char * mb,
+                         const char * gb, const char * tb )
+{
+    tr_speed_K = kilo;
+    formatter_init( &speed_units, kilo, kb, mb, gb, tb );
+}
+
+char*
+tr_formatter_speed_KBps( char * buf, double KBps, size_t buflen )
+{
+    const double K = speed_units.units[TR_FMT_KB].value;
+    double speed = KBps;
+
+    if( speed <= 999.95 ) /* 0.0 KB to 999.9 KB */
+        tr_snprintf( buf, buflen, "%d %s", (int)speed, speed_units.units[TR_FMT_KB].name );
+    else {
+        speed /= K;
+        if( speed <= 99.995 ) /* 0.98 MB to 99.99 MB */
+            tr_snprintf( buf, buflen, "%.2f %s", speed, speed_units.units[TR_FMT_MB].name );
+        else if (speed <= 999.95) /* 100.0 MB to 999.9 MB */
+            tr_snprintf( buf, buflen, "%.1f %s", speed, speed_units.units[TR_FMT_MB].name );
+        else {
+            speed /= K;
+            tr_snprintf( buf, buflen, "%.1f %s", speed, speed_units.units[TR_FMT_GB].name );
+        }
+    }
+
+    return buf;
+}
+
+static struct formatter_units mem_units;
+
+unsigned int tr_mem_K = 0u;
+
+void
+tr_formatter_mem_init( unsigned int kilo,
+                       const char * kb, const char * mb,
+                       const char * gb, const char * tb )
+{
+    tr_mem_K = kilo;
+    formatter_init( &mem_units, kilo, kb, mb, gb, tb );
+}
+
+char*
+tr_formatter_mem_B( char * buf, int64_t bytes_per_second, size_t buflen )
+{
+    return formatter_get_size_str( &mem_units, buf, bytes_per_second, buflen );
+}
+
+void
+tr_formatter_get_units( tr_benc * d )
+{
+    int i;
+    tr_benc * l;
+
+    tr_bencDictReserve( d, 6 );
+
+    tr_bencDictAddInt( d, "memory-bytes", mem_units.units[TR_FMT_KB].value );
+    l = tr_bencDictAddList( d, "memory-units", 4 );
+    for( i=0; i<4; i++ ) tr_bencListAddStr( l, mem_units.units[i].name );
+
+    tr_bencDictAddInt( d, "size-bytes",   size_units.units[TR_FMT_KB].value );
+    l = tr_bencDictAddList( d, "size-units", 4 );
+    for( i=0; i<4; i++ ) tr_bencListAddStr( l, size_units.units[i].name );
+
+    tr_bencDictAddInt( d, "speed-bytes",  speed_units.units[TR_FMT_KB].value );
+    l = tr_bencDictAddList( d, "speed-units", 4 );
+    for( i=0; i<4; i++ ) tr_bencListAddStr( l, speed_units.units[i].name );
+}
+
