@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: AboutWindowController.m 9003 2009-08-29 00:09:18Z livings124 $
+ * $Id: AboutWindowController.m 11617 2011-01-01 20:42:14Z livings124 $
  *
- * Copyright (c) 2007-2009 Transmission authors and contributors
+ * Copyright (c) 2007-2011 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -47,14 +47,14 @@ AboutWindowController * fAboutBoxInstance = nil;
             [[NSBundle mainBundle] pathForResource: @"Credits" ofType: @"rtf"] documentAttributes: nil] autorelease]];
     
     //size license button
-    CGFloat oldButtonWidth = [fLicenseButton frame].size.width;
+    const CGFloat oldButtonWidth = NSWidth([fLicenseButton frame]);
     
     [fLicenseButton setTitle: NSLocalizedString(@"License", "About window -> license button")];
     [fLicenseButton sizeToFit];
     
     NSRect buttonFrame = [fLicenseButton frame];
-    buttonFrame.size.width += 10.0f;
-    buttonFrame.origin.x -= buttonFrame.size.width - oldButtonWidth;
+    buttonFrame.size.width += 10.0;
+    buttonFrame.origin.x -= NSWidth(buttonFrame) - oldButtonWidth;
     [fLicenseButton setFrame: buttonFrame];
 }
 
