@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: AddWindowController.h 8843 2009-07-20 00:40:01Z livings124 $
+ * $Id: AddWindowController.h 11617 2011-01-01 20:42:14Z livings124 $
  *
- * Copyright (c) 2008-2009 Transmission authors and contributors
+ * Copyright (c) 2008-2011 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -23,17 +23,18 @@
  *****************************************************************************/
 
 #import <Cocoa/Cocoa.h>
-#import "Torrent.h"
 
-@class FileOutlineController;
 @class Controller;
+@class FileOutlineController;
+@class Torrent;
 
 @interface AddWindowController : NSWindowController
 {
     IBOutlet NSImageView * fIconView, * fLocationImageView;
     IBOutlet NSTextField * fNameField, * fStatusField, * fLocationField;
     IBOutlet NSButton * fStartCheck, * fDeleteCheck;
-    IBOutlet NSPopUpButton * fGroupPopUp;
+    IBOutlet NSPopUpButton * fGroupPopUp, * fPriorityPopUp;
+    IBOutlet NSProgressIndicator * fVerifyIndicator;
     
     IBOutlet FileOutlineController * fFileController;
     
@@ -43,7 +44,7 @@
     NSString * fDestination, * fTorrentFile;
     BOOL fLockDestination;
     
-    BOOL fDeleteTorrent, fDeleteEnable;
+    BOOL fDeleteTorrentInitial, fDeleteEnableInitial;
     NSInteger fGroupValue;
     
     NSTimer * fTimer;
@@ -62,6 +63,8 @@
 
 - (void) verifyLocalData: (id) sender;
 
+- (void) changePriority: (id) sender;
+
 - (void) updateStatusField: (NSNotification *) notification;
 
 - (void) updateGroupMenu: (NSNotification *) notification;
