@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: FileNameCell.m 7659 2009-01-10 23:37:37Z livings124 $
+ * $Id: FileNameCell.m 12483 2011-05-31 22:26:04Z livings124 $
  * 
- * Copyright (c) 2007-2009 Transmission authors and contributors
+ * Copyright (c) 2007-2011 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -26,23 +26,27 @@
 #import "FileOutlineView.h"
 #import "Torrent.h"
 #import "FileListNode.h"
+#import "NSApplicationAdditions.h"
 #import "NSStringAdditions.h"
 
-#define PADDING_HORIZONAL 2.0f
-#define IMAGE_FOLDER_SIZE 16.0f
-#define IMAGE_ICON_SIZE 32.0f
-#define PADDING_BETWEEN_IMAGE_AND_TITLE 4.0f
-#define PADDING_ABOVE_TITLE_FILE 2.0f
-#define PADDING_BELOW_STATUS_FILE 2.0f
-#define PADDING_BETWEEN_NAME_AND_FOLDER_STATUS 4.0f
+#import "transmission.h" // required by utils.h
+#import "utils.h"
+
+#define PADDING_HORIZONAL 2.0
+#define IMAGE_FOLDER_SIZE 16.0
+#define IMAGE_ICON_SIZE 32.0
+#define PADDING_BETWEEN_IMAGE_AND_TITLE 4.0
+#define PADDING_ABOVE_TITLE_FILE 2.0
+#define PADDING_BELOW_STATUS_FILE 2.0
+#define PADDING_BETWEEN_NAME_AND_FOLDER_STATUS 4.0
 
 @interface FileNameCell (Private)
 
 - (NSRect) rectForTitleWithString: (NSAttributedString *) string inBounds: (NSRect) bounds;
 - (NSRect) rectForStatusWithString: (NSAttributedString *) string withTitleRect: (NSRect) titleRect inBounds: (NSRect) bounds;
 
-- (NSAttributedString *) attributedTitleWithColor: (NSColor *) color;
-- (NSAttributedString *) attributedStatusWithColor: (NSColor *) color;
+- (NSAttributedString *) attributedTitle;
+- (NSAttributedString *) attributedStatus;
 
 @end
 
@@ -53,17 +57,21 @@
     if ((self = [super init]))
     {
         NSMutableParagraphStyle * paragraphStyle = [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
-        [paragraphStyle setLineBreakMode: NSLineBreakByTruncatingTail];
+        [paragraphStyle setLineBreakMode: NSLineBreakByTruncatingMiddle];
         
         fTitleAttributes = [[NSMutableDictionary alloc] initWithObjectsAndKeys:
-                            [NSFont messageFontOfSize: 12.0f], NSFontAttributeName,
+                            [NSFont messageFontOfSize: 12.0], NSFontAttributeName,
                             paragraphStyle, NSParagraphStyleAttributeName, nil];
         
+        NSMutableParagraphStyle * statusParagraphStyle = [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
+        [statusParagraphStyle setLineBreakMode: NSLineBreakByTruncatingTail];
+        
         fStatusAttributes = [[NSMutableDictionary alloc] initWithObjectsAndKeys:
-                                [NSFont messageFontOfSize: 9.0f], NSFontAttributeName,
-                                paragraphStyle, NSParagraphStyleAttributeName, nil];
+                                [NSFont messageFontOfSize: 9.0], NSFontAttributeName,
+                                statusParagraphStyle, NSParagraphStyleAttributeName,  nil];
         
         [paragraphStyle release];
+        [statusParagraphStyle release];
     }
     return self;
 }
@@ -73,8 +81,6 @@
     [fTitleAttributes release];
     [fStatusAttributes release];
     
-    [fFolderImage release];
-    
     [super dealloc];
 }
 
@@ -85,28 +91,13 @@
     copy->fTitleAttributes = [fTitleAttributes retain];
     copy->fStatusAttributes = [fStatusAttributes retain];
     
-    copy->fFolderImage = [fFolderImage retain];
-    
     return copy;
 }
 
 - (NSImage *) image
 {
     FileListNode * node = (FileListNode *)[self objectValue];
-    NSImage * image;
-    if ([node isFolder])
-    {
-        if (!fFolderImage)
-        {
-            fFolderImage = [[[NSWorkspace sharedWorkspace] iconForFileType: NSFileTypeForHFSTypeCode('fldr')] copy];
-            [fFolderImage setFlipped: YES];
-        }
-        image = fFolderImage;
-    }
-    else
-        image = [node icon];
-    
-    return image;
+    return [node icon];
 }
 
 - (NSRect) imageRectForBounds: (NSRect) bounds
@@ -116,33 +107,29 @@
     result.origin.x += PADDING_HORIZONAL;
     
     const CGFloat IMAGE_SIZE = [(FileListNode *)[self objectValue] isFolder] ? IMAGE_FOLDER_SIZE : IMAGE_ICON_SIZE;
-    result.origin.y += (result.size.height - IMAGE_SIZE) * 0.5f;
+    result.origin.y += (result.size.height - IMAGE_SIZE) * 0.5;
     result.size = NSMakeSize(IMAGE_SIZE, IMAGE_SIZE);
     
     return result;
 }
 
-- (NSRect) titleRectForBounds: (NSRect) bounds
-{
-    return [self rectForTitleWithString: [self attributedTitleWithColor: nil] inBounds: bounds];
-}
-
-- (NSRect) statusRectForBounds: (NSRect) bounds
-{
-    return [self rectForStatusWithString: [self attributedStatusWithColor: nil]
-            withTitleRect: [(FileListNode *)[self objectValue] isFolder] ? [self titleRectForBounds: bounds] : NSZeroRect
-            inBounds: bounds];
-}
-
 - (void) drawWithFrame: (NSRect) cellFrame inView: (NSView *) controlView
 {
     //icon
-    [[self image] drawInRect: [self imageRectForBounds: cellFrame] fromRect: NSZeroRect operation: NSCompositeSourceOver fraction: 1.0];
+    if ([NSApp isOnSnowLeopardOrBetter])
+        [[self image] drawInRect: [self imageRectForBounds: cellFrame] fromRect: NSZeroRect operation: NSCompositeSourceOver fraction: 1.0
+            respectFlipped: YES hints: nil];
+    else
+    {
+        NSImage * image = [self image];
+        [image setFlipped: YES];
+        [image drawInRect: [self imageRectForBounds: cellFrame] fromRect: NSZeroRect operation: NSCompositeSourceOver fraction: 1.0];
+    }
     
     NSColor * titleColor, * statusColor;
     if ([self backgroundStyle] == NSBackgroundStyleDark)
         titleColor = statusColor = [NSColor whiteColor];
-    else if ([[(FileOutlineView *)[self controlView] torrent] checkForFiles: [(FileListNode *)[self objectValue] indexes]] == NSOffState)
+    else if ([[(FileListNode *)[self objectValue] torrent] checkForFiles: [(FileListNode *)[self objectValue] indexes]] == NSOffState)
         titleColor = statusColor = [NSColor disabledControlTextColor];
     else
     {
@@ -150,13 +137,16 @@
         statusColor = [NSColor darkGrayColor];
     }
     
+    [fTitleAttributes setObject: titleColor forKey: NSForegroundColorAttributeName];
+    [fStatusAttributes setObject: statusColor forKey: NSForegroundColorAttributeName];
+    
     //title
-    NSAttributedString * titleString = [self attributedTitleWithColor: titleColor];
+    NSAttributedString * titleString = [self attributedTitle];
     NSRect titleRect = [self rectForTitleWithString: titleString inBounds: cellFrame];
     [titleString drawInRect: titleRect];
     
     //status
-    NSAttributedString * statusString = [self attributedStatusWithColor: statusColor];
+    NSAttributedString * statusString = [self attributedStatus];
     NSRect statusRect = [self rectForStatusWithString: statusString withTitleRect: titleRect inBounds: cellFrame];
     [statusString drawInRect: statusRect];
 }
@@ -167,70 +157,63 @@
 
 - (NSRect) rectForTitleWithString: (NSAttributedString *) string inBounds: (NSRect) bounds
 {
-    NSSize titleSize = [string size];
-    
-    NSRect result = bounds;
+    const NSSize titleSize = [string size];
     
+    NSRect result;
     if (![(FileListNode *)[self objectValue] isFolder])
     {
-        result.origin.x += PADDING_HORIZONAL + IMAGE_ICON_SIZE + PADDING_BETWEEN_IMAGE_AND_TITLE;
-        result.origin.y += PADDING_ABOVE_TITLE_FILE;
+        result.origin.x = NSMinX(bounds) + PADDING_HORIZONAL + IMAGE_ICON_SIZE + PADDING_BETWEEN_IMAGE_AND_TITLE;
+        result.origin.y = NSMinY(bounds) + PADDING_ABOVE_TITLE_FILE;
+        result.size.width = NSMaxX(bounds) - NSMinX(result) - PADDING_HORIZONAL;
     }
     else
     {
-        result.origin.x += PADDING_HORIZONAL + IMAGE_FOLDER_SIZE + PADDING_BETWEEN_IMAGE_AND_TITLE;
-        result.origin.y += (result.size.height - titleSize.height) * 0.5f;
+        result.origin.x = NSMinX(bounds) + PADDING_HORIZONAL + IMAGE_FOLDER_SIZE + PADDING_BETWEEN_IMAGE_AND_TITLE;
+        result.origin.y = NSMidY(bounds) - titleSize.height * 0.5;
+        result.size.width = MIN(titleSize.width, NSMaxX(bounds) - NSMinX(result) - PADDING_HORIZONAL);
     }
-    result.size = titleSize;
-    result.size.width = MIN(result.size.width, NSMaxX(bounds) - result.origin.x - PADDING_HORIZONAL);
+    result.size.height = titleSize.height;
     
     return result;
 }
 
 - (NSRect) rectForStatusWithString: (NSAttributedString *) string withTitleRect: (NSRect) titleRect inBounds: (NSRect) bounds;
 {
-    NSSize statusSize = [string size];
+    const NSSize statusSize = [string size];
     
     NSRect result;
     if (![(FileListNode *)[self objectValue] isFolder])
     {
-        result.origin.x = bounds.origin.x + PADDING_HORIZONAL + IMAGE_ICON_SIZE + PADDING_BETWEEN_IMAGE_AND_TITLE;
+        result.origin.x = NSMinX(titleRect);
         result.origin.y = NSMaxY(bounds) - PADDING_BELOW_STATUS_FILE - statusSize.height;
+        result.size.width = NSWidth(titleRect);
     }
     else
     {
         result.origin.x = NSMaxX(titleRect) + PADDING_BETWEEN_NAME_AND_FOLDER_STATUS;
-        result.origin.y = NSMaxY(titleRect) - statusSize.height - 1.0f;
+        result.origin.y = NSMaxY(titleRect) - statusSize.height - 1.0;
+        result.size.width = NSMaxX(bounds) - NSMaxX(titleRect) - PADDING_HORIZONAL;
     }
-        
-    result.size = statusSize;
-    result.size.width = MIN(result.size.width, NSMaxX(bounds) - result.origin.x - PADDING_HORIZONAL);
+    result.size.height = statusSize.height;
     
     return result;
 }
 
-- (NSAttributedString *) attributedTitleWithColor: (NSColor *) color
+- (NSAttributedString *) attributedTitle
 {
-    if (color)
-        [fTitleAttributes setObject: color forKey: NSForegroundColorAttributeName];
-        
     NSString * title = [(FileListNode *)[self objectValue] name];
     return [[[NSAttributedString alloc] initWithString: title attributes: fTitleAttributes] autorelease];
 }
 
-- (NSAttributedString *) attributedStatusWithColor: (NSColor *) color
+- (NSAttributedString *) attributedStatus
 {
-    if (color)
-        [fStatusAttributes setObject: color forKey: NSForegroundColorAttributeName];
-    
-    Torrent * torrent = [(FileOutlineView *)[self controlView] torrent];
     FileListNode * node = (FileListNode *)[self objectValue];
+    Torrent * torrent = [node torrent];
     
-    CGFloat progress = [torrent fileProgress: node];
-    NSString * percentString = progress == 1.0f ? @"100%" : [NSString localizedStringWithFormat: @"%.2f%%", progress * 100.0f];
+    const CGFloat progress = [torrent fileProgress: node];
+    NSString * percentString = [NSString percentString: progress longDecimals: YES];
     
-    
-    NSString * status = [NSString localizedStringWithFormat: NSLocalizedString(@"%@ of %@",
+    NSString * status = [NSString stringWithFormat: NSLocalizedString(@"%@ of %@",
                             "Inspector -> Files tab -> file status string"), percentString, [NSString stringForFileSize: [node size]]];
     
     return [[[NSAttributedString alloc] initWithString: status attributes: fStatusAttributes] autorelease];
