@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: FileOutlineController.m 7005 2008-11-01 14:26:51Z livings124 $
+ * $Id: FileOutlineController.m 12483 2011-05-31 22:26:04Z livings124 $
  *
- * Copyright (c) 2008 Transmission authors and contributors
+ * Copyright (c) 2008-2011 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,10 +27,10 @@
 #import "FileOutlineView.h"
 #import "FilePriorityCell.h"
 #import "FileListNode.h"
-#import "QuickLookController.h"
 #import "NSApplicationAdditions.h"
+#import <Quartz/Quartz.h>
 
-#define ROW_SMALL_HEIGHT 18.0f
+#define ROW_SMALL_HEIGHT 18.0
 
 typedef enum
 {
@@ -59,48 +59,87 @@ typedef enum
     [fOutline setTarget: self];
     
     //set table header tool tips
-    if ([NSApp isOnLeopardOrBetter])
-    {
-        [[fOutline tableColumnWithIdentifier: @"Check"] setHeaderToolTip: NSLocalizedString(@"Download",
-                                                                            "file table -> header tool tip")];
-        [[fOutline tableColumnWithIdentifier: @"Priority"] setHeaderToolTip: NSLocalizedString(@"Priority",
-                                                                            "file table -> header tool tip")];
-    }
+    [[fOutline tableColumnWithIdentifier: @"Check"] setHeaderToolTip: NSLocalizedString(@"Download",
+                                                                        "file table -> header tool tip")];
+    [[fOutline tableColumnWithIdentifier: @"Priority"] setHeaderToolTip: NSLocalizedString(@"Priority",
+                                                                        "file table -> header tool tip")];
     
     [fOutline setMenu: [self menu]];
     
     [self setTorrent: nil];
 }
 
+- (void) dealloc
+{
+    [fFileList release];
+    [fFilterText release];
+    [super dealloc];
+}
+
 - (FileOutlineView *) outlineView
 {
     return fOutline;
 }
 
-- (void) outlineViewSelectionDidChange: (NSNotification *) notification
-{
-    [[QuickLookController quickLook] updateQuickLook];
-}
-
 - (void) setTorrent: (Torrent *) torrent
 {
     fTorrent = torrent;
-    [fOutline setTorrent: fTorrent];
+    
+    [fFileList release];
+    fFileList = [[fTorrent fileList] retain];
+    
+    [fFilterText release];
+    fFilterText = nil;
     
     [fOutline deselectAll: nil];
     [fOutline reloadData];
 }
 
+- (void) setFilterText: (NSString *) text
+{
+    if ([text isEqualToString: @""])
+        text = nil;
+    
+    if ((!text && !fFilterText) || (text && fFilterText && [text isEqualToString: fFilterText]))
+        return;
+    
+    [fFilterText release];
+    fFilterText = [text retain];
+    
+    [fFileList release];
+    if (!fFilterText)
+        fFileList = [[fTorrent fileList] retain];
+    else
+    {
+        NSMutableArray * list = [NSMutableArray arrayWithCapacity: [fTorrent fileCount]];
+        
+        for (FileListNode * node in [fTorrent flatFileList])
+            if ([[node name] rangeOfString: fFilterText options: (NSCaseInsensitiveSearch | NSDiacriticInsensitiveSearch)].location != NSNotFound)
+                [list addObject: node];
+        
+        fFileList = [[NSArray alloc] initWithArray: list];
+    }
+    
+    [fOutline reloadData];
+}
+
 - (void) reloadData
 {
     [fTorrent updateFileStat];
     [fOutline reloadData];
 }
 
+- (void) outlineViewSelectionDidChange: (NSNotification *) notification
+{
+    if ([NSApp isOnSnowLeopardOrBetter] && [QLPreviewPanelSL sharedPreviewPanelExists]
+        && [[QLPreviewPanelSL sharedPreviewPanel] isVisible])
+        [[QLPreviewPanelSL sharedPreviewPanel] reloadData];
+}
+
 - (NSInteger) outlineView: (NSOutlineView *) outlineView numberOfChildrenOfItem: (id) item
 {
     if (!item)
-        return fTorrent ? [[fTorrent fileList] count] : 0;
+        return fFileList ? [fFileList count] : 0;
     else
     {
         FileListNode * node = (FileListNode *)item;
@@ -115,13 +154,13 @@ typedef enum
 
 - (id) outlineView: (NSOutlineView *) outlineView child: (NSInteger) index ofItem: (id) item
 {
-    return [(item ? [(FileListNode *)item children] : [fTorrent fileList]) objectAtIndex: index];
+    return [(item ? [(FileListNode *)item children] : fFileList) objectAtIndex: index];
 }
 
 - (id) outlineView: (NSOutlineView *) outlineView objectValueForTableColumn: (NSTableColumn *) tableColumn byItem: (id) item
 {
     if ([[tableColumn identifier] isEqualToString: @"Check"])
-        return [NSNumber numberWithInt: [fTorrent checkForFiles: [(FileListNode *)item indexes]]];
+        return [NSNumber numberWithInteger: [fTorrent checkForFiles: [(FileListNode *)item indexes]]];
     else
         return item;
 }
@@ -149,7 +188,7 @@ typedef enum
     if ([identifier isEqualToString: @"Check"])
     {
         NSIndexSet * indexSet;
-        if ([[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask)
+        if (([NSApp isOnSnowLeopardOrBetter] ? [NSEvent modifierFlags] : [[NSApp currentEvent] modifierFlags]) & NSAlternateKeyMask)
             indexSet = [NSIndexSet indexSetWithIndexesInRange: NSMakeRange(0, [fTorrent fileCount])];
         else
             indexSet = [(FileListNode *)item indexes];
@@ -171,7 +210,12 @@ typedef enum
 {
     NSString * ident = [tableColumn identifier];
     if ([ident isEqualToString: @"Name"])
-        return [[fTorrent downloadFolder] stringByAppendingPathComponent: [(FileListNode *)item fullPath]];
+    {
+        NSString * path = [fTorrent fileLocation: item];
+        if (!path)
+            path = [[item path] stringByAppendingPathComponent: [item name]];
+        return path;
+    }
     else if ([ident isEqualToString: @"Check"])
     {
         switch ([cell state])
@@ -250,7 +294,7 @@ typedef enum
 
 - (void) setPriority: (id) sender
 {
-    NSInteger priority;
+    tr_priority_t priority;
     switch ([sender tag])
     {
         case FILE_PRIORITY_HIGH_TAG:
@@ -274,11 +318,29 @@ typedef enum
 
 - (void) revealFile: (id) sender
 {
-    NSString * folder = [fTorrent downloadFolder];
     NSIndexSet * indexes = [fOutline selectedRowIndexes];
-    for (NSInteger i = [indexes firstIndex]; i != NSNotFound; i = [indexes indexGreaterThanIndex: i])
-        [[NSWorkspace sharedWorkspace] selectFile: [folder stringByAppendingPathComponent:
-            [[fOutline itemAtRow: i] fullPath]] inFileViewerRootedAtPath: nil];
+    if ([NSApp isOnSnowLeopardOrBetter])
+    {
+        NSMutableArray * paths = [NSMutableArray arrayWithCapacity: [indexes count]];
+        for (NSUInteger i = [indexes firstIndex]; i != NSNotFound; i = [indexes indexGreaterThanIndex: i])
+        {
+            NSString * path = [fTorrent fileLocation: [fOutline itemAtRow: i]];
+            if (path)
+                [paths addObject: [NSURL fileURLWithPath: path]];
+        }
+        
+        if ([paths count])
+            [[NSWorkspace sharedWorkspace] activateFileViewerSelectingURLs: paths];
+    }
+    else
+    {
+        for (NSUInteger i = [indexes firstIndex]; i != NSNotFound; i = [indexes indexGreaterThanIndex: i])
+        {
+            NSString * path = [fTorrent fileLocation: [fOutline itemAtRow: i]];
+            if (path)
+                [[NSWorkspace sharedWorkspace] selectFile: path inFileViewerRootedAtPath: nil];
+        }
+    }
 }
 
 #warning make real view controller (Leopard-only) so that Command-R will work
@@ -291,18 +353,16 @@ typedef enum
     
     if (action == @selector(revealFile:))
     {
-        NSString * downloadFolder = [fTorrent downloadFolder];
         NSIndexSet * indexSet = [fOutline selectedRowIndexes];
         for (NSInteger i = [indexSet firstIndex]; i != NSNotFound; i = [indexSet indexGreaterThanIndex: i])
-            if ([[NSFileManager defaultManager] fileExistsAtPath:
-                    [downloadFolder stringByAppendingPathComponent: [[[fTorrent fileList] objectAtIndex: i] fullPath]]])
+            if ([fTorrent fileLocation: [fOutline itemAtRow: i]] != nil)
                 return YES;
         return NO;
     }
     
     if (action == @selector(setCheck:))
     {
-        if ([fOutline numberOfSelectedRows] <= 0)
+        if ([fOutline numberOfSelectedRows] == 0)
             return NO;
         
         NSIndexSet * indexSet = [fOutline selectedRowIndexes];
@@ -316,20 +376,20 @@ typedef enum
     
     if (action == @selector(setOnlySelectedCheck:))
     {
-        if ([fOutline numberOfSelectedRows] <= 0)
+        if ([fOutline numberOfSelectedRows] == 0)
             return NO;
         
         NSIndexSet * indexSet = [fOutline selectedRowIndexes];
         NSMutableIndexSet * itemIndexes = [NSMutableIndexSet indexSet];
         for (NSInteger i = [indexSet firstIndex]; i != NSNotFound; i = [indexSet indexGreaterThanIndex: i])
             [itemIndexes addIndexes: [[fOutline itemAtRow: i] indexes]];
-            
+        
         return [fTorrent canChangeDownloadCheckForFiles: itemIndexes];
     }
     
     if (action == @selector(setPriority:))
     {
-        if ([fOutline numberOfSelectedRows] <= 0)
+        if ([fOutline numberOfSelectedRows] == 0)
         {
             [menuItem setState: NSOffState];
             return NO;
@@ -337,7 +397,7 @@ typedef enum
         
         //determine which priorities are checked
         NSIndexSet * indexSet = [fOutline selectedRowIndexes];
-        NSInteger priority;
+        tr_priority_t priority;
         switch ([menuItem tag])
         {
             case FILE_PRIORITY_HIGH_TAG:
@@ -348,13 +408,13 @@ typedef enum
                 break;
             case FILE_PRIORITY_LOW_TAG:
                 priority = TR_PRI_LOW;
+                break;
         }
         
         BOOL current = NO, canChange = NO;
-        NSIndexSet * fileIndexSet;
         for (NSInteger i = [indexSet firstIndex]; i != NSNotFound; i = [indexSet indexGreaterThanIndex: i])
         {
-            fileIndexSet = [[fOutline itemAtRow: i] indexes];
+            NSIndexSet * fileIndexSet = [[fOutline itemAtRow: i] indexes];
             if (![fTorrent canChangeDownloadCheckForFiles: fileIndexSet])
                 continue;
             
@@ -441,7 +501,7 @@ typedef enum
     [menu addItem: [NSMenuItem separatorItem]];
     
     //reveal in finder
-    item = [[NSMenuItem alloc] initWithTitle: NSLocalizedString(@"Reveal in Finder", "File Outline -> Menu")
+    item = [[NSMenuItem alloc] initWithTitle: NSLocalizedString(@"Show in Finder", "File Outline -> Menu")
             action: @selector(revealFile:) keyEquivalent: @""];
     [item setTarget: self];
     [menu addItem: item];
