@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: FileOutlineView.h 5382 2008-03-25 03:17:35Z livings124 $
+ * $Id: FileOutlineView.h 12483 2011-05-31 22:26:04Z livings124 $
  * 
- * Copyright (c) 2007-2008 Transmission authors and contributors
+ * Copyright (c) 2007-2011 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -25,20 +25,14 @@
 #import <Cocoa/Cocoa.h>
 
 @class Torrent;
-@class CTGradient;
 
 @interface FileOutlineView : NSOutlineView
 {
-    Torrent * fTorrent;
-    
-    CTGradient * fHighPriorityGradient, * fLowPriorityGradient, * fMixedPriorityGradient;
-    
-    int fMouseRow;
+    NSInteger fMouseRow;
 }
 
-- (void) setTorrent: (Torrent *) torrent;
-- (Torrent *) torrent;
+- (NSRect) iconRectForRow: (int) row;
 
-- (int) hoveredRow;
+- (NSInteger) hoveredRow;
 
 @end
