@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: GroupsController.m 7659 2009-01-10 23:37:37Z livings124 $
+ * $Id: GroupsController.m 11617 2011-01-01 20:42:14Z livings124 $
  *
- * Copyright (c) 2007-2009 Transmission authors and contributors
+ * Copyright (c) 2007-2011 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -66,37 +66,37 @@ GroupsController * fGroupsInstance = nil;
             NSMutableDictionary * red = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                             [NSColor redColor], @"Color",
                                             NSLocalizedString(@"Red", "Groups -> Name"), @"Name",
-                                            [NSNumber numberWithInt: 0], @"Index", nil];
+                                            [NSNumber numberWithInteger: 0], @"Index", nil];
             
             NSMutableDictionary * orange = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                             [NSColor orangeColor], @"Color",
                                             NSLocalizedString(@"Orange", "Groups -> Name"), @"Name",
-                                            [NSNumber numberWithInt: 1], @"Index", nil];
+                                            [NSNumber numberWithInteger: 1], @"Index", nil];
             
             NSMutableDictionary * yellow = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                             [NSColor yellowColor], @"Color",
                                             NSLocalizedString(@"Yellow", "Groups -> Name"), @"Name",
-                                            [NSNumber numberWithInt: 2], @"Index", nil];
+                                            [NSNumber numberWithInteger: 2], @"Index", nil];
             
             NSMutableDictionary * green = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                             [NSColor greenColor], @"Color",
                                             NSLocalizedString(@"Green", "Groups -> Name"), @"Name",
-                                            [NSNumber numberWithInt: 3], @"Index", nil];
+                                            [NSNumber numberWithInteger: 3], @"Index", nil];
             
             NSMutableDictionary * blue = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                             [NSColor blueColor], @"Color",
                                             NSLocalizedString(@"Blue", "Groups -> Name"), @"Name",
-                                            [NSNumber numberWithInt: 4], @"Index", nil];
+                                            [NSNumber numberWithInteger: 4], @"Index", nil];
             
             NSMutableDictionary * purple = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                             [NSColor purpleColor], @"Color",
                                             NSLocalizedString(@"Purple", "Groups -> Name"), @"Name",
-                                            [NSNumber numberWithInt: 5], @"Index", nil];
+                                            [NSNumber numberWithInteger: 5], @"Index", nil];
             
             NSMutableDictionary * gray = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                             [NSColor grayColor], @"Color",
                                             NSLocalizedString(@"Gray", "Groups -> Name"), @"Name",
-                                            [NSNumber numberWithInt: 6], @"Index", nil];
+                                            [NSNumber numberWithInteger: 6], @"Index", nil];
             
             fGroups = [[NSMutableArray alloc] initWithObjects: red, orange, yellow, green, blue, purple, gray, nil];
             [self saveGroups]; //make sure this is saved right away
@@ -122,7 +122,7 @@ GroupsController * fGroupsInstance = nil;
     if (index != -1)
     {
         for (NSInteger i = 0; i < [fGroups count]; i++)
-            if (index == [[[fGroups objectAtIndex: i] objectForKey: @"Index"] intValue])
+            if (index == [[[fGroups objectAtIndex: i] objectForKey: @"Index"] integerValue])
                 return i;
     }
     return -1;
@@ -130,7 +130,7 @@ GroupsController * fGroupsInstance = nil;
 
 - (NSInteger) indexForRow: (NSInteger) row
 {
-    return [[[fGroups objectAtIndex: row] objectForKey: @"Index"] intValue];
+    return [[[fGroups objectAtIndex: row] objectForKey: @"Index"] integerValue];
 }
 
 - (NSString *) nameForIndex: (NSInteger) index
@@ -251,23 +251,14 @@ GroupsController * fGroupsInstance = nil;
 - (void) addNewGroup
 {
     //find the lowest index
-    NSInteger index;
-    for (index = 0; index < [fGroups count]; index++)
-    {
-        BOOL found = NO;
-        for (NSDictionary * dict in fGroups)
-            if ([[dict objectForKey: @"Index"] intValue] == index)
-            {
-                found = YES;
-                break;
-            }
-        
-        if (!found)
-            break;
-    }
+    NSMutableIndexSet * candidates = [NSMutableIndexSet indexSetWithIndexesInRange: NSMakeRange(0, [fGroups count]+1)];
+    for (NSDictionary * dict in fGroups)
+        [candidates removeIndex: [[dict objectForKey: @"Index"] integerValue]];
     
-    [fGroups addObject: [NSMutableDictionary dictionaryWithObjectsAndKeys: [NSNumber numberWithInt: index], @"Index",
-                            [NSColor cyanColor], @"Color", @"", @"Name", nil]];
+    const NSInteger index = [candidates firstIndex];
+    
+    [fGroups addObject: [NSMutableDictionary dictionaryWithObjectsAndKeys: [NSNumber numberWithInteger: index], @"Index",
+                            [NSColor colorWithCalibratedRed: 0.0 green: 0.65 blue: 1.0 alpha: 1.0], @"Color", @"", @"Name", nil]];
     
     [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateGroups" object: self];
     [self saveGroups];
@@ -275,11 +266,11 @@ GroupsController * fGroupsInstance = nil;
 
 - (void) removeGroupWithRowIndex: (NSInteger) row
 {
-    NSInteger index = [[[fGroups objectAtIndex: row] objectForKey: @"Index"] intValue];
+    NSInteger index = [[[fGroups objectAtIndex: row] objectForKey: @"Index"] integerValue];
     [fGroups removeObjectAtIndex: row];
     
     [[NSNotificationCenter defaultCenter] postNotificationName: @"GroupValueRemoved" object: self userInfo:
-        [NSDictionary dictionaryWithObject: [NSNumber numberWithInt: index] forKey: @"Index"]];
+        [NSDictionary dictionaryWithObject: [NSNumber numberWithInteger: index] forKey: @"Index"]];
     
     if (index == [[NSUserDefaults standardUserDefaults] integerForKey: @"FilterGroup"])
         [[NSUserDefaults standardUserDefaults] setInteger: -2 forKey: @"FilterGroup"];
@@ -319,7 +310,6 @@ GroupsController * fGroupsInstance = nil;
     if (small)
     {
         icon = [icon copy];
-        [icon setScalesWhenResized: YES];
         [icon setSize: NSMakeSize(ICON_WIDTH_SMALL, ICON_WIDTH_SMALL)];
         
         [item setImage: icon];
@@ -336,13 +326,12 @@ GroupsController * fGroupsInstance = nil;
         item = [[NSMenuItem alloc] initWithTitle: [dict objectForKey: @"Name"] action: action keyEquivalent: @""];
         [item setTarget: target];
         
-        [item setTag: [[dict objectForKey: @"Index"] intValue]];
+        [item setTag: [[dict objectForKey: @"Index"] integerValue]];
         
         NSImage * icon = [self imageForGroup: dict];
         if (small)
         {
             icon = [icon copy];
-            [icon setScalesWhenResized: YES];
             [icon setSize: NSMakeSize(ICON_WIDTH_SMALL, ICON_WIDTH_SMALL)];
             
             [item setImage: icon];
@@ -362,7 +351,7 @@ GroupsController * fGroupsInstance = nil;
 {
     for (NSDictionary * group in fGroups)
     {
-        NSInteger row = [[group objectForKey: @"Index"] intValue];
+        NSInteger row = [[group objectForKey: @"Index"] integerValue];
         if ([self torrent: torrent doesMatchRulesForGroupAtIndex: row])
             return row;
     }
