@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: GroupsPrefsController.m 9002 2009-08-28 23:40:11Z livings124 $
+ * $Id: GroupsPrefsController.m 11617 2011-01-01 20:42:14Z livings124 $
  *
- * Copyright (c) 2007-2009 Transmission authors and contributors
+ * Copyright (c) 2007-2011 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -35,6 +35,7 @@
 @interface GroupsPrefsController (Private)
 
 - (void) updateSelectedGroup;
+- (void) refreshCustomLocationWithSingleGroup;
 
 @end
 
@@ -167,9 +168,16 @@
                         
             [fTableView reloadData];
             
-            NSInteger selectedRow = [fTableView selectedRow];
-            if (selectedRow != -1)
-                [fTableView scrollRowToVisible: selectedRow];
+            if ([fTableView numberOfRows] > 0)
+            {
+                if (row == [fTableView numberOfRows])
+                {
+                    --row;
+                    [fTableView selectRowIndexes: [NSIndexSet indexSetWithIndex: row] byExtendingSelection: NO];
+                }
+                
+                [fTableView scrollRowToVisible: row];
+            }
             
             break;
     }
@@ -216,18 +224,15 @@
         NSString * path = [[openPanel filenames] objectAtIndex: 0];
         [[GroupsController groups] setCustomDownloadLocation: path forIndex: index];
         [[GroupsController groups] setUsesCustomDownloadLocation: YES forIndex: index];
-        [self updateSelectedGroup]; //update the popup's icon/title
     }
     else
     {
         if (![[GroupsController groups] customDownloadLocationForIndex: index])
-        {
             [[GroupsController groups] setUsesCustomDownloadLocation: NO forIndex: index];
-            [fCustomLocationEnableCheck setState: NSOffState];
-            [fCustomLocationPopUp setEnabled: NO];
-        }
     }
-
+    
+    [self refreshCustomLocationWithSingleGroup];
+    
     [fCustomLocationPopUp selectItemAtIndex: 0];
 }
 
@@ -297,13 +302,11 @@
 
 - (void) ruleEditorRowsDidChange: (NSNotification *) notification
 {
-    CGFloat rowHeight        = [fRuleEditor rowHeight];
-    NSInteger numberOfRows   = [fRuleEditor numberOfRows];
-    CGFloat ruleEditorHeight = numberOfRows * rowHeight;
-    CGFloat heightDifference = ruleEditorHeight - [fRuleEditor frame].size.height;
-    NSRect windowFrame       = [fRuleEditor window].frame;
+    const CGFloat heightDifference = [fRuleEditor numberOfRows] * [fRuleEditor rowHeight] - [fRuleEditor frame].size.height;
+    NSRect windowFrame = [fRuleEditor window].frame;
     windowFrame.size.height += heightDifference;
-    windowFrame.origin.y    -= heightDifference;
+    windowFrame.origin.y -= heightDifference;
+    
     [fRuleEditor.window setFrame: windowFrame display: YES animate: YES];
 }
 
@@ -316,27 +319,13 @@
     [fAddRemoveControl setEnabled: [fTableView numberOfSelectedRows] > 0 forSegment: REMOVE_TAG];
     if ([fTableView numberOfSelectedRows] == 1)
     {
-        NSInteger index = [[GroupsController groups] indexForRow: [fTableView selectedRow]];
+        const NSInteger index = [[GroupsController groups] indexForRow: [fTableView selectedRow]];
         [fSelectedColorView setColor: [[GroupsController groups] colorForIndex: index]];
         [fSelectedColorView setEnabled: YES];
         [fSelectedColorNameField setStringValue: [[GroupsController groups] nameForIndex: index]];
         [fSelectedColorNameField setEnabled: YES];
-        [fCustomLocationEnableCheck setState: [[GroupsController groups] usesCustomDownloadLocationForIndex: index]];
-        [fCustomLocationEnableCheck setEnabled: YES];
-        [fCustomLocationPopUp setEnabled: [fCustomLocationEnableCheck state] == NSOnState];
-        if ([[GroupsController groups] customDownloadLocationForIndex: index])
-        {
-            NSString * location = [[GroupsController groups] customDownloadLocationForIndex: index];
-            ExpandedPathToPathTransformer * pathTransformer = [[[ExpandedPathToPathTransformer alloc] init] autorelease];
-            [[fCustomLocationPopUp itemAtIndex: 0] setTitle: [pathTransformer transformedValue: location]];
-            ExpandedPathToIconTransformer * iconTransformer = [[[ExpandedPathToIconTransformer alloc] init] autorelease];
-            [[fCustomLocationPopUp itemAtIndex: 0] setImage: [iconTransformer transformedValue: location]];
-        }
-        else
-        {
-            [[fCustomLocationPopUp itemAtIndex: 0] setTitle: @""];
-            [[fCustomLocationPopUp itemAtIndex: 0] setImage: nil];
-        }
+        
+        [self refreshCustomLocationWithSingleGroup];
 
         [fAutoAssignRulesEnableCheck setState: [[GroupsController groups] usesAutoAssignRulesForIndex: index]];
         [fAutoAssignRulesEnableCheck setEnabled: YES];
@@ -355,4 +344,28 @@
     }
 }
 
+- (void) refreshCustomLocationWithSingleGroup
+{
+    const NSInteger index = [[GroupsController groups] indexForRow: [fTableView selectedRow]];
+    
+    const BOOL hasCustomLocation = [[GroupsController groups] usesCustomDownloadLocationForIndex: index];
+    [fCustomLocationEnableCheck setState: hasCustomLocation];
+    [fCustomLocationEnableCheck setEnabled: YES];
+    [fCustomLocationPopUp setEnabled: hasCustomLocation];
+    
+    NSString * location = [[GroupsController groups] customDownloadLocationForIndex: index];
+    if (location)
+    {
+        ExpandedPathToPathTransformer * pathTransformer = [[[ExpandedPathToPathTransformer alloc] init] autorelease];
+        [[fCustomLocationPopUp itemAtIndex: 0] setTitle: [pathTransformer transformedValue: location]];
+        ExpandedPathToIconTransformer * iconTransformer = [[[ExpandedPathToIconTransformer alloc] init] autorelease];
+        [[fCustomLocationPopUp itemAtIndex: 0] setImage: [iconTransformer transformedValue: location]];
+    }
+    else
+    {
+        [[fCustomLocationPopUp itemAtIndex: 0] setTitle: @""];
+        [[fCustomLocationPopUp itemAtIndex: 0] setImage: nil];
+    }
+}
+
 @end
