@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: StringAdditions.h 2869 2007-08-19 03:03:28Z livings124 $
+ * $Id: NSStringAdditions.h 11647 2011-01-08 05:11:28Z livings124 $
  *
- * Copyright (c) 2005-2009 Transmission authors and contributors
+ * Copyright (c) 2005-2011 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -29,12 +29,17 @@
 + (NSString *) ellipsis;
 - (NSString *) stringByAppendingEllipsis;
 
++ (NSString *) formattedUInteger: (NSUInteger) value;
+
 + (NSString *) stringForFileSize: (uint64_t) size;
++ (NSString *) stringForFilePartialSize: (uint64_t) partialSize fullSize: (uint64_t) fullSize;
 
 + (NSString *) stringForSpeed: (CGFloat) speed;
 + (NSString *) stringForSpeedAbbrev: (CGFloat) speed;
 + (NSString *) stringForRatio: (CGFloat) ratio;
 
++ (NSString *) percentString: (CGFloat) progress longDecimals: (BOOL) longDecimals;
+
 + (NSString *) timeString: (uint64_t) seconds showSeconds: (BOOL) showSeconds;
 + (NSString *) timeString: (uint64_t) seconds showSeconds: (BOOL) showSeconds maxFields: (NSUInteger) max;
 
