@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: PeerTableView.m 7659 2009-01-10 23:37:37Z livings124 $
+ * $Id: PeerTableView.m 11617 2011-01-01 20:42:14Z livings124 $
  * 
- * Copyright (c) 2007-2009 Transmission authors and contributors
+ * Copyright (c) 2007-2011 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -23,6 +23,7 @@
  *****************************************************************************/
 
 #import "PeerTableView.h"
+#import "NSApplicationAdditions.h"
 
 @implementation PeerTableView
 
@@ -33,7 +34,15 @@
     {
         [[NSUserDefaults standardUserDefaults] setBool: ![[NSUserDefaults standardUserDefaults]
             boolForKey: @"DisplayPeerProgressBarNumber"] forKey: @"DisplayPeerProgressBarNumber"];
-        [self reloadData];
+        
+        if ([NSApp isOnSnowLeopardOrBetter])
+        {
+            NSIndexSet * rowIndexes = [NSIndexSet indexSetWithIndexesInRange: NSMakeRange(0, [self numberOfRows])],
+                        * columnIndexes = [NSIndexSet indexSetWithIndex: [self columnAtPoint: point]];
+            [self reloadDataForRowIndexes: rowIndexes columnIndexes: columnIndexes];
+        }
+        else
+            [self reloadData];
     }
 }
 
