@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: PrefsController.m 11105 2010-08-03 23:57:11Z livings124 $
+ * $Id: PrefsController.m 12468 2011-05-28 01:53:37Z livings124 $
  *
- * Copyright (c) 2005-2010 Transmission authors and contributors
+ * Copyright (c) 2005-2011 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -39,10 +39,6 @@
 #define DOWNLOAD_FOLDER     0
 #define DOWNLOAD_TORRENT    2
 
-#define PROXY_HTTP      0
-#define PROXY_SOCKS4    1
-#define PROXY_SOCKS5    2
-
 #define RPC_IP_ADD_TAG      0
 #define RPC_IP_REMOVE_TAG   1
 
@@ -54,13 +50,10 @@
 #define TOOLBAR_NETWORK     @"TOOLBAR_NETWORK"
 #define TOOLBAR_REMOTE      @"TOOLBAR_REMOTE"
 
-#define PROXY_KEYCHAIN_SERVICE  "Transmission:Proxy"
-#define PROXY_KEYCHAIN_NAME     "Proxy"
-
 #define RPC_KEYCHAIN_SERVICE    "Transmission:Remote"
 #define RPC_KEYCHAIN_NAME       "Remote"
 
-#define WEBUI_URL   @"http://localhost:%d/transmission/web/"
+#define WEBUI_URL   @"http://localhost:%d/"
 
 @interface PrefsController (Private)
 
@@ -69,6 +62,7 @@
 - (void) folderSheetClosed: (NSOpenPanel *) openPanel returnCode: (int) code contextInfo: (void *) info;
 - (void) incompleteFolderSheetClosed: (NSOpenPanel *) openPanel returnCode: (int) code contextInfo: (void *) info;
 - (void) importFolderSheetClosed: (NSOpenPanel *) openPanel returnCode: (int) code contextInfo: (void *) info;
+- (void) doneScriptSheetClosed: (NSOpenPanel *) openPanel returnCode: (int) code contextInfo: (void *) info;
 
 - (void) setKeychainPassword: (const char *) password forService: (const char *) service username: (const char *) username;
 
@@ -103,6 +97,21 @@ tr_session * fHandle;
             [fDefaults removeObjectForKey: @"DownloadChoice"];
         }
         
+        //check for old version blocklist (before 2.12)
+        NSDate * blocklistDate;
+        if ((blocklistDate = [fDefaults objectForKey: @"BlocklistLastUpdate"]))
+        {
+            [fDefaults setObject: blocklistDate forKey: @"BlocklistNewLastUpdateSuccess"];
+            [fDefaults setObject: blocklistDate forKey: @"BlocklistNewLastUpdate"];
+            [fDefaults removeObjectForKey: @"BlocklistLastUpdate"];
+            
+            NSString * blocklistDir = [NSHomeDirectory() stringByAppendingPathComponent:
+                                        @"/Library/Application Support/Transmission/blocklists/"];
+            [[NSFileManager defaultManager] moveItemAtPath: [blocklistDir stringByAppendingPathComponent: @"level1.bin"]
+                toPath: [blocklistDir stringByAppendingPathComponent: [NSString stringWithUTF8String: DEFAULT_BLOCKLIST_FILENAME]]
+                error: nil];
+        }
+        
         //save a new random port
         if ([fDefaults boolForKey: @"RandomPort"])
             [fDefaults setInteger: tr_sessionGetPeerPort(fHandle) forKey: @"BindPort"];
@@ -118,10 +127,6 @@ tr_session * fHandle;
         //set encryption
         [self setEncryptionMode: nil];
         
-        //set proxy type
-        [self updateProxyType];
-        [self updateProxyPassword];
-        
         //update rpc whitelist
         [self updateRPCPassword];
         
@@ -183,6 +188,9 @@ tr_session * fHandle;
     //set stop ratio
     [fRatioStopField setFloatValue: [fDefaults floatForKey: @"RatioLimit"]];
     
+    //set idle seeding minutes
+    [fIdleStopField setIntegerValue: [fDefaults integerForKey: @"IdleLimitMinutes"]];
+    
     //set limits
     [self updateLimitFields];
     
@@ -207,36 +215,20 @@ tr_session * fHandle;
     [fQueueSeedField setIntValue: [fDefaults integerForKey: @"QueueSeedNumber"]];
     [fStalledField setIntValue: [fDefaults integerForKey: @"StalledMinutes"]];
     
-    //set proxy type
-    [fProxyAddressField setStringValue: [fDefaults stringForKey: @"ProxyAddress"]];
-    NSInteger proxyType;
-    switch (tr_sessionGetProxyType(fHandle))
-    {
-        case TR_PROXY_SOCKS4:
-            proxyType = PROXY_SOCKS4;
-            break;
-        case TR_PROXY_SOCKS5:
-            proxyType = PROXY_SOCKS5;
-            break;
-        case TR_PROXY_HTTP:
-            proxyType = PROXY_HTTP;
-            break;
-        default:
-            NSAssert(NO, @"Unknown proxy type received");
-    }
-    [fProxyTypePopUp selectItemAtIndex: proxyType];
-    
-    //set proxy password - does NOT need to be released
-    [fProxyPasswordField setStringValue: [NSString stringWithUTF8String: tr_sessionGetProxyPassword(fHandle)]];
-    
-    //set proxy port
-    [fProxyPortField setIntValue: [fDefaults integerForKey: @"ProxyPort"]];
-    
     //set blocklist
+    NSString * blocklistURL = [fDefaults stringForKey: @"BlocklistURL"];
+    if (blocklistURL)
+        [fBlocklistURLField setStringValue: blocklistURL];
+    
+    [self updateBlocklistButton];
     [self updateBlocklistFields];
+    
     [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(updateBlocklistFields)
         name: @"BlocklistUpdated" object: nil];
     
+    [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(updateBlocklistURLField)
+        name: NSControlTextDidChangeNotification object: fBlocklistURLField];
+    
     //set rpc port
     [fRPCPortField setIntValue: [fDefaults integerForKey: @"RPCPort"]];
     
@@ -417,6 +409,9 @@ tr_session * fHandle;
             [fPortStatusField setStringValue: NSLocalizedString(@"Port check site is down", "Preferences -> Network -> port status")];
             [fPortStatusImage setImage: [NSImage imageNamed: @"YellowDot.png"]];
             break;
+        default:
+            NSAssert1(NO, @"Port checker returned invalid status: %d", [fPortChecker status]);
+            break;
     }
     [fPortChecker release];
     fPortChecker = nil;
@@ -457,6 +452,11 @@ tr_session * fHandle;
         [sound play];
 }
 
+- (void) setUTP: (id) sender
+{
+    tr_sessionSetUTPEnabled(fHandle, [fDefaults boolForKey: @"UTPGlobal"]);
+}
+
 - (void) setPeersGlobal: (id) sender
 {
     const int count = [sender intValue];
@@ -495,11 +495,11 @@ tr_session * fHandle;
 
 - (void) setBlocklistEnabled: (id) sender
 {
-    const BOOL enable = [sender state] == NSOnState;
-    [fDefaults setBool: enable forKey: @"Blocklist"];
-    tr_blocklistSetEnabled(fHandle, enable);
+    tr_blocklistSetEnabled(fHandle, [fDefaults boolForKey: @"BlocklistNew"]);
     
     [[BlocklistScheduler scheduler] updateSchedule];
+    
+    [self updateBlocklistButton];
 }
 
 - (void) updateBlocklist: (id) sender
@@ -518,12 +518,7 @@ tr_session * fHandle;
     
     if (exists)
     {
-        NSNumberFormatter * numberFormatter = [[NSNumberFormatter alloc] init];
-        [numberFormatter setNumberStyle: NSNumberFormatterDecimalStyle];
-        [numberFormatter setMaximumFractionDigits: 0];
-        NSString * countString = [numberFormatter stringFromNumber: [NSNumber numberWithInt: tr_blocklistGetRuleCount(fHandle)]];
-        [numberFormatter release];
-        
+        NSString * countString = [NSString formattedUInteger: tr_blocklistGetRuleCount(fHandle)];
         [fBlocklistMessageField setStringValue: [NSString stringWithFormat: NSLocalizedString(@"%@ IP address rules in list",
             "Prefs -> blocklist -> message"), countString]];
     }
@@ -531,13 +526,11 @@ tr_session * fHandle;
         [fBlocklistMessageField setStringValue: NSLocalizedString(@"A blocklist must first be downloaded",
             "Prefs -> blocklist -> message")];
     
-    [fBlocklistEnableCheck setEnabled: exists];
-    [fBlocklistEnableCheck setState: exists && [fDefaults boolForKey: @"Blocklist"]];
-    
     NSString * updatedDateString;
     if (exists)
     {
-        NSDate * updatedDate = [fDefaults objectForKey: @"BlocklistLastUpdate"];
+        NSDate * updatedDate = [fDefaults objectForKey: @"BlocklistNewLastUpdateSuccess"];
+        
         if (updatedDate)
         {
             if ([NSApp isOnSnowLeopardOrBetter])
@@ -563,6 +556,24 @@ tr_session * fHandle;
         NSLocalizedString(@"Last updated", "Prefs -> blocklist -> message"), updatedDateString]];
 }
 
+- (void) updateBlocklistURLField
+{
+    NSString * blocklistString = [fBlocklistURLField stringValue];
+    
+    [fDefaults setObject: blocklistString forKey: @"BlocklistURL"];
+    tr_blocklistSetURL(fHandle, [blocklistString UTF8String]);
+    
+    [self updateBlocklistButton];
+}
+
+- (void) updateBlocklistButton
+{
+    NSString * blocklistString = [fDefaults objectForKey: @"BlocklistURL"];
+    const BOOL enable = (blocklistString && ![blocklistString isEqualToString: @""])
+                            && [fDefaults boolForKey: @"BlocklistNew"];
+    [fBlocklistButton setEnabled: enable];
+}
+
 - (void) setAutoStartDownloads: (id) sender
 {
     tr_sessionSetPaused(fHandle, ![fDefaults boolForKey: @"AutoStartDownload"]);
@@ -571,27 +582,36 @@ tr_session * fHandle;
 - (void) applySpeedSettings: (id) sender
 {
     tr_sessionLimitSpeed(fHandle, TR_UP, [fDefaults boolForKey: @"CheckUpload"]);
-    tr_sessionSetSpeedLimit(fHandle, TR_UP, [fDefaults integerForKey: @"UploadLimit"]);
+    tr_sessionSetSpeedLimit_KBps(fHandle, TR_UP, [fDefaults integerForKey: @"UploadLimit"]);
     
     tr_sessionLimitSpeed(fHandle, TR_DOWN, [fDefaults boolForKey: @"CheckDownload"]);
-    tr_sessionSetSpeedLimit(fHandle, TR_DOWN, [fDefaults integerForKey: @"DownloadLimit"]);
+    tr_sessionSetSpeedLimit_KBps(fHandle, TR_DOWN, [fDefaults integerForKey: @"DownloadLimit"]);
     
     [[NSNotificationCenter defaultCenter] postNotificationName: @"SpeedLimitUpdate" object: nil];
 }
 
 - (void) applyAltSpeedSettings
 {
-    tr_sessionSetAltSpeed(fHandle, TR_UP, [fDefaults integerForKey: @"SpeedLimitUploadLimit"]);
-    tr_sessionSetAltSpeed(fHandle, TR_DOWN, [fDefaults integerForKey: @"SpeedLimitDownloadLimit"]);
+    tr_sessionSetAltSpeed_KBps(fHandle, TR_UP, [fDefaults integerForKey: @"SpeedLimitUploadLimit"]);
+    tr_sessionSetAltSpeed_KBps(fHandle, TR_DOWN, [fDefaults integerForKey: @"SpeedLimitDownloadLimit"]);
         
     [[NSNotificationCenter defaultCenter] postNotificationName: @"SpeedLimitUpdate" object: nil];
 }
 
 - (void) applyRatioSetting: (id) sender
 {
-    //[[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateUI" object: nil];
     tr_sessionSetRatioLimited(fHandle, [fDefaults boolForKey: @"RatioCheck"]);
     tr_sessionSetRatioLimit(fHandle, [fDefaults floatForKey: @"RatioLimit"]);
+    
+    //reload global settings in inspector
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateGlobalOptions" object: nil];
+}
+
+- (void) setRatioStop: (id) sender
+{
+    [fDefaults setFloat: [sender floatValue] forKey: @"RatioLimit"];
+    
+    [self applyRatioSetting: nil];
 }
 
 - (void) updateRatioStopField
@@ -602,11 +622,20 @@ tr_session * fHandle;
     [self applyRatioSetting: nil];
 }
 
-- (void) setRatioStop: (id) sender
+- (void) applyIdleStopSetting: (id) sender
 {
-    [fDefaults setFloat: [sender floatValue] forKey: @"RatioLimit"];
+    tr_sessionSetIdleLimited(fHandle, [fDefaults boolForKey: @"IdleLimitCheck"]);
+    tr_sessionSetIdleLimit(fHandle, [fDefaults integerForKey: @"IdleLimitMinutes"]);
     
-    [self applyRatioSetting: nil];
+    //reload global settings in inspector
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateGlobalOptions" object: nil];
+}
+
+- (void) setIdleStop: (id) sender
+{
+    [fDefaults setInteger: [sender integerValue] forKey: @"IdleLimitMinutes"];
+    
+    [self applyIdleStopSetting: nil];
 }
 
 - (void) updateLimitFields
@@ -685,7 +714,7 @@ tr_session * fHandle;
 
 - (void) setBadge: (id) sender
 {
-    [[NSNotificationCenter defaultCenter] postNotificationName: @"DockBadgeChange" object: self];
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateUI" object: self];
 }
 
 - (void) resetWarnings: (id) sender
@@ -698,6 +727,7 @@ tr_session * fHandle;
     [fDefaults removeObjectForKey: @"WarningCreatorPrivateBlankAddress"];
     [fDefaults removeObjectForKey: @"WarningRemoveTrackers"];
     [fDefaults removeObjectForKey: @"WarningInvalidOpen"];
+    [fDefaults removeObjectForKey: @"WarningRemoveCompleted"];
     [fDefaults removeObjectForKey: @"WarningDonate"];
     //[fDefaults removeObjectForKey: @"WarningLegal"];
 }
@@ -767,6 +797,21 @@ tr_session * fHandle;
         @selector(incompleteFolderSheetClosed:returnCode:contextInfo:) contextInfo: nil];
 }
 
+- (void) doneScriptSheetShow:(id)sender
+{
+    NSOpenPanel * panel = [NSOpenPanel openPanel];
+    
+    [panel setPrompt: NSLocalizedString(@"Select", "Preferences -> Open panel prompt")];
+    [panel setAllowsMultipleSelection: NO];
+    [panel setCanChooseFiles: YES];
+    [panel setCanChooseDirectories: NO];
+    [panel setCanCreateDirectories: NO];
+    
+    [panel beginSheetForDirectory: nil file: nil types: nil
+                   modalForWindow: [self window] modalDelegate: self didEndSelector:
+     @selector(doneScriptSheetClosed:returnCode:contextInfo:) contextInfo: nil];
+}
+
 - (void) setUseIncompleteFolder: (id) sender
 {
     tr_sessionSetIncompleteDirEnabled(fHandle, [fDefaults boolForKey: @"UseIncompleteDownloadFolder"]);
@@ -777,6 +822,17 @@ tr_session * fHandle;
     tr_sessionSetIncompleteFileNamingEnabled(fHandle, [fDefaults boolForKey: @"RenamePartialFiles"]);
 }
 
+- (void) setDoneScriptEnabled: (id) sender
+{
+    if ([fDefaults boolForKey: @"DoneScriptEnabled"] && ![[NSFileManager defaultManager] fileExistsAtPath: [fDefaults stringForKey:@"DoneScriptPath"]])
+    {
+        // enabled is set but script file doesn't exist, so prompt for one and disable until they pick one
+        [fDefaults setBool: NO forKey: @"DoneScriptEnabled"];
+        [self doneScriptSheetShow: sender];
+    }
+    tr_sessionSetTorrentDoneScriptEnabled(fHandle, [fDefaults boolForKey: @"DoneScriptEnabled"]);
+}
+
 - (void) setAutoImport: (id) sender
 {
     NSString * path;
@@ -814,107 +870,6 @@ tr_session * fHandle;
     [[NSNotificationCenter defaultCenter] postNotificationName: @"AutoSizeSettingChange" object: self];
 }
 
-- (void) setProxyEnabled: (id) sender
-{
-    tr_sessionSetProxyEnabled(fHandle, [fDefaults boolForKey: @"Proxy"]);
-}
-
-- (void) setProxyAddress: (id) sender
-{
-    NSString * address = [sender stringValue];
-    tr_sessionSetProxy(fHandle, [address UTF8String]);
-    [fDefaults setObject: address forKey: @"ProxyAddress"];
-}
-
-- (void) setProxyPort: (id) sender
-{
-    int port = [sender intValue];
-    [fDefaults setInteger: port forKey: @"ProxyPort"];
-    tr_sessionSetProxyPort(fHandle, port);
-}
-
-- (void) setProxyType: (id) sender
-{
-    NSString * type;
-    switch ([sender indexOfSelectedItem])
-    {
-        case PROXY_HTTP:
-            type = @"HTTP";
-            break;
-        case PROXY_SOCKS4:
-            type = @"SOCKS4";
-            break;
-        case PROXY_SOCKS5:
-            type = @"SOCKS5";
-            break;
-        default:
-            NSAssert1(NO, @"Unknown index %d received for proxy type", [sender indexOfSelectedItem]);
-    }
-    
-    [fDefaults setObject: type forKey: @"ProxyType"];
-    [self updateProxyType];
-}
-
-- (void) updateProxyType
-{
-    NSString * typeString = [fDefaults stringForKey: @"ProxyType"];
-    tr_proxy_type type;
-    if ([typeString isEqualToString: @"SOCKS4"])
-        type = TR_PROXY_SOCKS4;
-    else if ([typeString isEqualToString: @"SOCKS5"])
-        type = TR_PROXY_SOCKS5;
-    else
-    {
-        //safety
-        if (![typeString isEqualToString: @"HTTP"])
-        {
-            typeString = @"HTTP";
-            [fDefaults setObject: typeString forKey: @"ProxyType"];
-        }
-        type = TR_PROXY_HTTP;
-    }
-    
-    tr_sessionSetProxyType(fHandle, type);
-}
-
-- (void) setProxyAuthorize: (id) sender
-{
-    BOOL enable = [fDefaults boolForKey: @"ProxyAuthorize"];
-    tr_sessionSetProxyAuthEnabled(fHandle, enable);
-}
-
-- (void) setProxyUsername: (id) sender
-{
-    tr_sessionSetProxyUsername(fHandle, [[fDefaults stringForKey: @"ProxyUsername"] UTF8String]);
-}
-
-- (void) setProxyPassword: (id) sender
-{
-    const char * password = [[sender stringValue] UTF8String];
-    [self setKeychainPassword: password forService: PROXY_KEYCHAIN_SERVICE username: PROXY_KEYCHAIN_NAME];
-    
-    tr_sessionSetProxyPassword(fHandle, password);
-}
-
-- (void) updateProxyPassword
-{
-    UInt32 passwordLength;
-    const char * password = nil;
-    SecKeychainFindGenericPassword(NULL, strlen(PROXY_KEYCHAIN_SERVICE), PROXY_KEYCHAIN_SERVICE,
-        strlen(PROXY_KEYCHAIN_NAME), PROXY_KEYCHAIN_NAME, &passwordLength, (void **)&password, NULL);
-    
-    if (password != NULL)
-    {
-        char fullPassword[passwordLength+1];
-        strncpy(fullPassword, password, passwordLength);
-        fullPassword[passwordLength] = '\0';
-        SecKeychainItemFreeContent(NULL, (void *)password);
-        
-        tr_sessionSetProxyPassword(fHandle, fullPassword);
-        [fProxyPasswordField setStringValue: [NSString stringWithUTF8String: fullPassword]];
-    }
-}
-
 - (void) setRPCEnabled: (id) sender
 {
     BOOL enable = [fDefaults boolForKey: @"RPC"];
@@ -1101,6 +1056,12 @@ tr_session * fHandle;
     [fRPCAddRemoveControl setEnabled: [fRPCWhitelistTable numberOfSelectedRows] > 0 forSegment: RPC_IP_REMOVE_TAG];
 }
 
+- (void) helpForScript: (id) sender
+{
+    [[NSHelpManager sharedHelpManager] openHelpAnchor: @"script"
+        inBook: [[NSBundle mainBundle] objectForInfoDictionaryKey: @"CFBundleHelpBookName"]];
+}
+
 - (void) helpForPeers: (id) sender
 {
     [[NSHelpManager sharedHelpManager] openHelpAnchor: @"peers"
@@ -1139,6 +1100,10 @@ tr_session * fHandle;
     const BOOL usePartialFileRanaming = tr_sessionIsIncompleteFileNamingEnabled(fHandle);
     [fDefaults setBool: usePartialFileRanaming forKey: @"RenamePartialFiles"];
     
+    //utp
+    const BOOL utp = tr_sessionIsUTPEnabled(fHandle);
+    [fDefaults setBool: utp forKey: @"UTPGlobal"];
+    
     //peers
     const uint16_t peersTotal = tr_sessionGetPeerLimit(fHandle);
     [fDefaults setInteger: peersTotal forKey: @"PeersTotal"];
@@ -1154,7 +1119,7 @@ tr_session * fHandle;
     const BOOL dht = tr_sessionIsDHTEnabled(fHandle);
     [fDefaults setBool: dht forKey: @"DHTGlobal"];
     
-    //dht
+    //lpd
     const BOOL lpd = tr_sessionIsLPDEnabled(fHandle);
     [fDefaults setBool: lpd forKey: @"LocalPeerDiscoveryGlobal"];
     
@@ -1180,14 +1145,14 @@ tr_session * fHandle;
     const BOOL downLimitEnabled = tr_sessionIsSpeedLimited(fHandle, TR_DOWN);
     [fDefaults setBool: downLimitEnabled forKey: @"CheckDownload"];
     
-    const int downLimit = tr_sessionGetSpeedLimit(fHandle, TR_DOWN);
+    const int downLimit = tr_sessionGetSpeedLimit_KBps(fHandle, TR_DOWN);
     [fDefaults setInteger: downLimit forKey: @"DownloadLimit"];
     
     //speed limit - up
     const BOOL upLimitEnabled = tr_sessionIsSpeedLimited(fHandle, TR_UP);
     [fDefaults setBool: upLimitEnabled forKey: @"CheckUpload"];
     
-    const int upLimit = tr_sessionGetSpeedLimit(fHandle, TR_UP);
+    const int upLimit = tr_sessionGetSpeedLimit_KBps(fHandle, TR_UP);
     [fDefaults setInteger: upLimit forKey: @"UploadLimit"];
     
     //alt speed limit enabled
@@ -1195,11 +1160,11 @@ tr_session * fHandle;
     [fDefaults setBool: useAltSpeed forKey: @"SpeedLimit"];
     
     //alt speed limit - down
-    const int downLimitAlt = tr_sessionGetAltSpeed(fHandle, TR_DOWN);
+    const int downLimitAlt = tr_sessionGetAltSpeed_KBps(fHandle, TR_DOWN);
     [fDefaults setInteger: downLimitAlt forKey: @"SpeedLimitDownloadLimit"];
     
     //alt speed limit - up
-    const int upLimitAlt = tr_sessionGetAltSpeed(fHandle, TR_UP);
+    const int upLimitAlt = tr_sessionGetAltSpeed_KBps(fHandle, TR_UP);
     [fDefaults setInteger: upLimitAlt forKey: @"SpeedLimitUploadLimit"];
     
     //alt speed limit schedule
@@ -1217,7 +1182,10 @@ tr_session * fHandle;
     
     //blocklist
     const BOOL blocklist = tr_blocklistIsEnabled(fHandle);
-    [fDefaults setBool: blocklist forKey: @"Blocklist"];
+    [fDefaults setBool: blocklist forKey: @"BlocklistNew"];
+    
+    NSString * blocklistURL = [NSString stringWithUTF8String: tr_blocklistGetURL(fHandle)];
+    [fDefaults setObject: blocklistURL forKey: @"BlocklistURL"];
     
     //seed ratio
     const BOOL ratioLimited = tr_sessionIsRatioLimited(fHandle);
@@ -1226,6 +1194,20 @@ tr_session * fHandle;
     const float ratioLimit = tr_sessionGetRatioLimit(fHandle);
     [fDefaults setFloat: ratioLimit forKey: @"RatioLimit"];
     
+    //Idle seed limit
+    const BOOL idleLimited = tr_sessionIsIdleLimited(fHandle);
+    [fDefaults setBool: idleLimited forKey: @"IdleLimitCheck"];
+    
+    const NSUInteger idleLimitMin = tr_sessionGetIdleLimit(fHandle);
+    [fDefaults setInteger: idleLimitMin forKey: @"IdleLimitMinutes"];
+    
+    //done script
+    const BOOL doneScriptEnabled = tr_sessionIsTorrentDoneScriptEnabled(fHandle);
+    [fDefaults setBool: doneScriptEnabled forKey: @"DoneScriptEnabled"];
+    
+    NSString * doneScriptPath = [NSString stringWithUTF8String: tr_sessionGetTorrentDoneScript(fHandle)];
+    [fDefaults setObject: doneScriptPath forKey: @"DoneScriptPath"];
+    
     //update gui if loaded
     if (fHasLoaded)
     {
@@ -1233,6 +1215,8 @@ tr_session * fHandle;
         
         //download directory handled by bindings
         
+        //utp handled by bindings
+        
         [fPeersGlobalField setIntValue: peersTotal];
         [fPeersTorrentField setIntValue: peersTorrent];
         
@@ -1260,13 +1244,21 @@ tr_session * fHandle;
         
         //speed limit schedule times and day handled by bindings
         
+        [fBlocklistURLField setStringValue: blocklistURL];
+        [self updateBlocklistButton];
         [self updateBlocklistFields];
         
         //ratio limit enabled handled by bindings
         [fRatioStopField setFloatValue: ratioLimit];
+        
+        //idle limit enabled handled by bindings
+        [fIdleStopField setIntegerValue: idleLimitMin];
     }
     
     [[NSNotificationCenter defaultCenter] postNotificationName: @"SpeedLimitUpdate" object: nil];
+    
+    //reload global settings in inspector
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateGlobalOptions" object: nil];
 }
 
 @end
@@ -1387,6 +1379,28 @@ tr_session * fHandle;
     [fImportFolderPopUp selectItemAtIndex: 0];
 }
 
+- (void) doneScriptSheetClosed: (NSOpenPanel *) openPanel returnCode: (int) code contextInfo: (void *) info
+{
+    if (code == NSOKButton)
+    {
+        NSString * filePath = [[openPanel filenames] objectAtIndex: 0];
+        
+        if ([[NSFileManager defaultManager] fileExistsAtPath: filePath])  // script file exists
+        {
+            [fDefaults setObject: filePath forKey: @"DoneScriptPath"];
+            [fDefaults setBool: YES forKey: @"DoneScriptEnabled"];
+        }
+        else // script file doesn't exist so don't enable
+        {
+            [fDefaults setObject: nil forKey:@"DoneScriptPath"];
+            [fDefaults setBool: NO forKey: @"DoneScriptEnabled"];
+        }
+        tr_sessionSetTorrentDoneScript(fHandle, [[fDefaults stringForKey:@"DoneScriptPath"] UTF8String]);
+        tr_sessionSetTorrentDoneScriptEnabled(fHandle, [fDefaults boolForKey:@"DoneScriptEnabled"]);
+    }
+    [fDoneScriptPopUp selectItemAtIndex: 0];
+}
+
 - (void) setKeychainPassword: (const char *) password forService: (const char *) service username: (const char *) username
 {
     SecKeychainItemRef item = NULL;
