@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: Torrent.h 8870 2009-08-05 01:55:37Z livings124 $
+ * $Id: Torrent.h 12325 2011-04-05 23:03:33Z livings124 $
  *
- * Copyright (c) 2006-2009 Transmission authors and contributors
+ * Copyright (c) 2006-2011 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -24,28 +24,22 @@
 
 #import <Cocoa/Cocoa.h>
 #import <transmission.h>
+#import <Quartz/Quartz.h>
 
 @class FileListNode;
 
-#define STAT_TIME_NONE -1
-#define STAT_TIME_NOW -2
-
-@interface Torrent : NSObject
+#warning uncomment
+@interface Torrent : NSObject <NSCopying>//, QLPreviewItem>
 {
     tr_torrent * fHandle;
     const tr_info * fInfo;
     const tr_stat * fStat;
-    
-    BOOL fResumeOnWake;
-    
-    BOOL fUseIncompleteFolder;
-    NSString * fDownloadFolder, * fIncompleteFolder;
 	
     NSUserDefaults * fDefaults;
 
     NSImage * fIcon;
     
-    NSString * fNameString, * fHashString;
+    NSString * fHashString;
     
     tr_file_stat * fFileStat;
     NSArray * fFileList, * fFlatFileList;
@@ -53,27 +47,28 @@
     NSIndexSet * fPreviousFinishedIndexes;
     NSDate * fPreviousFinishedIndexesDate;
     
-    BOOL fFinishedSeeding, fWaitToStart, fStalled;
+    BOOL fWaitToStart, fStalled;
     
     NSInteger fGroupValue;
     
-    BOOL fAddedTrackers;
+    BOOL fResumeOnWake;
     
-    NSDictionary * fQuickPauseDict;
+    NSString * fTimeMachineExclude;
 }
 
 - (id) initWithPath: (NSString *) path location: (NSString *) location deleteTorrentFile: (BOOL) torrentDelete
         lib: (tr_session *) lib;
 - (id) initWithTorrentStruct: (tr_torrent *) torrentStruct location: (NSString *) location lib: (tr_session *) lib;
+- (id) initWithMagnetAddress: (NSString *) address location: (NSString *) location lib: (tr_session *) lib;
 - (id) initWithHistory: (NSDictionary *) history lib: (tr_session *) lib forcePause: (BOOL) pause;
 
 - (NSDictionary *) history;
 
-- (void) closeRemoveTorrent;
+- (void) closeRemoveTorrent: (BOOL) trashFiles;
 
-- (void) changeIncompleteDownloadFolder: (NSString *) folder;
-- (void) changeDownloadFolder: (NSString *) folder;
-- (NSString *) downloadFolder;
+- (void) changeDownloadFolderBeforeUsing: (NSString *) folder;
+
+- (NSString *) currentDirectory;
 
 - (void) getAvailability: (int8_t *) tab size: (NSInteger) size;
 - (void) getAmountFinished: (float *) tab size: (NSInteger) size;
@@ -92,14 +87,21 @@
 
 - (void) resetCache;
 
+- (BOOL) isMagnet;
+- (NSString *) magnetLink;
+
 - (CGFloat) ratio;
 - (tr_ratiolimit) ratioSetting;
 - (void) setRatioSetting: (tr_ratiolimit) setting;
 - (CGFloat) ratioLimit;
 - (void) setRatioLimit: (CGFloat) limit;
-- (BOOL) seedRatioSet;
 - (CGFloat) progressStopRatio;
 
+- (tr_idlelimit) idleSetting;
+- (void) setIdleSetting: (tr_idlelimit) setting;
+- (NSUInteger) idleLimitMinutes;
+- (void) setIdleLimitMinutes: (NSUInteger) limit;
+
 - (BOOL) usesSpeedLimit: (BOOL) upload;
 - (void) setUseSpeedLimit: (BOOL) use upload: (BOOL) upload;
 - (NSInteger) speedLimit: (BOOL) upload;
@@ -116,15 +118,11 @@
 - (tr_priority_t) priority;
 - (void) setPriority: (tr_priority_t) priority;
 
-- (void) revealData;
 + (void) trashFile: (NSString *) path;
-- (void) trashData;
 - (void) moveTorrentDataFileTo: (NSString *) folder;
 - (void) copyTorrentFileTo: (NSString *) path;
 
 - (BOOL) alertForRemainingDiskSpace;
-- (BOOL) alertForFolderAvailable;
-- (BOOL) alertForMoveFolderAvailable;
 
 - (NSImage *) icon;
 
@@ -133,21 +131,10 @@
 - (uint64_t) size;
 - (uint64_t) sizeLeft;
 
-- (NSString *) trackerAddressAnnounce;
-- (NSDate *) lastAnnounceTime;
-- (NSInteger) nextAnnounceTime;
-- (NSString *) announceResponse;
-
-- (NSString *) trackerAddressScrape;
-- (NSDate *) lastScrapeTime;
-- (NSInteger) nextScrapeTime;
-- (NSString *) scrapeResponse;
-
-- (NSMutableArray *) allTrackers: (BOOL) separators;
-- (NSArray *) allTrackersFlat;
-- (BOOL) updateAllTrackersForAdd: (NSMutableArray *) trackers;
-- (void) updateAllTrackersForRemove: (NSMutableArray *) trackers;
-- (BOOL) hasAddedTrackers;
+- (NSMutableArray *) allTrackerStats;
+- (NSArray *) allTrackersFlat; //used by GroupRules
+- (BOOL) addTrackerToNewTier: (NSString *) tracker;
+- (void) removeTrackers: (NSSet *) trackers;
 
 - (NSString *) comment;
 - (NSString *) creator;
@@ -160,15 +147,14 @@
 
 - (NSString *) torrentLocation;
 - (NSString *) dataLocation;
+- (NSString *) fileLocation: (FileListNode *) node;
 
 - (CGFloat) progress;
 - (CGFloat) progressDone;
 - (CGFloat) progressLeft;
 - (CGFloat) checkingProgress;
 
-- (NSInteger) eta;
-
-- (CGFloat) notAvailableDesired;
+- (CGFloat) availableDesired;
 
 - (BOOL) isActive;
 - (BOOL) isSeeding;
@@ -176,8 +162,9 @@
 - (BOOL) isCheckingWaiting;
 - (BOOL) allDownloaded;
 - (BOOL) isComplete;
+- (BOOL) isFinishedSeeding;
 - (BOOL) isError;
-- (BOOL) isErrorOrWarning;
+- (BOOL) isAnyErrorOrWarning;
 - (NSString *) errorMessage;
 
 - (NSArray *) peers;
@@ -191,18 +178,14 @@
 - (NSString *) remainingTimeString;
 
 - (NSString *) stateString;
-
-- (NSInteger) seeders;
-- (NSInteger) leechers;
-- (NSInteger) completedFromTracker;
-
 - (NSInteger) totalPeersConnected;
 - (NSInteger) totalPeersTracker;
 - (NSInteger) totalPeersIncoming;
 - (NSInteger) totalPeersCache;
 - (NSInteger) totalPeersPex;
 - (NSInteger) totalPeersDHT;
-- (NSInteger) totalPeersKnown;
+- (NSInteger) totalPeersLocal;
+- (NSInteger) totalPeersLTEP;
 
 - (NSInteger) peersSendingToUs;
 - (NSInteger) peersGettingFromUs;
@@ -216,7 +199,6 @@
 - (uint64_t) downloadedTotal;
 - (uint64_t) uploadedTotal;
 - (uint64_t) failedHash;
-- (CGFloat) swarmSpeed;
 
 - (NSInteger) groupValue;
 - (void) setGroupValue: (NSInteger) groupValue;
@@ -224,13 +206,13 @@
 - (void) checkGroupValueForRemoval: (NSNotification *) notification;
 
 - (NSArray *) fileList;
+- (NSArray *) flatFileList;
 - (NSInteger) fileCount;
 - (void) updateFileStat;
-- (NSArray *) flatFileList;
 
 //methods require fileStats to have been updated recently to be accurate
 - (CGFloat) fileProgress: (FileListNode *) node;
-- (BOOL) canChangeDownloadCheckForFile: (NSInteger) index;
+- (BOOL) canChangeDownloadCheckForFile: (NSUInteger) index;
 - (BOOL) canChangeDownloadCheckForFiles: (NSIndexSet *) indexSet;
 - (NSInteger) checkForFiles: (NSIndexSet *) indexSet;
 - (void) setFileCheckState: (NSInteger) state forIndexes: (NSIndexSet *) indexSet;
@@ -243,10 +225,16 @@
 - (NSDate *) dateActivity;
 - (NSDate *) dateActivityOrAdd;
 
+- (NSInteger) secondsDownloading;
+- (NSInteger) secondsSeeding;
+
 - (NSInteger) stalledMinutes;
 - (BOOL) isStalled;
 
+- (void) updateTimeMachineExclude;
+
 - (NSInteger) stateSortKey;
+- (NSString *) trackerSortKey;
 
 - (tr_torrent *) torrentStruct;
 
