@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: TorrentTableView.h 8261 2009-04-20 03:36:35Z livings124 $
+ * $Id: TorrentTableView.h 12472 2011-05-29 15:35:36Z livings124 $
  *
- * Copyright (c) 2005-2009 Transmission authors and contributors
+ * Copyright (c) 2005-2011 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -24,8 +24,9 @@
 
 #import <Cocoa/Cocoa.h>
 #import <transmission.h>
-#import "Controller.h"
 
+@class Controller;
+@class Torrent;
 @class TorrentCell;
 
 #define GROUP_SEPARATOR_HEIGHT 18.0
@@ -42,7 +43,7 @@
     
     IBOutlet NSMenu * fContextRow, * fContextNoRow;
     
-    NSInteger fMouseControlRow, fMouseRevealRow, fMouseActionRow, fActionPushedRow;
+    NSInteger fMouseRow, fMouseControlRow, fMouseRevealRow, fMouseActionRow, fActionPushedRow;
     NSArray * fSelectedValues;
     
     IBOutlet NSMenu * fActionMenu, * fUploadMenu, * fDownloadMenu, * fRatioMenu, * fPriorityMenu;
@@ -58,7 +59,8 @@
 - (void) removeAllCollapsedGroups;
 - (void) saveCollapsedGroups;
 
-- (void) removeButtonTrackingAreas;
+- (void) removeTrackingAreas;
+- (void) setRowHover: (NSInteger) row;
 - (void) setControlButtonHover: (NSInteger) row;
 - (void) setRevealButtonHover: (NSInteger) row;
 - (void) setActionButtonHover: (NSInteger) row;
@@ -84,9 +86,6 @@
 
 - (void) setPriority: (id) sender;
 
-- (void) checkFile: (id) sender;
-- (void) moveDataFile: (id) sender;
-
 - (void) togglePiecesBar;
 - (CGFloat) piecesBarPercent;
 
