@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: TorrentTableView.m 8883 2009-08-07 23:34:48Z livings124 $
+ * $Id: TorrentTableView.m 12496 2011-06-13 22:26:43Z livings124 $
  *
- * Copyright (c) 2005-2009 Transmission authors and contributors
+ * Copyright (c) 2005-2011 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -23,11 +23,13 @@
  *****************************************************************************/
 
 #import "TorrentTableView.h"
-#import "TorrentCell.h"
-#import "Torrent.h"
-#import "TorrentGroup.h"
+#import "Controller.h"
 #import "FileListNode.h"
-#import "QuickLookController.h"
+#import "NSApplicationAdditions.h"
+#import "NSStringAdditions.h"
+#import "Torrent.h"
+#import "TorrentCell.h"
+#import "TorrentGroup.h"
 
 #define MAX_GROUP 999999
 
@@ -39,9 +41,6 @@
 #define ACTION_MENU_PRIORITY_NORMAL_TAG 102
 #define ACTION_MENU_PRIORITY_LOW_TAG 103
 
-#define GROUP_SPEED_IMAGE_COLUMN_WIDTH 8.0f
-#define GROUP_RATIO_IMAGE_COLUMN_WIDTH 10.0f
-
 #define TOGGLE_PROGRESS_SECONDS 0.175
 
 @interface TorrentTableView (Private)
@@ -50,8 +49,6 @@
 
 - (void) setGroupStatusColumns;
 
-- (void) createFileMenu: (NSMenu *) menu forFiles: (NSArray *) files;
-
 @end
 
 @implementation TorrentTableView
@@ -70,6 +67,7 @@
         else
             fCollapsedGroups = [[NSMutableIndexSet alloc] init];
         
+        fMouseRow = -1;
         fMouseControlRow = -1;
         fMouseRevealRow = -1;
         fMouseActionRow = -1;
@@ -77,7 +75,7 @@
         
         [self setDelegate: self];
         
-        fPiecesBarPercent = [fDefaults boolForKey: @"PiecesBar"] ? 1.0f : 0.0f;
+        fPiecesBarPercent = [fDefaults boolForKey: @"PiecesBar"] ? 1.0 : 0.0;
     }
     
     return self;
@@ -141,7 +139,7 @@
 
 - (NSCell *) outlineView: (NSOutlineView *) outlineView dataCellForTableColumn: (NSTableColumn *) tableColumn item: (id) item
 {
-    BOOL group = ![item isKindOfClass: [Torrent class]];
+    const BOOL group = ![item isKindOfClass: [Torrent class]];
     if (!tableColumn)
         return !group ? fTorrentCell : nil;
     else
@@ -156,6 +154,7 @@
         [cell setRepresentedObject: item];
         
         const NSInteger row = [self rowForItem: item];
+        [cell setHover: row == fMouseRow];
         [cell setControlHover: row == fMouseControlRow];
         [cell setRevealHover: row == fMouseRevealRow];
         [cell setActionHover: row == fMouseActionRow];
@@ -167,13 +166,18 @@
         if ([ident isEqualToString: @"UL Image"] || [ident isEqualToString: @"DL Image"])
         {
             //ensure arrows are white only when selected
-            NSImage * image = [cell image];
-            BOOL template = [cell backgroundStyle] == NSBackgroundStyleLowered;
-            if ([image isTemplate] != template)
+            if ([NSApp isOnSnowLeopardOrBetter])
+                [[cell image] setTemplate: [cell backgroundStyle] == NSBackgroundStyleLowered];
+            else
             {
-                [image setTemplate: template];
-                [cell setImage: nil];
-                [cell setImage: image];
+                NSImage * image = [cell image];
+                const BOOL template = [cell backgroundStyle] == NSBackgroundStyleLowered;
+                if ([image isTemplate] != template)
+                {
+                    [image setTemplate: template];
+                    [cell setImage: nil];
+                    [cell setImage: image];
+                }
             }
         }
     }
@@ -212,11 +216,12 @@
                 : NSLocalizedString(@"Upload speed", "Torrent table -> group row -> tooltip");
     else if (ident)
     {
-        NSInteger count = [[item torrents] count];
+        NSUInteger count = [[item torrents] count];
         if (count == 1)
             return NSLocalizedString(@"1 transfer", "Torrent table -> group row -> tooltip");
         else
-            return [NSString stringWithFormat: NSLocalizedString(@"%d transfers", "Torrent table -> group row -> tooltip"), count];
+            return [NSString stringWithFormat: NSLocalizedString(@"%@ transfers", "Torrent table -> group row -> tooltip"),
+                    [NSString formattedUInteger: count]];
     }
     else
         return nil;
@@ -225,9 +230,9 @@
 - (void) updateTrackingAreas
 {
     [super updateTrackingAreas];
-    [self removeButtonTrackingAreas];
+    [self removeTrackingAreas];
     
-    NSRange rows = [self rowsInRect: [self visibleRect]];
+    const NSRange rows = [self rowsInRect: [self visibleRect]];
     if (rows.length == 0)
         return;
     
@@ -237,14 +242,15 @@
         if (![[self itemAtRow: row] isKindOfClass: [Torrent class]])
             continue;
         
-        NSDictionary * userInfo = [NSDictionary dictionaryWithObject: [NSNumber numberWithInt: row] forKey: @"Row"];
+        NSDictionary * userInfo = [NSDictionary dictionaryWithObject: [NSNumber numberWithInteger: row] forKey: @"Row"];
         TorrentCell * cell = (TorrentCell *)[self preparedCellAtColumn: -1 row: row];
         [cell addTrackingAreasForView: self inRect: [self rectOfRow: row] withUserInfo: userInfo mouseLocation: mouseLocation];
     }
 }
 
-- (void) removeButtonTrackingAreas
+- (void) removeTrackingAreas
 {
+    fMouseRow = -1;
     fMouseControlRow = -1;
     fMouseRevealRow = -1;
     fMouseActionRow = -1;
@@ -256,6 +262,15 @@
     }
 }
 
+- (void) setRowHover: (NSInteger) row
+{
+    NSAssert([fDefaults boolForKey: @"SmallView"], @"cannot set a hover row when not in compact view");
+    
+    fMouseRow = row;
+    if (row >= 0)
+        [self setNeedsDisplayInRect: [self rectOfRow: row]];
+}
+
 - (void) setControlButtonHover: (NSInteger) row
 {
     fMouseControlRow = row;
@@ -284,14 +299,20 @@
     NSNumber * row;
     if ((row = [dict objectForKey: @"Row"]))
     {
-        NSInteger rowVal = [row intValue];
+        NSInteger rowVal = [row integerValue];
         NSString * type = [dict objectForKey: @"Type"];
         if ([type isEqualToString: @"Action"])
             fMouseActionRow = rowVal;
         else if ([type isEqualToString: @"Control"])
             fMouseControlRow = rowVal;
-        else
+        else if ([type isEqualToString: @"Reveal"])
             fMouseRevealRow = rowVal;
+        else
+        {
+            fMouseRow = rowVal;
+            if (![fDefaults boolForKey: @"SmallView"])
+                return;
+        }
         
         [self setNeedsDisplayInRect: [self rectOfRow: rowVal]];
     }
@@ -309,10 +330,16 @@
             fMouseActionRow = -1;
         else if ([type isEqualToString: @"Control"])
             fMouseControlRow = -1;
-        else
+        else if ([type isEqualToString: @"Reveal"])
             fMouseRevealRow = -1;
+        else
+        {
+            fMouseRow = -1;
+            if (![fDefaults boolForKey: @"SmallView"])
+                return;
+        }
         
-        [self setNeedsDisplayInRect: [self rectOfRow: [row intValue]]];
+        [self setNeedsDisplayInRect: [self rectOfRow: [row integerValue]]];
     }
 }
 
@@ -408,20 +435,17 @@
     {
         if ([item isKindOfClass: [Torrent class]])
         {
-            NSInteger index = [self rowForItem: item];
+            const NSInteger index = [self rowForItem: item];
             if (index != -1)
                 [indexSet addIndex: index];
         }
         else
         {
-            NSInteger group = [item groupIndex];
+            const NSInteger group = [item groupIndex];
             for (NSInteger i = 0; i < [self numberOfRows]; i++)
             {
-                if ([indexSet containsIndex: i])
-                    continue;
-                
                 id tableItem = [self itemAtRow: i];
-                if (![tableItem isKindOfClass: [Torrent class]] && group == [tableItem groupIndex])
+                if ([tableItem isKindOfClass: [TorrentGroup class]] && group == [tableItem groupIndex])
                 {
                     [indexSet addIndex: i];
                     break;
@@ -496,19 +520,10 @@
     
     if (firstChar == 'f' && [event modifierFlags] & NSAlternateKeyMask && [event modifierFlags] & NSCommandKeyMask)
         [fController focusFilterField];
+    else if (firstChar == ' ')
+        [fController toggleQuickLook: nil];
     else
-    {
-        //handle quicklook
-        if (firstChar == ' ')
-            [[QuickLookController quickLook] toggleQuickLook];
-        else if (firstChar == NSRightArrowFunctionKey)
-            [[QuickLookController quickLook] pressRight];
-        else if (firstChar == NSLeftArrowFunctionKey)
-            [[QuickLookController quickLook] pressLeft];
-        else;
-        
         [super keyDown: event];
-    }
 }
 
 - (NSRect) iconRectForRow: (NSInteger) row
@@ -516,11 +531,12 @@
     return [fTorrentCell iconRectForBounds: [self rectOfRow: row]];
 }
 
+#warning catch string urls?
 - (void) paste: (id) sender
 {
     NSURL * url;
     if ((url = [NSURL URLFromPasteboard: [NSPasteboard generalPasteboard]]))
-        [fController openURL: url];
+        [fController openURL: [url absoluteString]];
 }
 
 - (BOOL) validateMenuItem: (NSMenuItem *) menuItem
@@ -539,7 +555,7 @@
         [fController stopTorrents: [NSArray arrayWithObject: torrent]];
     else
     {
-        if ([[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask)
+        if (([NSApp isOnSnowLeopardOrBetter] ? [NSEvent modifierFlags] : [[NSApp currentEvent] modifierFlags]) & NSAlternateKeyMask)
             [fController resumeTorrentsNoWait: [NSArray arrayWithObject: torrent]];
         else if ([torrent waitingToStart])
             [fController stopTorrents: [NSArray arrayWithObject: torrent]];
@@ -554,11 +570,8 @@
     if (row < 0)
         return;
     
-    const NSInteger numberOfNonFileItems = [fActionMenu numberOfItems];
-    
     //update file action menu
     fMenuTorrent = [[self itemAtRow: row] retain];
-    [self createFileMenu: fActionMenu forFiles: [fMenuTorrent fileList]];
     
     //update global limit check
     [fGlobalLimitItem setState: [fMenuTorrent usesGlobalSpeedLimit] ? NSOnState : NSOffState];
@@ -566,17 +579,22 @@
     //place menu below button
     NSRect rect = [fTorrentCell iconRectForBounds: [self rectOfRow: row]];
     NSPoint location = rect.origin;
-    location.y += rect.size.height + 5.0f;
-    location = [self convertPoint: location toView: nil];
+    location.y += rect.size.height + 5.0;
     
-    NSEvent * newEvent = [NSEvent mouseEventWithType: [event type] location: location
-        modifierFlags: [event modifierFlags] timestamp: [event timestamp] windowNumber: [event windowNumber]
-        context: [event context] eventNumber: [event eventNumber] clickCount: [event clickCount] pressure: [event pressure]];
-    
-    [NSMenu popUpContextMenu: fActionMenu withEvent: newEvent forView: self];
-    
-    for (NSInteger i = [fActionMenu numberOfItems]-1; i >= numberOfNonFileItems; i--)
-        [fActionMenu removeItemAtIndex: i];
+    if ([NSApp isOnSnowLeopardOrBetter])
+    {
+        location = [self convertPoint: location toView: self];
+        [fActionMenu popUpMenuPositioningItem: nil atLocation: location inView: self];
+    }
+    else
+    {
+        location = [self convertPoint: location toView: nil];
+        NSEvent * newEvent = [NSEvent mouseEventWithType: [event type] location: location
+            modifierFlags: [event modifierFlags] timestamp: [event timestamp] windowNumber: [event windowNumber]
+            context: [event context] eventNumber: [event eventNumber] clickCount: [event clickCount] pressure: [event pressure]];
+        
+        [NSMenu popUpContextMenu: fActionMenu withEvent: newEvent forView: self];
+    }
     
     [fMenuTorrent release];
     fMenuTorrent = nil;
@@ -593,7 +611,8 @@
         NSMenuItem * item;
         if ([menu numberOfItems] == 3)
         {
-            const NSInteger speedLimitActionValue[] = { 0, 5, 10, 20, 30, 40, 50, 75, 100, 150, 200, 250, 500, 750, 1000, 1500, 2000, -1 };
+            const NSInteger speedLimitActionValue[] = { 0, 5, 10, 20, 30, 40, 50, 75, 100, 150, 200, 250, 500,
+                                                        750, 1000, 1500, 2000, -1 };
             
             for (NSInteger i = 0; speedLimitActionValue[i] != -1; i++)
             {
@@ -623,9 +642,9 @@
         NSMenuItem * item;
         if ([menu numberOfItems] == 4)
         {
-            const CGFloat ratioLimitActionValue[] = { 0.25f, 0.5f, 0.75f, 1.0f, 1.5f, 2.0f, 3.0f, -1.0f };
+            const float ratioLimitActionValue[] = { 0.25, 0.5, 0.75, 1.0, 1.5, 2.0, 3.0, -1.0 };
             
-            for (NSInteger i = 0; ratioLimitActionValue[i] != -1.0f; i++)
+            for (NSInteger i = 0; ratioLimitActionValue[i] != -1.0; i++)
             {
                 item = [[NSMenuItem alloc] initWithTitle: [NSString localizedStringWithFormat: @"%.2f", ratioLimitActionValue[i]]
                         action: @selector(setQuickRatio:) keyEquivalent: @""];
@@ -662,80 +681,6 @@
         item = [menu itemWithTag: ACTION_MENU_PRIORITY_LOW_TAG];
         [item setState: priority == TR_PRI_LOW ? NSOnState : NSOffState];
     }
-    else //assume the menu is part of the file list
-    {
-        if ([menu numberOfItems] > 0)
-            return;
-        
-        NSMenu * supermenu = [menu supermenu];
-        [self createFileMenu: menu forFiles: [(FileListNode *)[[supermenu itemAtIndex: [supermenu indexOfItemWithSubmenu: menu]]
-                                                representedObject] children]];
-    }
-}
-
-//alternating rows - first row after group row is white
-- (void) highlightSelectionInClipRect: (NSRect) clipRect
-{
-    NSColor * altColor = [[NSColor controlAlternatingRowBackgroundColors] objectAtIndex: 1];
-    [altColor set];
-    
-    NSRect visibleRect = clipRect;
-    NSRange rows = [self rowsInRect: visibleRect];
-    BOOL start = YES;
-    
-    if (rows.length > 0)
-    {
-        //determine what the first row color should be
-        if ([[self itemAtRow: rows.location] isKindOfClass: [Torrent class]])
-        {
-            for (NSInteger i = rows.location-1; i>=0; i--)
-            {
-                if (![[self itemAtRow: i] isKindOfClass: [Torrent class]])
-                    break;
-                start = !start;
-            }
-        }
-        else
-        {
-            rows.location++;
-            rows.length--;
-        }
-        
-        NSInteger i;
-        for (i = rows.location; i < NSMaxRange(rows); i++)
-        {
-            if (![[self itemAtRow: i] isKindOfClass: [Torrent class]])
-            {
-                start = YES;
-                continue;
-            }
-            
-            if (!start && ![self isRowSelected: i])
-                NSRectFill([self rectOfRow: i]);
-            
-            start = !start;
-        }
-        
-        CGFloat newY = NSMaxY([self rectOfRow: i-1]);
-        visibleRect.size.height -= newY - visibleRect.origin.y;
-        visibleRect.origin.y = newY;
-    }
-        
-    //remaining visible rows continue alternating
-    const CGFloat height = [self rowHeight] + [self intercellSpacing].height;
-    const NSInteger numberOfRects = ceil(visibleRect.size.height / height);
-    
-    visibleRect.size.height = height;
-    if (start)
-        visibleRect.origin.y += height;
-    
-    for (NSInteger i = start ? 1 : 0; i < numberOfRects; i += 2)
-    {
-        NSRectFill(visibleRect);
-        visibleRect.origin.y += 2.0 * height;
-    }
-    
-    [super highlightSelectionInClipRect: clipRect];
 }
 
 - (void) setQuickLimitMode: (id) sender
@@ -808,7 +753,7 @@
             priority = TR_PRI_LOW;
             break;
         default:
-            return;
+            NSAssert1(NO, @"Unknown priority: %d", [sender tag]);
     }
     
     [fMenuTorrent setPriority: priority];
@@ -816,19 +761,6 @@
     [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateUI" object: nil];
 }
 
-- (void) checkFile: (id) sender
-{
-    NSIndexSet * indexSet = [(FileListNode *)[sender representedObject] indexes];
-    [fMenuTorrent setFileCheckState: [sender state] != NSOnState ? NSOnState : NSOffState forIndexes: indexSet];
-    
-    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateStats" object: nil];
-}
-
-- (void) moveDataFile: (id) sender
-{
-    [fController moveDataFiles: [NSArray arrayWithObject: fMenuTorrent]];
-}
-
 - (void) togglePiecesBar
 {
     //stop previous animation
@@ -836,8 +768,8 @@
         [fPiecesBarAnimation release];
     
     NSMutableArray * progressMarks = [NSMutableArray arrayWithCapacity: 16];
-    for (NSAnimationProgress i = 0.0625f; i <= 1.0f; i += 0.0625f)
-        [progressMarks addObject: [NSNumber numberWithFloat: i]];
+    for (NSAnimationProgress i = 0.0625; i <= 1.0; i += 0.0625)
+        [progressMarks addObject: [NSNumber numberWithDouble: i]];
     
     fPiecesBarAnimation = [[NSAnimation alloc] initWithDuration: TOGGLE_PROGRESS_SECONDS animationCurve: NSAnimationEaseIn];
     [fPiecesBarAnimation setAnimationBlockingMode: NSAnimationNonblocking];
@@ -863,7 +795,7 @@
         if ([fDefaults boolForKey: @"PiecesBar"])
             fPiecesBarPercent = progress;
         else
-            fPiecesBarPercent = 1.0f - progress;
+            fPiecesBarPercent = 1.0 - progress;
         
         [self reloadData];
     }
@@ -891,58 +823,10 @@
 
 - (void) setGroupStatusColumns
 {
-    BOOL ratio = [fDefaults boolForKey: @"DisplayGroupRowRatio"];
+    const BOOL ratio = [fDefaults boolForKey: @"DisplayGroupRowRatio"];
     
     [[self tableColumnWithIdentifier: @"DL"] setHidden: ratio];
     [[self tableColumnWithIdentifier: @"DL Image"] setHidden: ratio];
-    
-    //change size of image column
-    NSTableColumn * ulImageTableColumn = [self tableColumnWithIdentifier: @"UL Image"];
-    CGFloat oldWidth = [ulImageTableColumn width], newWidth = ratio ? GROUP_RATIO_IMAGE_COLUMN_WIDTH : GROUP_SPEED_IMAGE_COLUMN_WIDTH;
-    if (oldWidth != newWidth)
-    {
-        [ulImageTableColumn setWidth: newWidth];
-        
-        NSTableColumn * groupTableColumn = [self tableColumnWithIdentifier: @"Group"];
-        [groupTableColumn setWidth: [groupTableColumn width] - (newWidth - oldWidth)];
-    }
-}
-
-- (void) createFileMenu: (NSMenu *) menu forFiles: (NSArray *) files
-{
-    for (FileListNode * node in files)
-    {
-        NSString * name = [node name];
-        
-        NSMenuItem * item = [[NSMenuItem alloc] initWithTitle: name action: @selector(checkFile:) keyEquivalent: @""];
-        
-        NSImage * icon;
-        if (![node isFolder])
-            icon = [[NSWorkspace sharedWorkspace] iconForFileType: [name pathExtension]];
-        else
-        {
-            NSMenu * itemMenu = [[NSMenu alloc] initWithTitle: name];
-            [itemMenu setAutoenablesItems: NO];
-            [item setSubmenu: itemMenu];
-            [itemMenu setDelegate: self];
-            [itemMenu release];
-            
-            icon = [[NSWorkspace sharedWorkspace] iconForFileType: NSFileTypeForHFSTypeCode('fldr')];
-        }
-        
-        [item setRepresentedObject: node];
-        
-        [icon setScalesWhenResized: YES];
-        [icon setSize: NSMakeSize(16.0, 16.0)];
-        [item setImage: icon];
-        
-        NSIndexSet * indexSet = [node indexes];
-        [item setState: [fMenuTorrent checkForFiles: indexSet]];
-        [item setEnabled: [fMenuTorrent canChangeDownloadCheckForFiles: indexSet]];
-        
-        [menu addItem: item];
-        [item release];
-    }
 }
 
 @end
