@@ -1,39 +1,31 @@
 /*
- * This file Copyright (C) 2009-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
- * Transmission project are granted a special exemption to clause 2(b)
- * so that the bulk of its code can remain under the MIT license.
- * This exemption does not extend to derived works not owned by
- * the Transmission project.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2
+ * as published by the Free Software Foundation.
  *
- * $Id: mainwin.cc 11475 2010-12-05 18:52:09Z charles $
+ * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+ *
+ * $Id: mainwin.cc 12557 2011-07-19 21:19:18Z jordan $
  */
 
 #include <cassert>
 #include <iostream>
 
-#include <QCheckBox>
-#include <QCloseEvent>
-#include <QDesktopServices>
-#include <QFileDialog>
-#include <QHBoxLayout>
-#include <QInputDialog>
-#include <QLabel>
-#include <QMessageBox>
-#include <QSignalMapper>
-#include <QSize>
-#include <QStyle>
-#include <QSystemTrayIcon>
-#include <QUrl>
+#include <QtGui>
 
 #include <libtransmission/transmission.h>
 #include <libtransmission/utils.h>
 #include <libtransmission/version.h>
 
 #include "about.h"
+#include "add-data.h"
+#include "app.h"
 #include "details.h"
+#include "filterbar.h"
 #include "filters.h"
+#include "formatter.h"
 #include "hig.h"
 #include "mainwin.h"
 #include "make-dialog.h"
@@ -51,20 +43,18 @@
 #include "torrent-model.h"
 #include "triconpushbutton.h"
 #include "ui_mainwin.h"
-#include "utils.h"
-#include "qticonloader.h"
 
 #define PREFS_KEY "prefs-key";
 
 QIcon
-TrMainWindow :: getStockIcon( const QString& freedesktop_name, int fallback )
+TrMainWindow :: getStockIcon( const QString& name, int fallback )
 {
-    QIcon fallbackIcon;
+    QIcon icon = QIcon::fromTheme( name );
 
-    if( fallback > 0 )
-        fallbackIcon = style()->standardIcon( QStyle::StandardPixmap( fallback ), 0, this );
+    if( icon.isNull( ) && ( fallback >= 0 ) )
+        icon = style()->standardIcon( QStyle::StandardPixmap( fallback ), 0, this );
 
-    return QtIconLoader::icon( freedesktop_name, fallbackIcon );
+    return icon;
 }
 
 namespace
@@ -100,8 +90,12 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     mySpeedModeOnIcon( ":/icons/alt-limit-on.png" ),
     myLastSendTime( 0 ),
     myLastReadTime( 0 ),
-    myNetworkTimer( this )
+    myNetworkTimer( this ),
+    myRefreshTrayIconTimer( this ),
+    myRefreshActionSensitivityTimer( this )
 {
+    setAcceptDrops( true );
+
     QAction * sep = new QAction( this );
     sep->setSeparator( true );
 
@@ -113,7 +107,7 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     const QSize smallIconSize( i, i );
 
     // icons
-    ui.action_AddFile->setIcon( getStockIcon( "list-add", QStyle::SP_DialogOpenButton ) );
+    ui.action_OpenFile->setIcon( getStockIcon( "folder-open", QStyle::SP_DialogOpenButton ) );
     ui.action_New->setIcon( getStockIcon( "document-new", QStyle::SP_DesktopIcon ) );
     ui.action_Properties->setIcon( getStockIcon( "document-properties", QStyle::SP_DesktopIcon ) );
     ui.action_OpenFolder->setIcon( getStockIcon( "folder-open", QStyle::SP_DirOpenIcon ) );
@@ -133,7 +127,6 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
 
     // ui signals
     connect( ui.action_Toolbar, SIGNAL(toggled(bool)), this, SLOT(setToolbarVisible(bool)));
-    connect( ui.action_TrayIcon, SIGNAL(toggled(bool)), this, SLOT(setTrayIconVisible(bool)));
     connect( ui.action_Filterbar, SIGNAL(toggled(bool)), this, SLOT(setFilterbarVisible(bool)));
     connect( ui.action_Statusbar, SIGNAL(toggled(bool)), this, SLOT(setStatusbarVisible(bool)));
     connect( ui.action_CompactView, SIGNAL(toggled(bool)), this, SLOT(setCompactView(bool)));
@@ -145,7 +138,6 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     connect( ui.action_SortByRatio,    SIGNAL(toggled(bool)), this, SLOT(onSortByRatioToggled(bool)));
     connect( ui.action_SortBySize,     SIGNAL(toggled(bool)), this, SLOT(onSortBySizeToggled(bool)));
     connect( ui.action_SortByState,    SIGNAL(toggled(bool)), this, SLOT(onSortByStateToggled(bool)));
-    connect( ui.action_SortByTracker,  SIGNAL(toggled(bool)), this, SLOT(onSortByTrackerToggled(bool)));
     connect( ui.action_ReverseSortOrder, SIGNAL(toggled(bool)), this, SLOT(setSortAscendingPref(bool)));
     connect( ui.action_Start, SIGNAL(triggered()), this, SLOT(startSelected()));
     connect( ui.action_Pause, SIGNAL(triggered()), this, SLOT(pauseSelected()));
@@ -155,11 +147,12 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     connect( ui.action_Announce, SIGNAL(triggered()), this, SLOT(reannounceSelected()) );
     connect( ui.action_StartAll, SIGNAL(triggered()), this, SLOT(startAll()));
     connect( ui.action_PauseAll, SIGNAL(triggered()), this, SLOT(pauseAll()));
-    connect( ui.action_AddFile, SIGNAL(triggered()), this, SLOT(openTorrent()));
+    connect( ui.action_OpenFile, SIGNAL(triggered()), this, SLOT(openTorrent()));
     connect( ui.action_AddURL, SIGNAL(triggered()), this, SLOT(openURL()));
     connect( ui.action_New, SIGNAL(triggered()), this, SLOT(newTorrent()));
     connect( ui.action_Preferences, SIGNAL(triggered()), this, SLOT(openPreferences()));
     connect( ui.action_Statistics, SIGNAL(triggered()), myStatsDialog, SLOT(show()));
+    connect( ui.action_Donate, SIGNAL(triggered()), this, SLOT(openDonate()));
     connect( ui.action_About, SIGNAL(triggered()), myAboutDialog, SLOT(show()));
     connect( ui.action_Contents, SIGNAL(triggered()), this, SLOT(openHelp()));
     connect( ui.action_OpenFolder, SIGNAL(triggered()), this, SLOT(openFolder()));
@@ -198,8 +191,8 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
 
     connect( &myFilterModel, SIGNAL(rowsInserted(const QModelIndex&,int,int)), this, SLOT(refreshVisibleCount()));
     connect( &myFilterModel, SIGNAL(rowsRemoved(const QModelIndex&,int,int)), this, SLOT(refreshVisibleCount()));
-    connect( &myFilterModel, SIGNAL(rowsInserted(const QModelIndex&,int,int)), this, SLOT(refreshActionSensitivity()));
-    connect( &myFilterModel, SIGNAL(rowsRemoved(const QModelIndex&,int,int)), this, SLOT(refreshActionSensitivity()));
+    connect( &myFilterModel, SIGNAL(rowsInserted(const QModelIndex&,int,int)), this, SLOT(refreshActionSensitivitySoon()));
+    connect( &myFilterModel, SIGNAL(rowsRemoved(const QModelIndex&,int,int)), this, SLOT(refreshActionSensitivitySoon()));
 
     connect( ui.action_Quit, SIGNAL(triggered()), QCoreApplication::instance(), SLOT(quit()) );
 
@@ -208,8 +201,10 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     connect( &myModel, SIGNAL(modelReset()), this, SLOT(onModelReset()));
     connect( &myModel, SIGNAL(rowsRemoved(const QModelIndex&,int,int)), this, SLOT(onModelReset()));
     connect( &myModel, SIGNAL(rowsInserted(const QModelIndex&,int,int)), this, SLOT(onModelReset()));
+    connect( &myModel, SIGNAL(dataChanged(const QModelIndex&,const QModelIndex&)), this, SLOT(refreshTrayIconSoon()));
+
     ui.listView->setModel( &myFilterModel );
-    connect( ui.listView->selectionModel(), SIGNAL(selectionChanged(const QItemSelection&,const QItemSelection&)), this, SLOT(refreshActionSensitivity()));
+    connect( ui.listView->selectionModel(), SIGNAL(selectionChanged(const QItemSelection&,const QItemSelection&)), this, SLOT(refreshActionSensitivitySoon()));
 
     QActionGroup * actionGroup = new QActionGroup( this );
     actionGroup->addAction( ui.action_SortByActivity );
@@ -220,10 +215,9 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     actionGroup->addAction( ui.action_SortByRatio );
     actionGroup->addAction( ui.action_SortBySize );
     actionGroup->addAction( ui.action_SortByState );
-    actionGroup->addAction( ui.action_SortByTracker );
 
     QMenu * menu = new QMenu( );
-    menu->addAction( ui.action_AddFile );
+    menu->addAction( ui.action_OpenFile );
     menu->addAction( ui.action_AddURL );
     menu->addSeparator( );
     menu->addAction( ui.action_ShowMainWindow );
@@ -246,14 +240,13 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     ui.action_TrayIcon->setChecked( minimized || prefs.getBool( Prefs::SHOW_TRAY_ICON ) );
 
     ui.verticalLayout->addWidget( createStatusBar( ) );
-    ui.verticalLayout->insertWidget( 0, createFilterBar( ) );
+    ui.verticalLayout->insertWidget( 0, myFilterBar = new FilterBar( myPrefs, myModel, myFilterModel ) );
 
     QList<int> initKeys;
     initKeys << Prefs :: MAIN_WINDOW_X
              << Prefs :: SHOW_TRAY_ICON
              << Prefs :: SORT_REVERSED
              << Prefs :: SORT_MODE
-             << Prefs :: FILTER_MODE
              << Prefs :: FILTERBAR
              << Prefs :: STATUSBAR
              << Prefs :: STATUSBAR_STATS
@@ -282,7 +275,12 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
         myNetworkTimer.start( 1000 );
     }
 
-    refreshActionSensitivity( );
+    connect( &myRefreshTrayIconTimer, SIGNAL(timeout()), this, SLOT(refreshTrayIcon()) );
+    connect( &myRefreshActionSensitivityTimer, SIGNAL(timeout()), this, SLOT(refreshActionSensitivity()) );
+
+
+    refreshActionSensitivitySoon( );
+    refreshTrayIconSoon( );
     refreshStatusBar( );
     refreshTitle( );
     refreshVisibleCount( );
@@ -324,8 +322,9 @@ TrMainWindow :: onModelReset( )
 {
     refreshTitle( );
     refreshVisibleCount( );
-    refreshActionSensitivity( );
+    refreshActionSensitivitySoon( );
     refreshStatusBar( );
+    refreshTrayIconSoon( );
 }
 
 /****
@@ -352,94 +351,11 @@ TrMainWindow :: onSetPrefs( bool isChecked )
 
 #define SHOW_KEY "show-mode"
 
-void
-TrMainWindow :: onShowModeClicked( )
-{
-    setShowMode( sender()->property(SHOW_KEY).toInt() );
-}
-
-QWidget *
-TrMainWindow :: createFilterBar( )
-{
-    int i;
-    QMenu * m;
-    QLineEdit * e;
-    QPushButton * p;
-    QHBoxLayout * h;
-    QActionGroup * a;
-    const int smallSize = style( )->pixelMetric( QStyle::PM_SmallIconSize, 0, this );
-    const QSize smallIconSize( smallSize, smallSize );
-
-    QWidget * top = myFilterBar = new QWidget;
-    h = new QHBoxLayout( top );
-    h->setContentsMargins( HIG::PAD_SMALL, HIG::PAD_SMALL, HIG::PAD_SMALL, HIG::PAD_SMALL );
-    h->setSpacing( HIG::PAD_SMALL );
-#ifdef Q_OS_MAC
-    top->setStyleSheet( "QPushButton{ "
-                        "  border-radius: 10px; "
-                        "  padding: 0 5px; "
-                        "  border: 1px none; "
-                        "} "
-                        "QPushButton:pressed, QPushButton:checked{ "
-                        "  border-width: 1px; "
-                        "  border-style: solid; "
-                        "  border-color: #5f5f5f #979797 #979797; "
-                        "  background-color: #979797; "
-                        "  color: white; "
-                        "} ");
-#endif
-
-        QList<QString> titles;
-        titles << tr( "A&ll" ) << tr( "&Active" ) << tr( "&Downloading" ) << tr( "&Seeding" ) << tr( "&Paused" );
-        for( i=0; i<titles.size(); ++i ) {
-            p = myFilterButtons[i] = new QPushButton( titles[i] );
-            p->setProperty( SHOW_KEY, i );
-            p->setFlat( true );
-            p->setCheckable( true );
-            p->setMaximumSize( calculateTextButtonSizeHint( p ) );
-            connect( p, SIGNAL(clicked()), this, SLOT(onShowModeClicked()));
-            h->addWidget( p );
-        }
-
-    h->addStretch( 1 );
-
-        a = new QActionGroup( this );
-        a->addAction( ui.action_FilterByName );
-        a->addAction( ui.action_FilterByFiles );
-        a->addAction( ui.action_FilterByTracker );
-        m = new QMenu( );
-        m->addAction( ui.action_FilterByName );
-        m->addAction( ui.action_FilterByFiles );
-        m->addAction( ui.action_FilterByTracker );
-        connect( ui.action_FilterByName, SIGNAL(triggered()), this, SLOT(filterByName()));
-        connect( ui.action_FilterByFiles, SIGNAL(triggered()), this, SLOT(filterByFiles()));
-        connect( ui.action_FilterByTracker, SIGNAL(triggered()), this, SLOT(filterByTracker()));
-        ui.action_FilterByName->setChecked( true );
-        p = myFilterTextButton = new TrIconPushButton;
-        p->setIcon( getStockIcon( "edit-find", QStyle::SP_ArrowForward ) );
-        p->setFlat( true );
-        p->setMenu( m );
-        h->addWidget( p );
-
-        e = myFilterTextLineEdit = new QLineEdit;
-        connect( e, SIGNAL(textChanged(QString)), &myFilterModel, SLOT(setText(QString)));
-        h->addWidget( e );
-
-        p = myFilterTextButton = new TrIconPushButton;
-        p->setIcon( getStockIcon( "edit-clear", QStyle::SP_DialogCloseButton ) );
-        p->setFlat( true );
-        connect( p, SIGNAL(clicked()), myFilterTextLineEdit, SLOT(clear()));
-        h->addWidget( p );
-
-    return top;
-}
-
 QWidget *
 TrMainWindow :: createStatusBar( )
 {
     QMenu * m;
     QLabel * l;
-    QWidget * w;
     QHBoxLayout * h;
     QPushButton * p;
     QActionGroup * a;
@@ -452,8 +368,8 @@ TrMainWindow :: createStatusBar( )
     h->setSpacing( HIG::PAD_SMALL );
 
         p = myOptionsButton = new TrIconPushButton( this );
-        p->setIcon( QIcon( ":/icons/options.png" ) );
-        p->setIconSize( QPixmap( ":/icons/options.png" ).size() );
+        p->setIcon( QIcon( ":/icons/utilities.png" ) );
+        p->setIconSize( QPixmap( ":/icons/utilities.png" ).size() );
         p->setFlat( true );
         p->setMenu( createOptionsMenu( ) );
         h->addWidget( p );
@@ -500,25 +416,22 @@ TrMainWindow :: createStatusBar( )
         l = myStatsLabel = new QLabel( this );
         h->addWidget( l );
 
-    h->addSpacing( HIG::PAD_BIG );
+    h->addStretch( 1 );
 
-        w = new QWidget( this );
-        w->setMinimumSize( HIG::PAD_BIG, 1 );
-        w->setMaximumSize( HIG::PAD_BIG, 1 );
-        h->addWidget( w );
         l = myDownloadSpeedLabel = new QLabel( this );
+        const int minimumSpeedWidth = l->fontMetrics().width( Formatter::speedToString(Speed::fromKBps(999.99)));
+        l->setMinimumWidth( minimumSpeedWidth );
+        l->setAlignment( Qt::AlignRight|Qt::AlignVCenter );
         h->addWidget( l );
         l = new QLabel( this );
         l->setPixmap( getStockIcon( "go-down", QStyle::SP_ArrowDown ).pixmap( smallIconSize ) );
         h->addWidget( l );
 
-    h->addSpacing( HIG::PAD_BIG );
+    h->addStretch( 1 );
 
-        w = new QWidget( this );
-        w->setMinimumSize( HIG::PAD_BIG, 1 );
-        w->setMaximumSize( HIG::PAD_BIG, 1 );
-        h->addWidget( w );
         l = myUploadSpeedLabel = new QLabel;
+        l->setMinimumWidth( minimumSpeedWidth );
+        l->setAlignment( Qt::AlignRight|Qt::AlignVCenter );
         h->addWidget( l );
         l = new QLabel;
         l->setPixmap( getStockIcon( "go-up", QStyle::SP_ArrowUp ).pixmap( smallIconSize ) );
@@ -549,14 +462,14 @@ TrMainWindow :: createOptionsMenu( )
         a->setProperty( PREF_VARIANTS_KEY, QVariantList() << Prefs::DSPEED_ENABLED << false );
         g->addAction( a );
         connect( a, SIGNAL(triggered(bool)), this, SLOT(onSetPrefs(bool)) );
-        a = myDlimitOnAction = sub->addAction( tr( "Limited at %1" ).arg( Utils::speedToString( Speed::fromKbps( currentVal ) ) ) );
+        a = myDlimitOnAction = sub->addAction( tr( "Limited at %1" ).arg( Formatter::speedToString( Speed::fromKBps( currentVal ) ) ) );
         a->setCheckable( true );
         a->setProperty( PREF_VARIANTS_KEY, QVariantList() << Prefs::DSPEED << currentVal << Prefs::DSPEED_ENABLED << true );
         g->addAction( a );
         connect( a, SIGNAL(triggered(bool)), this, SLOT(onSetPrefs(bool)) );
         sub->addSeparator( );
         foreach( int i, stockSpeeds ) {
-            a = sub->addAction( Utils::speedToString( Speed::fromKbps(i) ) );
+            a = sub->addAction( Formatter::speedToString( Speed::fromKBps( i ) ) );
             a->setProperty( PREF_VARIANTS_KEY, QVariantList() << Prefs::DSPEED << i << Prefs::DSPEED_ENABLED << true );
             connect( a, SIGNAL(triggered(bool)), this, SLOT(onSetPrefs()));
         }
@@ -569,14 +482,14 @@ TrMainWindow :: createOptionsMenu( )
         a->setProperty( PREF_VARIANTS_KEY, QVariantList() << Prefs::USPEED_ENABLED << false );
         g->addAction( a );
         connect( a, SIGNAL(triggered(bool)), this, SLOT(onSetPrefs(bool)) );
-        a = myUlimitOnAction = sub->addAction( tr( "Limited at %1" ).arg( Utils::speedToString( Speed::fromKbps( currentVal ) ) ) );
+        a = myUlimitOnAction = sub->addAction( tr( "Limited at %1" ).arg( Formatter::speedToString( Speed::fromKBps( currentVal ) ) ) );
         a->setCheckable( true );
         a->setProperty( PREF_VARIANTS_KEY, QVariantList() << Prefs::USPEED << currentVal << Prefs::USPEED_ENABLED << true );
         g->addAction( a );
         connect( a, SIGNAL(triggered(bool)), this, SLOT(onSetPrefs(bool)) );
         sub->addSeparator( );
         foreach( int i, stockSpeeds ) {
-            a = sub->addAction( Utils::speedToString( Speed::fromKbps(i) ) );
+            a = sub->addAction( Formatter::speedToString( Speed::fromKBps( i ) ) );
             a->setProperty( PREF_VARIANTS_KEY, QVariantList() << Prefs::USPEED << i << Prefs::USPEED_ENABLED << true );
             connect( a, SIGNAL(triggered(bool)), this, SLOT(onSetPrefs()));
         }
@@ -591,14 +504,14 @@ TrMainWindow :: createOptionsMenu( )
         a->setProperty( PREF_VARIANTS_KEY, QVariantList() << Prefs::RATIO_ENABLED << false );
         g->addAction( a );
         connect( a, SIGNAL(triggered(bool)), this, SLOT(onSetPrefs(bool)) );
-        a = myRatioOnAction = sub->addAction( tr( "Stop at Ratio (%1)" ).arg( Utils::ratioToString( d ) ) );
+        a = myRatioOnAction = sub->addAction( tr( "Stop at Ratio (%1)" ).arg( Formatter::ratioToString( d ) ) );
         a->setCheckable( true );
         a->setProperty( PREF_VARIANTS_KEY, QVariantList() << Prefs::RATIO << d << Prefs::RATIO_ENABLED << true );
         g->addAction( a );
         connect( a, SIGNAL(triggered(bool)), this, SLOT(onSetPrefs(bool)) );
         sub->addSeparator( );
         foreach( double i, stockRatios ) {
-            a = sub->addAction( Utils::ratioToString( i ) );
+            a = sub->addAction( Formatter::ratioToString( i ) );
             a->setProperty( PREF_VARIANTS_KEY, QVariantList() << Prefs::RATIO << i << Prefs::RATIO_ENABLED << true );
             connect( a, SIGNAL(triggered(bool)), this, SLOT(onSetPrefs()));
         }
@@ -623,7 +536,6 @@ void TrMainWindow :: onSortByProgressToggled ( bool b ) { if( b ) setSortPref( S
 void TrMainWindow :: onSortByRatioToggled    ( bool b ) { if( b ) setSortPref( SortMode::SORT_BY_RATIO );    }
 void TrMainWindow :: onSortBySizeToggled     ( bool b ) { if( b ) setSortPref( SortMode::SORT_BY_SIZE );     }
 void TrMainWindow :: onSortByStateToggled    ( bool b ) { if( b ) setSortPref( SortMode::SORT_BY_STATE );    }
-void TrMainWindow :: onSortByTrackerToggled  ( bool b ) { if( b ) setSortPref( SortMode::SORT_BY_TRACKER );  }
 
 void
 TrMainWindow :: setSortAscendingPref( bool b )
@@ -693,6 +605,12 @@ TrMainWindow :: copyMagnetLinkToClipboard( )
     mySession.copyMagnetLinkToClipboard( id );
 }
 
+void
+TrMainWindow :: openDonate( )
+{
+    QDesktopServices :: openUrl( QUrl( "http://www.transmissionbt.com/donate.php" ) );
+}
+
 void
 TrMainWindow :: openHelp( )
 {
@@ -701,7 +619,7 @@ TrMainWindow :: openHelp( )
     sscanf( SHORT_VERSION_STRING, "%d.%d", &major, &minor );
     char url[128];
     tr_snprintf( url, sizeof( url ), fmt, major, minor/10 );
-    QDesktopServices :: openUrl( QUrl( QString( url ) ) );
+    QDesktopServices :: openUrl( QUrl( url ) );
 }
 
 void
@@ -728,13 +646,39 @@ TrMainWindow :: refreshVisibleCount( )
     myVisibleCountLabel->setVisible( totalCount > 0 );
 }
 
+void
+TrMainWindow :: refreshTrayIconSoon( )
+{
+    if( !myRefreshTrayIconTimer.isActive( ) )
+    {
+        myRefreshTrayIconTimer.setSingleShot( true );
+        myRefreshTrayIconTimer.start( 500 );
+    }
+}
+void
+TrMainWindow :: refreshTrayIcon( )
+{
+    Speed u, d;
+    const QString idle = tr( "Idle" );
+
+    foreach( int id, myModel.getIds( ) ) {
+        const Torrent * tor = myModel.getTorrentFromId( id );
+        u += tor->uploadSpeed( );
+        d += tor->downloadSpeed( );
+    }
+
+    myTrayIcon.setToolTip( tr( "Transmission\nUp: %1\nDown: %2" )
+                           .arg( u.isZero() ? idle : Formatter::speedToString( u ) )
+                           .arg( d.isZero() ? idle : Formatter::speedToString( d ) ) );
+}
+
 void
 TrMainWindow :: refreshStatusBar( )
 {
     const Speed up( myModel.getUploadSpeed( ) );
     const Speed down( myModel.getDownloadSpeed( ) );
-    myUploadSpeedLabel->setText( Utils :: speedToString( up ) );
-    myDownloadSpeedLabel->setText( Utils :: speedToString( down ) );
+    myUploadSpeedLabel->setText( Formatter:: speedToString( up ) );
+    myDownloadSpeedLabel->setText( Formatter:: speedToString( down ) );
 
     myNetworkLabel->setVisible( !mySession.isServer( ) );
 
@@ -743,28 +687,39 @@ TrMainWindow :: refreshStatusBar( )
 
     if( mode == "session-ratio" )
     {
-        str = tr( "Ratio: %1" ).arg( Utils :: ratioToString( mySession.getStats().ratio ) );
+        str = tr( "Ratio: %1" ).arg( Formatter:: ratioToString( mySession.getStats().ratio ) );
     }
     else if( mode == "session-transfer" )
     {
         const tr_session_stats& stats( mySession.getStats( ) );
-        str = tr( "Down: %1, Up: %2" ).arg( Utils :: sizeToString( stats.downloadedBytes ) )
-                                      .arg( Utils :: sizeToString( stats.uploadedBytes ) );
+        str = tr( "Down: %1, Up: %2" ).arg( Formatter:: sizeToString( stats.downloadedBytes ) )
+                                      .arg( Formatter:: sizeToString( stats.uploadedBytes ) );
     }
     else if( mode == "total-transfer" )
     {
         const tr_session_stats& stats( mySession.getCumulativeStats( ) );
-        str = tr( "Down: %1, Up: %2" ).arg( Utils :: sizeToString( stats.downloadedBytes ) )
-                                      .arg( Utils :: sizeToString( stats.uploadedBytes ) );
+        str = tr( "Down: %1, Up: %2" ).arg( Formatter:: sizeToString( stats.downloadedBytes ) )
+                                      .arg( Formatter:: sizeToString( stats.uploadedBytes ) );
     }
     else // default is "total-ratio"
     {
-        str = tr( "Ratio: %1" ).arg( Utils :: ratioToString( mySession.getCumulativeStats().ratio ) );
+        str = tr( "Ratio: %1" ).arg( Formatter:: ratioToString( mySession.getCumulativeStats().ratio ) );
     }
 
     myStatsLabel->setText( str );
 }
 
+
+
+void
+TrMainWindow :: refreshActionSensitivitySoon( )
+{
+    if( !myRefreshActionSensitivityTimer.isActive( ) )
+    {
+        myRefreshActionSensitivityTimer.setSingleShot( true );
+        myRefreshActionSensitivityTimer.start( 500 );
+    }
+}
 void
 TrMainWindow :: refreshActionSensitivity( )
 {
@@ -835,7 +790,7 @@ TrMainWindow :: getSelectedTorrents( ) const
 
     foreach( QModelIndex index, ui.listView->selectionModel( )->selectedRows( ) )
     {
-        const Torrent * tor( index.model()->data( index, TorrentModel::TorrentRole ).value<const Torrent*>( ) );
+        const Torrent * tor( index.data( TorrentModel::TorrentRole ).value<const Torrent*>( ) );
         ids.insert( tor->id( ) );
     }
 
@@ -887,17 +842,6 @@ TrMainWindow :: reannounceSelected( )
 ***
 **/
 
-void TrMainWindow :: setShowMode     ( int i ) { myPrefs.set( Prefs::FILTER_MODE, FilterMode( i ) ); }
-void TrMainWindow :: showAll         ( ) { setShowMode( FilterMode :: SHOW_ALL ); }
-void TrMainWindow :: showActive      ( ) { setShowMode( FilterMode :: SHOW_ACTIVE ); }
-void TrMainWindow :: showDownloading ( ) { setShowMode( FilterMode :: SHOW_DOWNLOADING ); }
-void TrMainWindow :: showSeeding     ( ) { setShowMode( FilterMode :: SHOW_SEEDING ); }
-void TrMainWindow :: showPaused      ( ) { setShowMode( FilterMode :: SHOW_PAUSED ); }
-
-void TrMainWindow :: filterByName    ( ) { myFilterModel.setTextMode( TorrentFilter :: FILTER_BY_NAME ); }
-void TrMainWindow :: filterByTracker ( ) { myFilterModel.setTextMode( TorrentFilter :: FILTER_BY_TRACKER ); }
-void TrMainWindow :: filterByFiles   ( ) { myFilterModel.setTextMode( TorrentFilter :: FILTER_BY_FILES ); }
-
 void TrMainWindow :: showTotalRatio      ( ) { myPrefs.set( Prefs::STATUSBAR_STATS, "total-ratio"); }
 void TrMainWindow :: showTotalTransfer   ( ) { myPrefs.set( Prefs::STATUSBAR_STATS, "total-transfer"); }
 void TrMainWindow :: showSessionRatio    ( ) { myPrefs.set( Prefs::STATUSBAR_STATS, "session-ratio"); }
@@ -913,11 +857,6 @@ TrMainWindow :: setCompactView( bool visible )
     myPrefs.set( Prefs :: COMPACT_VIEW, visible );
 }
 void
-TrMainWindow :: setTrayIconVisible( bool visible )
-{
-    myPrefs.set( Prefs :: SHOW_TRAY_ICON, visible );
-}
-void
 TrMainWindow :: toggleSpeedMode( )
 {
     myPrefs.toggleBool( Prefs :: ALT_SPEED_LIMIT_ENABLED );
@@ -953,8 +892,9 @@ TrMainWindow :: toggleWindows( bool doShow )
     {
         if ( !isVisible( ) ) show( );
         if ( isMinimized( ) ) showNormal( );
-        activateWindow( );
+        //activateWindow( );
         raise( );
+        QApplication::setActiveWindow( this );
     }
 }
 
@@ -1003,7 +943,6 @@ TrMainWindow :: refreshPref( int key )
             ui.action_SortByRatio->setChecked    ( i == SortMode::SORT_BY_RATIO );
             ui.action_SortBySize->setChecked     ( i == SortMode::SORT_BY_SIZE );
             ui.action_SortByState->setChecked    ( i == SortMode::SORT_BY_STATE );
-            ui.action_SortByTracker->setChecked  ( i == SortMode::SORT_BY_TRACKER );
             break;
 
         case Prefs::DSPEED_ENABLED:
@@ -1011,7 +950,7 @@ TrMainWindow :: refreshPref( int key )
             break;
 
         case Prefs::DSPEED:
-            myDlimitOnAction->setText( tr( "Limited at %1" ).arg( Utils::speedToString( Speed::fromKbps( myPrefs.get<int>(key) ) ) ) );
+            myDlimitOnAction->setText( tr( "Limited at %1" ).arg( Formatter::speedToString( Speed::fromKBps( myPrefs.get<int>(key) ) ) ) );
             break;
 
         case Prefs::USPEED_ENABLED:
@@ -1019,7 +958,7 @@ TrMainWindow :: refreshPref( int key )
             break;
 
         case Prefs::USPEED:
-            myUlimitOnAction->setText( tr( "Limited at %1" ).arg( Utils::speedToString( Speed::fromKbps( myPrefs.get<int>(key) ) ) ) );
+            myUlimitOnAction->setText( tr( "Limited at %1" ).arg( Formatter::speedToString( Speed::fromKBps( myPrefs.get<int>(key) ) ) ) );
             break;
 
         case Prefs::RATIO_ENABLED:
@@ -1027,13 +966,7 @@ TrMainWindow :: refreshPref( int key )
             break;
 
         case Prefs::RATIO:
-            myRatioOnAction->setText( tr( "Stop at Ratio (%1)" ).arg( Utils::ratioToString( myPrefs.get<double>(key) ) ) );
-            break;
-
-        case Prefs::FILTER_MODE:
-            i = myPrefs.get<FilterMode>(key).mode( );
-            for( int j=0; j<FilterMode::NUM_MODES; ++j )
-                myFilterButtons[j]->setChecked( i==j );
+            myRatioOnAction->setText( tr( "Stop at Ratio (%1)" ).arg( Formatter::ratioToString( myPrefs.get<double>(key) ) ) );
             break;
 
         case Prefs::FILTERBAR:
@@ -1058,6 +991,7 @@ TrMainWindow :: refreshPref( int key )
             b = myPrefs.getBool( key );
             ui.action_TrayIcon->setChecked( b );
             myTrayIcon.setVisible( b );
+            refreshTrayIconSoon( );
             break;
 
         case Prefs::COMPACT_VIEW: {
@@ -1092,10 +1026,10 @@ TrMainWindow :: refreshPref( int key )
             myAltSpeedButton->setIcon( b ? mySpeedModeOnIcon : mySpeedModeOffIcon );
             const QString fmt = b ? tr( "Click to disable Temporary Speed Limits\n(%1 down, %2 up)" )
                                   : tr( "Click to enable Temporary Speed Limits\n(%1 down, %2 up)" );
-            const Speed d = Speed::fromKbps( myPrefs.getInt( Prefs::ALT_SPEED_LIMIT_DOWN ) );
-            const Speed u = Speed::fromKbps( myPrefs.getInt( Prefs::ALT_SPEED_LIMIT_UP ) );
-            myAltSpeedButton->setToolTip( fmt.arg( Utils::speedToString( d ) )
-                                             .arg( Utils::speedToString( u ) ) );
+            const Speed d = Speed::fromKBps( myPrefs.getInt( Prefs::ALT_SPEED_LIMIT_DOWN ) );
+            const Speed u = Speed::fromKBps( myPrefs.getInt( Prefs::ALT_SPEED_LIMIT_UP ) );
+            myAltSpeedButton->setToolTip( fmt.arg( Formatter::speedToString( d ) )
+                                             .arg( Formatter::speedToString( u ) ) );
             break;
         }
 
@@ -1120,7 +1054,7 @@ TrMainWindow :: openTorrent( )
 {
     QFileDialog * myFileDialog;
     myFileDialog = new QFileDialog( this,
-                                    tr( "Add Torrent" ),
+                                    tr( "Open Torrent" ),
                                     myPrefs.getString( Prefs::OPEN_DIALOG_FOLDER ),
                                     tr( "Torrent Files (*.torrent);;All Files (*.*)" ) );
     myFileDialog->setFileMode( QFileDialog::ExistingFiles );
@@ -1140,8 +1074,15 @@ TrMainWindow :: openTorrent( )
 void
 TrMainWindow :: openURL( )
 {
-    QString tmp;
-    openURL( tmp );
+    QString str = QApplication::clipboard()->text( QClipboard::Selection );
+
+    if( !AddData::isSupported( str ) )
+        str = QApplication::clipboard()->text( QClipboard::Clipboard );
+
+    if( !AddData::isSupported( str ) )
+        str.clear();
+
+    openURL( str );
 }
 
 void
@@ -1149,8 +1090,8 @@ TrMainWindow :: openURL( QString url )
 {
     bool ok;
     const QString key = QInputDialog::getText( this,
-                                               tr( "Add URL or Magnet Link" ),
-                                               tr( "Add URL or Magnet Link" ),
+                                               tr( "Open URL or Magnet Link" ),
+                                               tr( "Open URL or Magnet Link" ),
                                                QLineEdit::Normal,
                                                url,
                                                &ok );
@@ -1190,7 +1131,7 @@ TrMainWindow :: removeTorrents( const bool deleteFiles )
 
     foreach( QModelIndex index, ui.listView->selectionModel( )->selectedRows( ) )
     {
-        const Torrent * tor( index.model()->data( index, TorrentModel::TorrentRole ).value<const Torrent*>( ) );
+        const Torrent * tor( index.data( TorrentModel::TorrentRole ).value<const Torrent*>( ) );
         ids.insert( tor->id( ) );
         if( tor->connectedPeers( ) )
             ++connected;
@@ -1300,7 +1241,7 @@ TrMainWindow :: updateNetworkIcon( )
     myNetworkLabel->setPixmap( pixmap );
     myNetworkLabel->setToolTip( isSending || isReading
         ? tr( "Transmission server is responding" )
-        : tr( "Last response from server was %1 ago" ).arg( Utils::timeToString( now-std::max(myLastReadTime,myLastSendTime))));
+        : tr( "Last response from server was %1 ago" ).arg( Formatter::timeToString( now-std::max(myLastReadTime,myLastSendTime))));
 }
 
 void
@@ -1329,3 +1270,29 @@ TrMainWindow :: wrongAuthentication( )
     mySession.stop( );
     mySessionDialog->show( );
 }
+
+/***
+****
+***/
+
+void
+TrMainWindow :: dragEnterEvent( QDragEnterEvent * event )
+{
+    const QMimeData * mime = event->mimeData( );
+
+    if( mime->hasFormat("application/x-bittorrent")
+            || mime->text().trimmed().endsWith(".torrent", Qt::CaseInsensitive) )
+        event->acceptProposedAction();
+}
+
+void
+TrMainWindow :: dropEvent( QDropEvent * event )
+{
+    QString key = event->mimeData()->text().trimmed();
+
+    const QUrl url( key );
+    if( url.scheme() == "file" )
+        key = QUrl::fromPercentEncoding( url.path().toUtf8( ) );
+
+    dynamic_cast<MyApp*>(QApplication::instance())->addTorrent( key );
+}
