@@ -1,13 +1,13 @@
 /*
- * This file Copyright (C) 2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
- * Transmission project are granted a special exemption to clause 2(b)
- * so that the bulk of its code can remain under the MIT license.
- * This exemption does not extend to derived works not owned by
- * the Transmission project.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2
+ * as published by the Free Software Foundation.
  *
- * $Id: mainwin.h 9367 2009-10-22 03:01:38Z charles $
+ * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+ *
+ * $Id: mainwin.h 12557 2011-07-19 21:19:18Z jordan $
  */
 
 #ifndef MAIN_WINDOW_H
@@ -15,11 +15,11 @@
 
 #include <ctime>
 #include <QCheckBox>
-#include <QFileDialog>
 #include <QLineEdit>
 #include <QIcon>
 #include <QMainWindow>
 #include <QMap>
+#include <QPushButton>
 #include <QSet>
 #include <QSystemTrayIcon>
 #include <QTimer>
@@ -45,6 +45,7 @@ class QLabel;
 class QMenu;
 class QModelIndex;
 class QSortFilterProxyModel;
+class Filterbar;
 
 class TrMainWindow: public QMainWindow
 {
@@ -60,7 +61,6 @@ class TrMainWindow: public QMainWindow
         QDialog * myAboutDialog;
         QDialog * myStatsDialog;
         Details * myDetailsDialog;
-        QFileDialog * myFileDialog;
         QCheckBox * myFileDialogOptionsCheck;
         QSystemTrayIcon myTrayIcon;
         TorrentFilter myFilterModel;
@@ -75,6 +75,8 @@ class TrMainWindow: public QMainWindow
         time_t myLastSendTime;
         time_t myLastReadTime;
         QTimer myNetworkTimer;
+        QTimer myRefreshTrayIconTimer;
+        QTimer myRefreshActionSensitivityTimer;
         QAction * myDlimitOffAction;
         QAction * myDlimitOnAction;
         QAction * myUlimitOffAction;
@@ -86,22 +88,17 @@ class TrMainWindow: public QMainWindow
         QIcon getStockIcon( const QString&, int fallback=-1 );
 
     private:
-        void setShowMode( int );
         QSet<int> getSelectedTorrents( ) const;
         void updateNetworkIcon( );
         QWidgetList myHidden;
 
+    public slots:
+        void openURL( QString );
+
     private slots:
+        void onPrefsDestroyed( );
+        void openPreferences( );
         void onDetailsDestroyed( );
-        void onShowModeClicked( );
-        void showAll( );
-        void showActive( );
-        void showDownloading( );
-        void showSeeding( );
-        void showPaused( );
-        void filterByName( );
-        void filterByFiles( );
-        void filterByTracker( );
         void showTotalRatio( );
         void showTotalTransfer( );
         void showSessionRatio( );
@@ -109,13 +106,19 @@ class TrMainWindow: public QMainWindow
         void refreshVisibleCount( );
         void refreshTitle( );
         void refreshStatusBar( );
+        void refreshTrayIcon( );
+        void refreshTrayIconSoon( );
         void openTorrent( );
+        void openURL( );
         void newTorrent( );
         void trayActivated( QSystemTrayIcon::ActivationReason );
         void refreshPref( int key );
         void addTorrents( const QStringList& filenames );
+        void removeTorrents( const bool deleteFiles );
+        void openDonate( );
         void openHelp( );
         void openFolder( );
+        void copyMagnetLinkToClipboard( );
         void setLocation( );
         void openProperties( );
         void toggleSpeedMode( );
@@ -138,21 +141,14 @@ class TrMainWindow: public QMainWindow
         void onSortByRatioToggled    ( bool );
         void onSortBySizeToggled     ( bool );
         void onSortByStateToggled    ( bool );
-        void onSortByTrackerToggled  ( bool );
 
     private:
-        QWidget * createFilterBar( void );
         QWidget * myFilterBar;
-        QPushButton * myFilterButtons[FilterMode::NUM_MODES];
-        QPushButton * myFilterTextButton;
-        QLineEdit * myFilterTextLineEdit;
 
     private:
         QMenu * createOptionsMenu( void );
         QWidget * createStatusBar( void );
         QWidget * myStatusBar;
-        QWidgetList myUpStatusWidgets;
-        QWidgetList myDownStatusWidgets;
         QPushButton * myAltSpeedButton;
         QPushButton * myOptionsButton;
         QLabel * myVisibleCountLabel;
@@ -181,14 +177,18 @@ class TrMainWindow: public QMainWindow
         void setToolbarVisible( bool );
         void setFilterbarVisible( bool );
         void setStatusbarVisible( bool );
-        void setTrayIconVisible( bool );
-        void setMinimalView( bool );
+        void setCompactView( bool );
         void refreshActionSensitivity( );
+        void refreshActionSensitivitySoon( );
         void wrongAuthentication( );
 
     public:
         TrMainWindow( Session&, Prefs&, TorrentModel&, bool minized );
         virtual ~TrMainWindow( );
+
+    protected:
+        virtual void dragEnterEvent( QDragEnterEvent * );
+        virtual void dropEvent( QDropEvent * );
 };
 
 #endif
