@@ -1,13 +1,13 @@
 /*
- * This file Copyright (C) 2009-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
- * Transmission project are granted a special exemption to clause 2(b)
- * so that the bulk of its code can remain under the MIT license.
- * This exemption does not extend to derived works not owned by
- * the Transmission project.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2
+ * as published by the Free Software Foundation.
  *
- * $Id: prefs-dialog.h 10722 2010-06-04 01:00:27Z Longinus00 $
+ * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+ *
+ * $Id: prefs-dialog.h 11452 2010-12-04 00:19:52Z charles $
  */
 
 #ifndef PREFS_DIALOG_H
@@ -41,12 +41,10 @@ class PrefsDialog: public QDialog
 
     private slots:
         void checkBoxToggled( bool checked );
-        void spinBoxChanged( int value );
-        void doubleSpinBoxChanged( double value );
-        void spinBoxChangedIdle( );
-        void timeChanged( const QTime& );
-        void textChanged( const QString& );
-        void updatePref( int key );
+        void spinBoxEditingFinished( );
+        void timeEditingFinished( );
+        void lineEditingFinished( );
+        void refreshPref( int key );
         void encryptionEdited( int );
         void altSpeedDaysEdited( int );
         void sessionUpdated( );
@@ -70,20 +68,21 @@ class PrefsDialog: public QDialog
         QTimeEdit * timeEditNew( int key );
         QLineEdit * lineEditNew( int key, int mode = 0 );
         void enableBuddyWhenChecked( QCheckBox *, QWidget * );
-        void updateBlocklistCheckBox( );
+        void updateBlocklistLabel( );
 
     public:
         PrefsDialog( Session&, Prefs&, QWidget * parent = 0 );
         ~PrefsDialog( );
 
     private:
+        void setPref( int key, const QVariant& v );
         bool isAllowed( int key ) const;
         QWidget * createTorrentsTab( );
-        QWidget * createNetworkTab( );
-        QWidget * createPrivacyTab( );
         QWidget * createSpeedTab( );
+        QWidget * createPrivacyTab( );
+        QWidget * createNetworkTab( );
+        QWidget * createDesktopTab( );
         QWidget * createWebTab( Session& );
-        QWidget * createTrackerTab( );
 
     private:
         typedef QMap<int,QWidget*> key2widget_t;
@@ -112,6 +111,7 @@ class PrefsDialog: public QDialog
         int myBlocklistHttpTag;
         QHttp * myBlocklistHttp;
         QMessageBox * myBlocklistDialog;
+        QLabel * myBlocklistLabel;
 };
 
 #endif
