@@ -1,13 +1,13 @@
 /*
- * This file Copyright (C) 2009-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
- * Transmission project are granted a special exemption to clause 2(b)
- * so that the bulk of its code can remain under the MIT license.
- * This exemption does not extend to derived works not owned by
- * the Transmission project.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2
+ * as published by the Free Software Foundation.
  *
- * $Id: session.cc 11478 2010-12-05 18:55:00Z charles $
+ * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+ *
+ * $Id: session.cc 12461 2011-05-27 13:36:53Z jordan $
  */
 
 #include <cassert>
@@ -19,21 +19,29 @@
 #include <QCoreApplication>
 #include <QDesktopServices>
 #include <QMessageBox>
+#include <QNetworkProxy>
+#include <QNetworkProxyFactory>
 #include <QNetworkReply>
 #include <QNetworkRequest>
 #include <QSet>
+#include <QStringList>
 #include <QStyle>
 #include <QTextStream>
 
+#include <curl/curl.h>
+
+#include <event2/buffer.h>
+
 #include <libtransmission/transmission.h>
 #include <libtransmission/bencode.h>
 #include <libtransmission/json.h>
 #include <libtransmission/rpcimpl.h>
-#include <libtransmission/utils.h> /* tr_free */
-#include <libtransmission/version.h> /* LONG_VERSION */
+#include <libtransmission/utils.h> // tr_free
+#include <libtransmission/version.h> // LONG_VERSION
+#include <libtransmission/web.h>
 
+#include "add-data.h"
 #include "prefs.h"
-#include "qticonloader.h"
 #include "session.h"
 #include "session-dialog.h"
 #include "torrent.h"
@@ -140,10 +148,13 @@ Session :: updatePref( int key )
         case Prefs :: ALT_SPEED_LIMIT_UP:
         case Prefs :: BLOCKLIST_DATE:
         case Prefs :: BLOCKLIST_ENABLED:
+        case Prefs :: BLOCKLIST_URL:
         case Prefs :: DHT_ENABLED:
         case Prefs :: DOWNLOAD_DIR:
         case Prefs :: DSPEED:
         case Prefs :: DSPEED_ENABLED:
+        case Prefs :: IDLE_LIMIT:
+        case Prefs :: IDLE_LIMIT_ENABLED:
         case Prefs :: INCOMPLETE_DIR:
         case Prefs :: INCOMPLETE_DIR_ENABLED:
         case Prefs :: LPD_ENABLED:
@@ -159,6 +170,7 @@ Session :: updatePref( int key )
         case Prefs :: TRASH_ORIGINAL:
         case Prefs :: USPEED:
         case Prefs :: USPEED_ENABLED:
+        case Prefs :: UTP_ENABLED:
             sessionSet( myPrefs.keyStr(key), myPrefs.variant(key) );
             break;
 
@@ -418,6 +430,21 @@ Session :: torrentSet( const QSet<int>& ids, const QString& key, bool value )
     tr_bencFree( &top );
 }
 
+void
+Session :: torrentSet( const QSet<int>& ids, const QString& key, const QStringList& value )
+{
+    tr_benc top;
+    tr_bencInitDict( &top, 2 );
+    tr_bencDictAddStr( &top, "method", "torrent-set" );
+    tr_benc * args = tr_bencDictAddDict( &top, "arguments", 2 );
+    addOptionalIds( args, ids );
+    tr_benc * list( tr_bencDictAddList( args, key.toUtf8().constData(), value.size( ) ) );
+    foreach( const QString str, value )
+        tr_bencListAddStr( list, str.toUtf8().constData() );
+    exec( &top );
+    tr_bencFree( &top );
+}
+
 void
 Session :: torrentSet( const QSet<int>& ids, const QString& key, const QList<int>& value )
 {
@@ -433,6 +460,21 @@ Session :: torrentSet( const QSet<int>& ids, const QString& key, const QList<int
     tr_bencFree( &top );
 }
 
+void
+Session :: torrentSet( const QSet<int>& ids, const QString& key, const QPair<int,QString>& value )
+{
+    tr_benc top;
+    tr_bencInitDict( &top, 2 );
+    tr_bencDictAddStr( &top, "method", "torrent-set" );
+    tr_benc * args( tr_bencDictAddDict( &top, "arguments", 2 ) );
+    addOptionalIds( args, ids );
+    tr_benc * list( tr_bencDictAddList( args, key.toUtf8().constData(), 2 ) );
+    tr_bencListAddInt( list, value.first );
+    tr_bencListAddStr( list, value.second.toUtf8().constData() );
+    exec( &top );
+    tr_bencFree( &top );
+}
+
 void
 Session :: torrentSetLocation( const QSet<int>& ids, const QString& location, bool doMove )
 {
@@ -591,11 +633,11 @@ Session :: exec( const tr_benc * request )
 }
 
 void
-Session :: localSessionCallback( tr_session * session, const char * json, size_t len, void * self )
+Session :: localSessionCallback( tr_session * session, struct evbuffer * json, void * self )
 {
     Q_UNUSED( session );
 
-    ((Session*)self)->parseResponse( json, len );
+    ((Session*)self)->parseResponse( (const char*) evbuffer_pullup( json, -1 ), evbuffer_get_length( json ) );
 }
 
 #define REQUEST_DATA_PROPERTY_KEY "requestData"
@@ -725,7 +767,7 @@ Session :: parseResponse( const char * json, size_t jsonLength )
                 }
 
                 case TAG_PORT_TEST: {
-                    tr_bool isOpen = 0;
+                    bool isOpen = 0;
                     if( tr_bencDictFindDict( &top, "arguments", &args ) )
                         tr_bencDictFindBool( args, "port-is-open", &isOpen );
                     emit portTested( (bool)isOpen );
@@ -752,12 +794,6 @@ Session :: parseResponse( const char * json, size_t jsonLength )
                                                            QString::fromUtf8(str),
                                                            QMessageBox::Close,
                                                            QApplication::activeWindow());
-                        QPixmap pixmap;
-                        QIcon icon = QtIconLoader :: icon( "dialog-information" );
-                        if( !icon.isNull( ) ) {
-                            const int size = QApplication::style()->pixelMetric( QStyle::PM_LargeIconSize );
-                            d->setIconPixmap( icon.pixmap( size, size ) );
-                        }
                         connect( d, SIGNAL(rejected()), d, SLOT(deleteLater()) );
                         d->show( );
                     }
@@ -849,7 +885,7 @@ Session :: updateInfo( tr_benc * d )
                 break;
             }
             case QVariant :: Bool: {
-                tr_bool val;
+                bool val;
                 if( tr_bencGetBool( b, &val ) )
                     myPrefs.set( i, (bool)val );
                 break;
@@ -867,7 +903,7 @@ Session :: updateInfo( tr_benc * d )
         }
     }
 
-    tr_bool b;
+    bool b;
     double x;
     if( tr_bencDictFindBool( d, "seedRatioLimited", &b ) )
         myPrefs.set( Prefs::RATIO_ENABLED, b ? true : false );
@@ -907,54 +943,38 @@ Session :: setBlocklistSize( int64_t i )
 }
 
 void
-Session :: addTorrent( QString filename )
+Session :: addTorrent( const AddData& addMe )
 {
-    addTorrent( filename, myPrefs.getString( Prefs::DOWNLOAD_DIR ) );
-}
+    const QByteArray b64 = addMe.toBase64();
 
-namespace
-{
-    bool isLink( const QString& str )
-    {
-        return Utils::isMagnetLink(str) || Utils::isURL(str);
+    tr_benc top, *args;
+    tr_bencInitDict( &top, 2 );
+    tr_bencDictAddStr( &top, "method", "torrent-add" );
+    args = tr_bencDictAddDict( &top, "arguments", 2 );
+    tr_bencDictAddBool( args, "paused", !myPrefs.getBool( Prefs::START ) );
+    switch( addMe.type ) {
+        case AddData::MAGNET:   tr_bencDictAddStr( args, "filename", addMe.magnet.toUtf8().constData() ); break;
+        case AddData::URL:      tr_bencDictAddStr( args, "filename", addMe.url.toString().toUtf8().constData() ); break;
+        case AddData::FILENAME: /* fall-through */
+        case AddData::METAINFO: tr_bencDictAddRaw( args, "metainfo", b64.constData(), b64.size() ); break;
+        default: std::cerr << "Unhandled AddData type: " << addMe.type << std::endl;
     }
+    exec( &top );
+    tr_bencFree( &top );
 }
 
 void
-Session :: addTorrent( QString key, QString localPath )
+Session :: addNewlyCreatedTorrent( const QString& filename, const QString& localPath )
 {
+    const QByteArray b64 = AddData(filename).toBase64();
+
     tr_benc top, *args;
     tr_bencInitDict( &top, 2 );
     tr_bencDictAddStr( &top, "method", "torrent-add" );
     args = tr_bencDictAddDict( &top, "arguments", 3 );
     tr_bencDictAddStr( args, "download-dir", qPrintable(localPath) );
     tr_bencDictAddBool( args, "paused", !myPrefs.getBool( Prefs::START ) );
-
-    // figure out what to do with "key"....
-    bool keyHandled = false;
-    if( !keyHandled && isLink( key  )) {
-        tr_bencDictAddStr( args, "filename", key.toUtf8().constData() );
-        keyHandled = true; // it's a URL or magnet link...
-    }
-    if( !keyHandled ) {
-        QFile file( key );
-        file.open( QIODevice::ReadOnly );
-        const QByteArray raw( file.readAll( ) );
-        file.close( );
-        if( !raw.isEmpty( ) ) {
-            int b64len = 0;
-            char * b64 = tr_base64_encode( raw.constData(), raw.size(), &b64len );
-            tr_bencDictAddRaw( args, "metainfo", b64, b64len  );
-            tr_free( b64 );
-            keyHandled = true; // it's a local file...
-        }
-    }
-    if( !keyHandled ) {
-        const QByteArray tmp = key.toUtf8();
-        tr_bencDictAddRaw( args, "metainfo", tmp.constData(), tmp.length() );
-        keyHandled = true; // treat it as base64
-    }
-
+    tr_bencDictAddRaw( args, "metainfo", b64.constData(), b64.size() );
     exec( &top );
     tr_bencFree( &top );
 }
