@@ -1,26 +1,27 @@
 /*
- * This file Copyright (C) 2009-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
- * Transmission project are granted a special exemption to clause 2(b)
- * so that the bulk of its code can remain under the MIT license.
- * This exemption does not extend to derived works not owned by
- * the Transmission project.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2
+ * as published by the Free Software Foundation.
  *
- * $Id: torrent-filter.cc 10732 2010-06-05 20:44:11Z Longinus00 $
+ * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+ *
+ * $Id: torrent-filter.cc 11519 2010-12-12 16:07:04Z charles $
  */
 
 #include <iostream>
 
 #include "filters.h"
+#include "hig.h"
 #include "prefs.h"
 #include "torrent.h"
 #include "torrent-filter.h"
 #include "torrent-model.h"
+#include "utils.h"
 
 TorrentFilter :: TorrentFilter( Prefs& prefs ):
-    myPrefs( prefs ),
-    myTextMode( FILTER_BY_NAME )
+    myPrefs( prefs )
 {
     // listen for changes to the preferences to know when to refilter / resort
     connect( &myPrefs, SIGNAL(changed(int)), this, SLOT(refreshPref(int)));
@@ -30,7 +31,9 @@ TorrentFilter :: TorrentFilter( Prefs& prefs ):
     // initialize our state from the current prefs
     QList<int> initKeys;
     initKeys << Prefs :: SORT_MODE
-             << Prefs :: FILTER_MODE;
+             << Prefs :: FILTER_MODE
+             << Prefs :: FILTER_TRACKERS
+             << Prefs :: FILTER_TEXT;
     foreach( int key, initKeys )
         refreshPref( key );
 }
@@ -44,10 +47,13 @@ TorrentFilter :: refreshPref( int key )
 {
     switch( key )
     {
+        case Prefs :: FILTER_TEXT:
         case Prefs :: FILTER_MODE:
+        case Prefs :: FILTER_TRACKERS:
             invalidateFilter( );
             /* force a re-sort */
             sort( 0, !myPrefs.getBool(Prefs::SORT_REVERSED) ? Qt::AscendingOrder : Qt::DescendingOrder );
+
         case Prefs :: SORT_MODE:
         case Prefs :: SORT_REVERSED:
             sort( 0, myPrefs.getBool(Prefs::SORT_REVERSED) ? Qt::AscendingOrder : Qt::DescendingOrder );
@@ -60,32 +66,6 @@ TorrentFilter :: refreshPref( int key )
 ****
 ***/
 
-void
-TorrentFilter :: setTextMode( int i )
-{
-    if( myTextMode != i )
-    {
-        myTextMode = TextMode( i );
-        invalidateFilter( );
-    }
-}
-
-void
-TorrentFilter :: setText( QString text )
-{
-    QString trimmed = text.trimmed( );
-
-    if( myText != trimmed )
-    {
-        myText = trimmed;
-        invalidateFilter( );
-    }
-}
-
-/***
-****
-***/
-
 namespace
 {
     template <typename T> int compare( const T a, const T b )
@@ -99,54 +79,47 @@ namespace
 bool
 TorrentFilter :: lessThan( const QModelIndex& left, const QModelIndex& right ) const
 {
+    int val = 0;
     const Torrent * a = sourceModel()->data( left, TorrentModel::TorrentRole ).value<const Torrent*>();
     const Torrent * b = sourceModel()->data( right, TorrentModel::TorrentRole ).value<const Torrent*>();
-    int less = 0;
 
     switch( myPrefs.get<SortMode>(Prefs::SORT_MODE).mode() )
     {
         case SortMode :: SORT_BY_SIZE:
-            less = compare( a->sizeWhenDone(), b->sizeWhenDone() );
+            if( !val ) val = compare( a->sizeWhenDone(), b->sizeWhenDone() );
             break;
         case SortMode :: SORT_BY_ACTIVITY:
-            less = compare( a->downloadSpeed() + a->uploadSpeed(), b->downloadSpeed() + b->uploadSpeed() );
-            if( !less )
-                less = compare( a->uploadedEver(), b->uploadedEver() );
+            if( !val ) val = compare( a->downloadSpeed() + a->uploadSpeed(), b->downloadSpeed() + b->uploadSpeed() );
+            if( !val ) val = compare( a->uploadedEver(), b->uploadedEver() );
             break;
         case SortMode :: SORT_BY_AGE:
-            less = compare( a->dateAdded().toTime_t(), b->dateAdded().toTime_t() );
+            val = compare( a->dateAdded().toTime_t(), b->dateAdded().toTime_t() );
             break;
         case SortMode :: SORT_BY_ID:
-            less = compare( a->id(), b->id() );
+            if( !val ) val = compare( a->id(), b->id() );
             break;
         case SortMode :: SORT_BY_STATE:
-            if( a->hasError() != b->hasError() )
-                less = a->hasError();
-            else
-                less = compare( a->getActivity(), b->getActivity() );
-            if( less )
-                break;
+            if( !val ) val = compare( a->hasError(), b->hasError() );
+            if( !val ) val = compare( a->getActivity(), b->getActivity() );
+            // fall through
         case SortMode :: SORT_BY_PROGRESS:
-            less = compare( a->percentDone(), b->percentDone() );
-            if( less )
-                break;
+            if( !val ) val = compare( a->percentComplete(), b->percentComplete() );
+            if( !val ) val = a->compareSeedRatio( *b );
+            // fall through
         case SortMode :: SORT_BY_RATIO:
-            less = a->compareRatio( *b );
+            if( !val ) val = a->compareRatio( *b );
             break;
         case SortMode :: SORT_BY_ETA:
-            less = a->compareETA( *b );
-            break;
-        case SortMode :: SORT_BY_TRACKER:
-            less = a->compareTracker( *b );
+            if( !val ) val = a->compareETA( *b );
             break;
         default:
             break;
     }
-    if( less == 0 )
-        less = -a->name().compare( b->name(), Qt::CaseInsensitive );
-    if( less == 0 )
-        less = compare( a->hashString(), b->hashString() );
-    return less < 0;
+    if( val == 0 )
+        val = -a->name().compare( b->name(), Qt::CaseInsensitive );
+    if( val == 0 )
+        val = compare( a->hashString(), b->hashString() );
+    return val < 0;
 }
 
 
@@ -155,32 +128,74 @@ TorrentFilter :: lessThan( const QModelIndex& left, const QModelIndex& right ) c
 ***/
 
 bool
-TorrentFilter :: filterAcceptsRow( int sourceRow, const QModelIndex& sourceParent ) const
+TorrentFilter :: trackerFilterAcceptsTorrent( const Torrent * tor, const QString& tracker ) const
+{
+    return tracker.isEmpty() || tor->hasTrackerSubstring( tracker );
+}
+
+bool
+TorrentFilter :: activityFilterAcceptsTorrent( const Torrent * tor, const FilterMode& m ) const
 {
-    QModelIndex childIndex = sourceModel()->index( sourceRow, 0, sourceParent );
-    const Torrent * tor = childIndex.model()->data( childIndex, TorrentModel::TorrentRole ).value<const Torrent*>();
-    const tr_torrent_activity activity = tor->getActivity( );
     bool accepts;
 
-    switch( myPrefs.get<FilterMode>(Prefs::FILTER_MODE).mode() )
+    switch( m.mode( ) )
     {
-        case FilterMode::SHOW_ALL:
-            accepts = true;
-            break;
         case FilterMode::SHOW_ACTIVE:
             accepts = tor->peersWeAreUploadingTo( ) > 0 || tor->peersWeAreDownloadingFrom( ) > 0 || tor->isVerifying( );
             break;
         case FilterMode::SHOW_DOWNLOADING:
-            accepts = activity == TR_STATUS_DOWNLOAD;
+            accepts = tor->isDownloading( );
             break;
         case FilterMode::SHOW_SEEDING:
-            accepts = activity == TR_STATUS_SEED;
+            accepts = tor->isSeeding( );
             break;
         case FilterMode::SHOW_PAUSED:
-            accepts = activity == TR_STATUS_STOPPED;
+            accepts = tor->isPaused( );
+            break;
+        case FilterMode::SHOW_FINISHED:
+            accepts = tor->isFinished( );
+            break;
+        case FilterMode::SHOW_QUEUED:
+            accepts = tor->isWaitingToVerify( );
+            break;
+        case FilterMode::SHOW_VERIFYING:
+            accepts = tor->isVerifying( ) || tor->isWaitingToVerify( );
+            break;
+        case FilterMode::SHOW_ERROR:
+            accepts = tor->hasError( );
+            break;
+        default: // FilterMode::SHOW_ALL
+            accepts = true;
             break;
     }
 
+    return accepts;
+}
+
+bool
+TorrentFilter :: filterAcceptsRow( int sourceRow, const QModelIndex& sourceParent ) const
+{
+    QModelIndex childIndex = sourceModel()->index( sourceRow, 0, sourceParent );
+    const Torrent * tor = childIndex.model()->data( childIndex, TorrentModel::TorrentRole ).value<const Torrent*>();
+    bool accepts = true;
+
+    if( accepts ) {
+        const FilterMode m = myPrefs.get<FilterMode>(Prefs::FILTER_MODE);
+        accepts = activityFilterAcceptsTorrent( tor, m );
+    }
+
+    if( accepts ) {
+        const QString trackers = myPrefs.getString(Prefs::FILTER_TRACKERS);
+        accepts = trackerFilterAcceptsTorrent( tor, trackers );
+    }
+
+    if( accepts ) {
+        const QString text = myPrefs.getString( Prefs::FILTER_TEXT );
+        if( !text.isEmpty( ) )
+            accepts = tor->name().contains( text, Qt::CaseInsensitive );
+    }
+
+#if 0
     if( accepts && !myText.isEmpty( ) ) switch( myTextMode )
     {
         case FILTER_BY_NAME:
@@ -193,6 +208,7 @@ TorrentFilter :: filterAcceptsRow( int sourceRow, const QModelIndex& sourceParen
             accepts = tor->hasTrackerSubstring( myText );
             break;
     }
+#endif
 
     return accepts;
 }
@@ -202,3 +218,20 @@ TorrentFilter :: hiddenRowCount( ) const
 {
     return sourceModel()->rowCount( ) - rowCount( );
 }
+
+int
+TorrentFilter :: count( const FilterMode& mode ) const
+{
+    int count = 0;
+
+    for( int row=0; ; ++row ) {
+        QModelIndex index = sourceModel()->index( row, 0 );
+        if( !index.isValid( ) )
+            break;
+        const Torrent * tor = index.data( TorrentModel::TorrentRole ).value<const Torrent*>();
+        if( activityFilterAcceptsTorrent( tor, mode ) )
+            ++count;
+    }
+
+    return count;
+}
