@@ -1,13 +1,13 @@
 /*
- * This file Copyright (C) 2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
- * Transmission project are granted a special exemption to clause 2(b)
- * so that the bulk of its code can remain under the MIT license.
- * This exemption does not extend to derived works not owned by
- * the Transmission project.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2
+ * as published by the Free Software Foundation.
  *
- * $Id: torrent-filter.h 8570 2009-05-31 19:33:48Z charles $
+ * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+ *
+ * $Id: torrent-filter.h 11092 2010-08-01 20:36:13Z charles $
  */
 
 #ifndef QTR_TORRENT_FILTER_H
@@ -17,9 +17,12 @@
 #include <QMetaType>
 #include <QVariant>
 
+class QString;
+class QWidget;
 
-struct Prefs;
-struct QString;
+class FilterMode;
+class Prefs;
+class Torrent;
 
 class TorrentFilter: public QSortFilterProxyModel
 {
@@ -31,13 +34,8 @@ class TorrentFilter: public QSortFilterProxyModel
 
     public:
         enum TextMode { FILTER_BY_NAME, FILTER_BY_FILES, FILTER_BY_TRACKER };
-        TextMode getTextMode( ) const { return myTextMode; }
         int hiddenRowCount( ) const;
 
-    public slots:
-        void setTextMode( int textMode );
-        void setText( QString );
-
     private slots:
         void refreshPref( int key );
 
@@ -45,10 +43,15 @@ class TorrentFilter: public QSortFilterProxyModel
         virtual bool filterAcceptsRow( int, const QModelIndex& ) const;
         virtual bool lessThan( const QModelIndex&, const QModelIndex& ) const;
 
+    private:
+        bool activityFilterAcceptsTorrent( const Torrent * tor, const FilterMode& mode ) const;
+        bool trackerFilterAcceptsTorrent( const Torrent * tor, const QString& tracker ) const;
+
+    public:
+        int count( const FilterMode& ) const;
+
     private:
         Prefs& myPrefs;
-        TextMode myTextMode;
-        QString myText;
 };
 
 #endif
