@@ -1,13 +1,13 @@
 /*
- * This file Copyright (C) 2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
- * Transmission project are granted a special exemption to clause 2(b)
- * so that the bulk of its code can remain under the MIT license.
- * This exemption does not extend to derived works not owned by
- * the Transmission project.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2
+ * as published by the Free Software Foundation.
  *
- * $Id: torrent.h 8869 2009-08-05 01:25:36Z charles $
+ * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+ *
+ * $Id: torrent.h 11446 2010-12-01 05:41:58Z charles $
  */
 
 #ifndef QTR_TORRENT_H
@@ -34,6 +34,7 @@ extern "C"
 }
 
 class Prefs;
+class QPixmap;
 class QStyle;
 
 struct Peer
@@ -42,6 +43,7 @@ struct Peer
     QString clientName;
     bool clientIsChoked;
     bool clientIsInterested;
+    QString flagStr;
     bool isDownloadingFrom;
     bool isEncrypted;
     bool isIncoming;
@@ -58,6 +60,40 @@ typedef QList<Peer> PeerList;
 Q_DECLARE_METATYPE(Peer)
 Q_DECLARE_METATYPE(PeerList)
 
+struct TrackerStat
+{
+    QString announce;
+    int announceState;
+    int downloadCount;
+    bool hasAnnounced;
+    bool hasScraped;
+    QString host;
+    int id;
+    bool isBackup;
+    int lastAnnouncePeerCount;
+    QString lastAnnounceResult;
+    int lastAnnounceStartTime;
+    bool lastAnnounceSucceeded;
+    int lastAnnounceTime;
+    bool lastAnnounceTimedOut;
+    QString lastScrapeResult;
+    int lastScrapeStartTime;
+    bool lastScrapeSucceeded;
+    int lastScrapeTime;
+    bool lastScrapeTimedOut;
+    int leecherCount;
+    int nextAnnounceTime;
+    int nextScrapeTime;
+    int scrapeState;
+    int seederCount;
+    int tier;
+    QPixmap getFavicon( ) const;
+};
+
+typedef QList<TrackerStat> TrackerStatsList;
+Q_DECLARE_METATYPE(TrackerStat)
+Q_DECLARE_METATYPE(TrackerStatsList)
+
 struct TrFile
 {
     TrFile(): index(-1), priority(0), wanted(true), size(0), have(0) { }
@@ -85,7 +121,6 @@ class Torrent: public QObject
             ID,
             UPLOAD_SPEED,
             DOWNLOAD_SPEED,
-            SWARM_SPEED,
             DOWNLOAD_DIR,
             ACTIVITY,
             NAME,
@@ -95,6 +130,7 @@ class Torrent: public QObject
             LEFT_UNTIL_DONE,
             HAVE_UNCHECKED,
             HAVE_VERIFIED,
+            DESIRED_AVAILABLE,
             TOTAL_SIZE,
             PIECE_SIZE,
             PIECE_COUNT,
@@ -102,6 +138,7 @@ class Torrent: public QObject
             PEERS_SENDING_TO_US,
             WEBSEEDS_SENDING_TO_US,
             PERCENT_DONE,
+            METADATA_PERCENT_DONE,
             PERCENT_VERIFIED,
             DATE_ACTIVITY,
             DATE_ADDED,
@@ -114,9 +151,12 @@ class Torrent: public QObject
             UPLOADED_EVER,
             FAILED_EVER,
             TRACKERS,
+            TRACKERSTATS,
             MIME_ICON,
             SEED_RATIO_LIMIT,
             SEED_RATIO_MODE,
+            SEED_IDLE_LIMIT,
+            SEED_IDLE_MODE,
             DOWN_LIMIT,
             DOWN_LIMITED,
             UP_LIMIT,
@@ -124,20 +164,11 @@ class Torrent: public QObject
             HONORS_SESSION_LIMITS,
             PEER_LIMIT,
             HASH_STRING,
+            IS_FINISHED,
             IS_PRIVATE,
             COMMENT,
             CREATOR,
-            LAST_ANNOUNCE_TIME,
-            LAST_SCRAPE_TIME,
             MANUAL_ANNOUNCE_TIME,
-            NEXT_ANNOUNCE_TIME,
-            NEXT_SCRAPE_TIME,
-            SCRAPE_RESPONSE,
-            ANNOUNCE_RESPONSE,
-            ANNOUNCE_URL,
-            SEEDERS,
-            LEECHERS,
-            TIMES_COMPLETED,
             PEERS,
             TORRENT_FILE,
             BANDWIDTH_PRIORITY,
@@ -172,6 +203,8 @@ class Torrent: public QObject
 
         static Property myProperties[];
 
+        bool magnetTorrent;
+
     public:
         typedef QList<const char*> KeyList;
         static const KeyList& getInfoKeys( );
@@ -210,9 +243,6 @@ class Torrent: public QObject
         QString getPath( ) const { return getString( DOWNLOAD_DIR ); }
         QString getError( ) const;
         QString hashString( ) const { return getString( HASH_STRING ); }
-        QString scrapeResponse( ) const { return getString( SCRAPE_RESPONSE ); }
-        QString announceResponse( ) const { return getString( ANNOUNCE_RESPONSE ); }
-        QString announceUrl( ) const { return getString( ANNOUNCE_URL ); }
         QString torrentFile( ) const { return getString( TORRENT_FILE ); }
         bool hasError( ) const { return !getError( ).isEmpty( ); }
         bool isDone( ) const { return getSize( LEFT_UNTIL_DONE ) == 0; }
@@ -221,18 +251,24 @@ class Torrent: public QObject
         bool getSeedRatio( double& setme ) const;
         uint64_t haveVerified( ) const { return getSize( HAVE_VERIFIED ); }
         uint64_t haveUnverified( ) const { return getSize( HAVE_UNCHECKED ); }
+        uint64_t desiredAvailable( ) const { return getSize( DESIRED_AVAILABLE ); }
         uint64_t haveTotal( ) const { return haveVerified( ) + haveUnverified(); }
         uint64_t totalSize( ) const { return getSize( TOTAL_SIZE ); }
         uint64_t sizeWhenDone( ) const { return getSize( SIZE_WHEN_DONE ); }
         uint64_t leftUntilDone( ) const { return getSize( LEFT_UNTIL_DONE ); }
         uint64_t pieceSize( ) const { return getSize( PIECE_SIZE ); }
+        bool hasMetadata( ) const { return getDouble( METADATA_PERCENT_DONE ) >= 1.0; }
+        bool isMagnet( ) const { return magnetTorrent; }
         int  pieceCount( ) const { return getInt( PIECE_COUNT ); }
         double ratio( ) const { return getDouble( RATIO ); }
+        double percentComplete( ) const { return haveTotal() / (double)totalSize(); }
         double percentDone( ) const { return getDouble( PERCENT_DONE ); }
+        double metadataPercentDone( ) const { return getDouble( METADATA_PERCENT_DONE ); }
         uint64_t downloadedEver( ) const { return getSize( DOWNLOADED_EVER ); }
         uint64_t uploadedEver( ) const { return getSize( UPLOADED_EVER ); }
         uint64_t failedEver( ) const { return getSize( FAILED_EVER ); }
         int compareTracker( const Torrent& ) const;
+        int compareSeedRatio( const Torrent& ) const;
         int compareRatio( const Torrent& ) const;
         int compareETA( const Torrent& ) const;
         bool hasETA( ) const { return getETA( ) >= 0; }
@@ -241,48 +277,48 @@ class Torrent: public QObject
         QDateTime lastStarted( ) const { return getDateTime( DATE_STARTED ); }
         QDateTime dateAdded( ) const { return getDateTime( DATE_ADDED ); }
         QDateTime dateCreated( ) const { return getDateTime( DATE_CREATED ); }
-        QDateTime lastAnnounceTime( ) const { return getDateTime( LAST_ANNOUNCE_TIME ); }
-        QDateTime lastScrapeTime( ) const { return getDateTime( LAST_SCRAPE_TIME ); }
         QDateTime manualAnnounceTime( ) const { return getDateTime( MANUAL_ANNOUNCE_TIME ); }
-        QDateTime nextAnnounceTime( ) const { return getDateTime( NEXT_ANNOUNCE_TIME ); }
-        QDateTime nextScrapeTime( ) const { return getDateTime( NEXT_SCRAPE_TIME ); }
         bool canManualAnnounce( ) const { return isReadyToTransfer() && (manualAnnounceTime()<=QDateTime::currentDateTime()); }
         int peersWeAreDownloadingFrom( ) const { return getInt( PEERS_SENDING_TO_US ) + getInt( WEBSEEDS_SENDING_TO_US ); }
         int peersWeAreUploadingTo( ) const { return getInt( PEERS_GETTING_FROM_US ); }
         bool isUploading( ) const { return peersWeAreUploadingTo( ) > 0; }
         int connectedPeers( ) const { return getInt( PEERS_CONNECTED ); }
         int connectedPeersAndWebseeds( ) const { return connectedPeers( ) + getInt( WEBSEEDS_SENDING_TO_US ); }
-        Speed downloadSpeed( ) const { return Speed::fromBps( getInt( DOWNLOAD_SPEED ) ); }
-        Speed uploadSpeed( ) const { return Speed::fromBps( getInt( UPLOAD_SPEED ) ); }
-        Speed swarmSpeed( ) const { return Speed::fromKbps( getInt( SWARM_SPEED ) ); }
+        Speed downloadSpeed( ) const { return Speed::fromBps( getSize( DOWNLOAD_SPEED ) ); }
+        Speed uploadSpeed( ) const { return Speed::fromBps( getSize( UPLOAD_SPEED ) ); }
         double getVerifyProgress( ) const { return getDouble( PERCENT_VERIFIED ); }
         bool hasFileSubstring( const QString& substr ) const;
         bool hasTrackerSubstring( const QString& substr ) const;
-        Speed uploadLimit( ) const { return Speed::fromKbps( getInt( UP_LIMIT ) ); }
-        Speed downloadLimit( ) const { return Speed::fromKbps( getInt( DOWN_LIMIT ) ); }
+        Speed uploadLimit( ) const { return Speed::fromKBps( getInt( UP_LIMIT ) ); }
+        Speed downloadLimit( ) const { return Speed::fromKBps( getInt( DOWN_LIMIT ) ); }
         bool uploadIsLimited( ) const { return getBool( UP_LIMITED ); }
         bool downloadIsLimited( ) const { return getBool( DOWN_LIMITED ); }
         bool honorsSessionLimits( ) const { return getBool( HONORS_SESSION_LIMITS ); }
         int peerLimit( ) const { return getInt( PEER_LIMIT ); }
         double seedRatioLimit( ) const { return getDouble( SEED_RATIO_LIMIT ); }
         tr_ratiolimit seedRatioMode( ) const { return (tr_ratiolimit) getInt( SEED_RATIO_MODE ); }
-        int seeders() const { return getInt( SEEDERS ); }
-        int leechers() const { return getInt( LEECHERS ); }
-        int timesCompleted() const { return getInt( TIMES_COMPLETED ); }
+        int seedIdleLimit( ) const { return getInt( SEED_IDLE_LIMIT ); }
+        tr_idlelimit seedIdleMode( ) const { return (tr_idlelimit) getInt( SEED_IDLE_MODE ); }
+        TrackerStatsList trackerStats( ) const{ return myValues[TRACKERSTATS].value<TrackerStatsList>(); }
+        QStringList trackers() const { return myValues[TRACKERS].value<QStringList>(); }
         PeerList peers( ) const{ return myValues[PEERS].value<PeerList>(); }
         const FileList& files( ) const { return myFiles; }
 
     public:
         QString activityString( ) const;
         tr_torrent_activity getActivity( ) const { return (tr_torrent_activity) getInt( ACTIVITY ); }
+        bool isFinished( ) const { return getBool( IS_FINISHED ); }
         bool isPaused( ) const { return getActivity( ) == TR_STATUS_STOPPED; }
+        bool isWaitingToVerify( ) const { return getActivity( ) == TR_STATUS_CHECK_WAIT; }
         bool isVerifying( ) const { return getActivity( ) == TR_STATUS_CHECK; }
         bool isDownloading( ) const { return getActivity( ) == TR_STATUS_DOWNLOAD; }
+        bool isSeeding( ) const { return getActivity( ) == TR_STATUS_SEED; }
         bool isReadyToTransfer( ) const { return getActivity()==TR_STATUS_DOWNLOAD || getActivity()==TR_STATUS_SEED; }
         void notifyComplete( ) const;
 
     public:
         void update( tr_benc * dict );
+        void setMagnet( bool magnet ) { magnetTorrent = magnet; }
 
     private:
         const char * getMimeTypeString( ) const;
